// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import LocalAuthentication

public extension LAContext {
    /// Supported types for authentication
    enum AuthenticationType: String {
        /// None
        case none

        /// TouchID, i.e. using the fingerprint scanner
        case touchID

        /// FaceID, i.e. using the front camera and face recognition
        case faceID
        
        /// PinCode i.e. code for the users phone
        case pinCode

        /// Returns the brand name of the type, e.g. "TouchID" or "FaceID"
        public func name() -> String {
            let title: String

            switch self {
            case .touchID:
                title = "touchID"
            case .faceID:
                title = "faceID"
            case .pinCode:
                title = "pinkode"
            case .none:
                title = "bioID" //Only showed in SwiftUI previews or for devices that doesn't support any kind of Bio.
            }
            return title
        }
    }

    /// Returns the authentication biometric type of the device
    static var authenticationType: AuthenticationType {
        let context: LAContext = LAContext()
        
        let canDoBiometrics = context.canEvaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, error: nil)
        let haveAnyAuthentication = context.canEvaluatePolicy(.deviceOwnerAuthentication, error: nil)
        
        if canDoBiometrics {
            if context.biometryType == .faceID {
                return .faceID
            }
            return .touchID
        }
        if haveAnyAuthentication {
            return .pinCode
        }
        return .none
    }
}
