// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Combine
import Foundation

extension Publisher {
    public func receiveOnMain() -> Publishers.ReceiveOn<Self, DispatchQueue> {
        return self.receive(on: DispatchQueue.main)
    }

    /// Zips conditionally based on `condition`.
    /// Uses `Just` with fallback value if `condition` is `false`
    public func zipIf<P>(
        _ condition: @autoclosure () -> Bool,
        _ constructPublisher: @autoclosure () -> P,
        fallbackOutput: P.Output) -> Publishers.Zip<Self, AnyPublisher<P.Output, P.Failure>> where P : Publisher, P.Failure == Self.Failure {
        let publisher = PublisherHelper.conditional(condition: condition(), constructPublisher: constructPublisher(), fallbackOutput: fallbackOutput)
        return zip(publisher)
    }
}


private final class PublisherHelper {

    /// Takes a bool (as autoclosure) which indicates whether the publisher should be constructed.
    /// If the `condition` returns `true` the publisher will be constructed, otherwise it will fallback on
    /// a `Just` publisher with `fallbackOutput`
    static func conditional<P>(
        condition: @autoclosure () -> Bool,
        constructPublisher: @autoclosure () -> P,
        fallbackOutput: P.Output) -> AnyPublisher<P.Output, P.Failure> where P : Publisher {
        if condition() {
            return constructPublisher().eraseToAnyPublisher()
        } else {
            return Just(fallbackOutput)
                .setFailureType(to: P.Failure.self)
                .eraseToAnyPublisher()
        }
    }
}
