// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import XCTest
@testable import AppleExtensions

class DateExtensionTests: XCTestCase {

    private var testDate: Date = Date(timeIntervalSince1970: 1503924052)

    override func setUp() {
        super.setUp()

        testDate = Date(timeIntervalSince1970: 1503924052)
    }

    func testDateByAddingSeconds() {
        let date = testDate.date(byAdding: .second, value: 2)

        XCTAssertEqual(date?.timeIntervalSince1970, testDate.timeIntervalSince1970 + 2)
    }

    func testIsAfter() {
        let date = Date()
        let dateAfter = date.addingTimeInterval(1)
        let dateBefore = date.addingTimeInterval(-1)
        XCTAssertTrue(date.isAfterOrEqual(dateBefore))
        XCTAssertTrue(date.isAfterOrEqual(date))
        XCTAssertFalse(date.isAfterOrEqual(dateAfter))
    }

    func testIsAfterNow() {

        let date1 = Date().addingTimeInterval(100)
        XCTAssertTrue(date1.isAfterOrEqualToNow())

        let date2 = Date().addingTimeInterval(-1)
        XCTAssertFalse(date2.isAfterOrEqualToNow())
    }

    func testIsBefore() {
        let date = Date()
        let dateAfter = date.addingTimeInterval(1)
        let dateBefore = date.addingTimeInterval(-1)

        XCTAssertTrue(date.isBefore(dateAfter))
        XCTAssertFalse(date.isBefore(date))
        XCTAssertFalse(date.isBefore(dateBefore))
    }

    func testIsBetween() {
        let date = Date()
        let dateAfter = date.addingTimeInterval(1)
        let dateBefore = date.addingTimeInterval(-1)
        XCTAssertTrue(date.isBetween(date1: dateBefore, date2: dateAfter))
        XCTAssertFalse(date.isBetween(date1: dateBefore, date2: dateBefore))
        XCTAssertFalse(date.isBetween(date1: dateAfter, date2: dateAfter))
        XCTAssertFalse(date.isBetween(date1: dateAfter, date2: dateBefore))
        
        XCTAssertFalse(date.isBetween(date1: date, date2: date))

        XCTAssertTrue(date.isBetween(date1: dateBefore, date2: nil))
        XCTAssertFalse(date.isBetween(date1: dateAfter, date2: nil))
        XCTAssertTrue(date.isBetween(date1: nil, date2: dateAfter))
        XCTAssertFalse(date.isBetween(date1: nil, date2: dateBefore))
        XCTAssertTrue(date.isBetween(date1: nil, date2: nil))
    }

    func testWeekNumberAfterBirthDate() {
        let birthTomorrow = Date().date(byAdding: .minute, value: (24*60) + 1)!
        let zeroWeek = birthTomorrow.weekNumberAfterBirthDate()
        XCTAssertEqual(zeroWeek, 0)

        let birthToday = Date()
        let firstWeek = birthToday.weekNumberAfterBirthDate()
        XCTAssertEqual(firstWeek, 1)

        let birthSixDaysAgo = Date().date(byAdding: .day, value: -6)!
        let stillFirstWeek = birthSixDaysAgo.weekNumberAfterBirthDate()
        XCTAssertEqual(stillFirstWeek, 1)

        let sevenDaysAgo = Date().date(byAdding: .day, value: -7)!
        let secondWeek = sevenDaysAgo.weekNumberAfterBirthDate()
        XCTAssertEqual(secondWeek, 2)

        let tenDaysAgo = Date().date(byAdding: .day, value: -10)!
        let stillSecondWeek = tenDaysAgo.weekNumberAfterBirthDate()
        XCTAssertEqual(stillSecondWeek, 2)

        let fourteenDaysAgo = Date().date(byAdding: .day, value: -14)!
        let thirdWeek = fourteenDaysAgo.weekNumberAfterBirthDate()
        XCTAssertEqual(thirdWeek, 3)
    }

    func testYear2020() throws {
        let date = DateComponents(calendar: .current, year: 2020, month: 12, day: 24).date

        XCTAssertEqual(2020, date?.year)
    }

    func testMonth12() throws {
        let date = DateComponents(calendar: .current, year: 2020, month: 12, day: 24).date

        XCTAssertEqual(12, date?.month)
    }
}
