// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
import Combine
@testable import AppleExtensions

final class PublisherTests: XCTestCase {

    private var cancelBag = Set<AnyCancellable>()

    override func setUp() {
        super.setUp()
        // Put setup code here. This method is called before the invocation of each test method in the class.
    }

    override func tearDown() {
        cancelBag.removeAll()
        super.tearDown()
    }

    func testIsOnMain() {
        var cancelBag = Set<AnyCancellable>()

        PassthroughSubject<Void, Never>()
            .receive(on: DispatchQueue.global(qos: .background))
            .receiveOnMain()
            .sink(receiveValue: { _ in
                XCTAssertTrue(Thread.isMainThread)
            })
            .store(in: &cancelBag)
    }

    func testZipIf() {
        let expectationFalseCondition = XCTestExpectation(description: "False expectation never returned")
        let expectationTrueFailCondition = XCTestExpectation(description: "True failing expectation never returned")
        let expectationTrueCondition = XCTestExpectation(description: "True expectation never returned")

        let publisher = Just("🥳").setFailureType(to: Error.self)
        publisher.zipIf(false, ForceFailPublisher(), fallbackOutput: "🥤")
            .sink(receiveCompletion: { (completion) in
                switch completion {
                case .failure:
                    XCTFail("Shouldn't fail")
                case .finished:
                    expectationFalseCondition.fulfill()
                }
            }, receiveValue: { (result) in
                XCTAssertEqual(result.0, "🥳")
                XCTAssertEqual(result.1, "🥤")
            })
            .store(in: &cancelBag)

        publisher.zipIf(true, Fail(error: TestError.fail), fallbackOutput: "🥤")
            .sink(receiveCompletion: { (completion) in
                switch completion {
                case .failure:
                    expectationTrueFailCondition.fulfill()
                case .finished:
                    XCTFail("This should have failed!")
                }
            }, receiveValue: { (result) in
                XCTFail("No value - should have failed!")
            })
            .store(in: &cancelBag)


        publisher.zipIf(true, Just("🍾").setFailureType(to: Error.self), fallbackOutput: "🥤")
            .sink(receiveCompletion: { (completion) in
                switch completion {
                case .failure:
                    XCTFail("Shouldn't fail")
                case .finished:
                    expectationTrueCondition.fulfill()
                }
            }, receiveValue: { (result) in
                XCTAssertEqual(result.0, "🥳")
                XCTAssertEqual(result.1, "🍾")
            })
            .store(in: &cancelBag)

        wait(for: [expectationFalseCondition, expectationTrueCondition], timeout: 1.0)
    }
}

@available(iOS 13, *)
private struct ForceFailPublisher: Publisher {

    typealias Output = String
    typealias Failure = Error

    init() {
        failTest()
    }

    func receive<S>(subscriber: S) where S : Subscriber, Self.Failure == S.Failure, Self.Output == S.Input {
        failTest()
    }

    private func failTest() {
        XCTFail("This publisher should never be constructed!")
    }
}

enum TestError : Error {
    case fail
}
