// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation

public protocol HTTPRequest {
    associatedtype ResponseType: Response

    /// The base URL for the request.
    var baseUrl: String { get }

    /// The HTTP request method. Default is `GET`.
    var method: HTTPMethod { get }

    /// Path components that should be added to the base URL.
    var pathComponents: [String] { get }

    /// An dictionary of query items for the URL.
    var query: [String: String]? { get }

    /// The data sent as the message body of a request, such as for an HTTP POST request.
    var body: Data? { get }

    /// Adds values to the header fields.
    var headers: [String: String] { get }
}

public extension HTTPRequest {
    var method: HTTPMethod { .get }
    var query: [String: String]? { nil }
    var body: Data? { nil }
    var headers: [String: String] { [:] }
    var pathComponents: [String] { [] }
}

extension HTTPRequest {
    var url: URL {
        var baseUrl = URL(string: self.baseUrl)!

        pathComponents.forEach { pathComponent in
            baseUrl = baseUrl.appendingPathComponent(pathComponent)
        }

        return baseUrl
    }
}

public enum HTTPMethod: String {
    case get
    case post
    case put
    case patch
    case delete
}
