// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
@testable import HTTPNetworking

final class RequestBuilderTests: XCTestCase {
    let baseUrl = "http://example.com"

    func testRequestBuilderWithQueryAndNoPathComponent() {
        let request = TestRequest(
            baseUrl: baseUrl,
            pathComponents: [],
            method: .get,
            query: ["bar": "baz"],
            headers: [:]
        )

        verifyRequest(request)
    }

    func testRequestBuilderWithQueryAndPathComponent() {
        let request = TestRequest(
            baseUrl: baseUrl,
            pathComponents: ["foo"],
            method: .get,
            query: ["bar": "baz"],
            headers: [:]
        )

        verifyRequest(request)
    }

    func testRequestBuilderWithBody() throws {
        let jsonEncoder = JSONEncoder()
        let data = try jsonEncoder.encode(UUID().uuidString)

        let request = TestRequest(
            baseUrl: baseUrl,
            pathComponents: ["foo"],
            method: .post,
            query: ["bar": "baz"],
            body: data,
            headers: [:]
        )

        verifyRequest(request)
    }

    func testRequestBuilderWithHeaders() throws {
        let jsonEncoder = JSONEncoder()
        let data = try jsonEncoder.encode(UUID().uuidString)
        
        let request = TestRequest(
            baseUrl: baseUrl,
            pathComponents: ["foo"],
            method: .post,
            query: ["bar": "baz"],
            body: data,
            headers: ["foo": "test"]
        )
        
        verifyRequest(request)
    }

    private func verifyRequest(_ request: TestRequest) {
        var requests = [URLRequest]()
        let mockSession = NetworkSessionMock()
        _ = mockSession.dataTaskPublisher(for: request)
        XCTAssertNotNil(mockSession.request)
        requests.append(mockSession.request)

        if #available(iOS 13.0, *) {
            _ = mockSession.dataTaskPublisher(for: request)
            XCTAssertNotNil(mockSession.request)
            requests.append(mockSession.request)
        }

        for urlRequest in requests {
            let path = ([baseUrl] + request.pathComponents).joined(separator: "/")
            let query = request.query?.map { $0 + "=" + $1 }.joined(separator: "&")

            let endUrl = [path, query].compactMap { $0 }.joined(separator: "?")

            XCTAssertEqual(endUrl, urlRequest.url?.absoluteString)
            XCTAssertEqual(request.method.rawValue.uppercased(), urlRequest.httpMethod)
            XCTAssertEqual(request.body, urlRequest.httpBody)
            XCTAssertEqual(request.headers, urlRequest.allHTTPHeaderFields)
        }
    }
}
