// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import TIM
import TIMEncryptedStorage
import DependencyInjection
import Logger
import UIKit
import Combine
import ConfigurationServices
import LocalAuthentication
import HTTPNetworking
import SharedServices
import UserSettingsServices
import MaternityServices
import CMSServices

// TODO MOVE!
struct KeychainIdentifiers {
    let service: String
    let keyTag: Data

    init(suffix: String) {
        let bundleID = Bundle.main.bundleIdentifier ?? "unknown.bundle"
        self.service = "\(bundleID).refreshToken.\(suffix)"
        self.keyTag = Data("\(bundleID).encryptionKey.\(suffix)".utf8)
    }
}


enum BuildConfiguration: String {
    case debug = "Debug"
    case internalTest = "InternalTest"
    case release = "Release"

    /// This is not a BuildConfiguration we can build as, but a mode we can force the app into, when needed
    /// The App Store reviewer can long press an element on the first screen of the app, and enter this mode to avoid NemID login.
    case appStoreReview = "AppStoreReview"
    
    var isFirebaseDistribution: Bool {
        switch self {
        case .debug, .release:
            return false
        case .internalTest, .appStoreReview:
            return true
        }
    }

    /// E.g. ErrorIDs from the App Facade API
    var canShowDebugValues: Bool {
        switch self {
        case .debug, .internalTest:
            return true
        case .release:
            return true // This is currently enabled for the pilot release. We will determine whether this is OK for the final release version too.
        case .appStoreReview:
            return false
        }
    }

    /// E.g. stack trace info from the backend
    var canShowDebugDescriptions: Bool {
        switch self {
        case .debug, .internalTest:
            return true
        case .release, .appStoreReview:
            return false
        }
    }
    
    var keychainIdentifiers: KeychainIdentifiers {
        switch self {
        case .debug:
            return KeychainIdentifiers(suffix: "debug")
        case .internalTest:
            return KeychainIdentifiers(suffix: "test")
        case .release:
            return KeychainIdentifiers(suffix: "release")
        case .appStoreReview:
            return KeychainIdentifiers(suffix: "asr")
        }
    }


}

struct Configuration {
    let useMock: Bool
    let loggerConfig: OSLoggerConfig
    let tim: TIMConfiguration
}

final class App {
    private(set) static var buildConfiguration = BuildConfiguration(rawValue: Bundle.main.buildConfiguration) ?? .debug
    private(set) static var configuration = Configuration.build(for: buildConfiguration)
    static let isTesting: Bool = isUnitTesting || isUITesting
    static let isUnitTesting: Bool = ProcessInfo.processInfo.environment["IS_TESTING"] == "1"
    static let isUITesting: Bool = ProcessInfo.processInfo.environment["IS_UI_TESTING"] == "1"
    
    static let preferredLandscapeOrientation: UIInterfaceOrientation = .landscapeRight
    
    private static let multiLogger = MultiLogger(loggers: [])
    
    /// Used to present OpenID Connect login view
    static var topViewController: UIViewController? {
        UIApplication.topViewController()
    }
    
    static func buildDependencies() {
        setupLoggers()
        setupTriforkIdentityManager()
        setupMockFailRisk()
        
        addUtilityDependencies()
        addTriforkIdentityManagementDependencies()
        addUserDependencies()
        addBackendForFrontendConfigurations()
        addAppleDependencies()
        addCMSDependencies()
        addMaternityDependencies()
        addConfigurationDependencies()
        addRepositories()
        addLocalStorage()
        
        addSpecialViews()

    }

    static func enterAppStoreReviewModeAndRebuildAllDependencies() {
        multiLogger.debug("Entering App Store Review build configuration mode!")
        Self.buildConfiguration = .appStoreReview
        configuration = Configuration.build(for: Self.buildConfiguration)
        Resolver.shared.allowOverride = true
        buildDependencies()
    }
    
    private static func addTriforkIdentityManagementDependencies() {
        // We do not mock these services, since we actually wants the UI tests to test a full register and login flow via TIM.
        Resolver.shared.registerAsSingleton(TIMDataStorage.self, TIM.storage)
        Resolver.shared.registerAsSingleton(TIMAuth.self, TIM.auth)
        Resolver.shared.registerAsSingleton(TokenService.self, TokenServiceImpl())
    }
    
    private static func addAppleDependencies() {
        Resolver.shared.register(LocalAuthenticationContext.self, { LAContext() })
        Resolver.shared.registerAsSingleton(NetworkSession.self, URLSession.shared)
    }
    
    private static func addCMSDependencies() {
        Resolver.shared.registerAsSingleton(CMSApiConfiguration.self, CMSConfiguration())
        
        if configuration.useMock {
            Resolver.shared.registerAsSingleton(DictionaryService.self, DictionaryServiceMock())
            Resolver.shared.registerAsSingleton(ChecklistContentService.self, ChecklistContentServiceMock())
            Resolver.shared.registerAsSingleton(PagesService.self, PagesServiceMock())
            Resolver.shared.registerAsSingleton(FAQService.self, FAQServiceMock())
            Resolver.shared.registerAsSingleton(ArticlesService.self, ArticlesServiceMock())
        } else {
            Resolver.shared.registerAsSingleton(DictionaryService.self, DictionaryServiceImpl())
            Resolver.shared.registerAsSingleton(ChecklistContentService.self, ChecklistContentServiceImpl())
            Resolver.shared.registerAsSingleton(PagesService.self, PagesServiceImpl())
            Resolver.shared.registerAsSingleton(FAQService.self, FAQServiceImpl())
            Resolver.shared.registerAsSingleton(ArticlesService.self, ArticlesServiceImpl())
        }
    }
    
    private static func addBackendForFrontendConfigurations() {
        let apiConfiguration = ApiConfiguration()
        Resolver.shared.registerAsSingleton(ConfigurationApiConfiguration.self, apiConfiguration)
    }
    
    private static func addMaternityDependencies() {
        if configuration.useMock {
            Resolver.shared.registerAsSingleton(CarePlanService.self, CarePlanServiceMock())
        } else {
            Resolver.shared.registerAsSingleton(CarePlanService.self, CarePlanServiceImpl())

        }
    }
    
    private static func addConfigurationDependencies() {
        if configuration.useMock {
            Resolver.shared.registerAsSingleton(ConfigurationService.self, ConfigurationServiceMock())
        } else {
            Resolver.shared.registerAsSingleton(ConfigurationService.self, ConfigurationServiceImpl())
        }
    }
    
    private static func addUserDependencies() {
        if configuration.useMock {
            Resolver.shared.registerAsSingleton(UserSettingsService.self, UserSettingsServiceMock(isPersistant: !isTesting))
        } else {
            Resolver.shared.registerAsSingleton(UserSettingsService.self, UserSettingsServiceImpl())
        }
        Resolver.shared.registerAsSingleton(UserContextProtocol.self, UserContext())
    }
    
    private static func addUtilityDependencies() {
        Resolver.shared.registerAsSingleton(Logger.self, multiLogger)
        
        if configuration.useMock {
            Resolver.shared.registerAsSingleton(SystemInformationService.self, SystemInformationServiceMock())
        } else {
            Resolver.shared.registerAsSingleton(SystemInformationService.self, SystemInformationServiceImpl())
        }
        Resolver.shared.registerAsSingleton(ChecklistRepository.self, ChecklistRepositoryImpl())
    }
    
    private static func addRepositories() {
        Resolver.shared.registerAsSingleton(MaternityRepository.self, MaternityRepositoryImpl.shared)
    }
    
    private static func addLocalStorage() {
        Resolver.shared.registerAsSingleton(MaternityStorageService.self, MaternityStorageServiceImpl.shared)
        Resolver.shared.registerAsSingleton(ChecklistStorageService.self, ChecklistStorageServiceImpl.shared)
    }
    
    private static func setupLoggers() {
        var loggers: [LoggerProtocol] = [OSLogger(config: configuration.loggerConfig)]
        if buildConfiguration == .debug {
            HTTPNetworkingConfig.isDebugPrintingEnabled = false // Can be enabled when debugging API requests
            loggers.append(ErrorFlashLogger())
        }
        multiLogger.loggers = loggers
    }
    
    static func setupMockFailRisk() {
        if buildConfiguration == .debug && !isTesting {
            MockFlags.failRisk = .small
        } else {
            MockFlags.failRisk = .disabled
        }
    }
    
    private static func addSpecialViews() {
        // The activity monitor view is not added in debug, to support view debugging (crashes when added)
        // This means that you will never see a login timeout during debugging
        // The timeout is set for one hour when debugging to avoid this.
        if buildConfiguration == .debug {
            Resolver.shared.registerAsSingleton(ActivityMonitor.self, DebugActivityMonitorWindow())
        } else {
            Resolver.shared.registerAsSingleton(ActivityMonitor.self, ActivityMonitorWindow())
        }
    }
    
    private static func setupTriforkIdentityManager() {
       // let allowReconfigure = buildConfiguration == .appStoreReview
       // TIM.configure(configuration: configuration.tim, allowReconfigure: allowReconfigure)

        let dataStorage = OurTimDataStorage()
        
        let auth = TIMAuthDefault(
            dataStorage: dataStorage,
            openIdController: AppAuthController(
                App.configuration.tim.oidcConfiguration,
                customOIDExternalUserAgent: nil // TODO IS THIS RIGHT????
            ),
            backgroundMonitor: TIMAppBackgroundMonitorDefault()
        )
        
        TIM.configure(
            dataStorage: dataStorage,
            auth: auth,
            customLogger: nil
        )
    }
}

private extension Configuration {
    static func build(for buildConfiguration: BuildConfiguration) -> Configuration {
        let timConfiguration = TIMConfiguration.forBuildConfiguration(buildConfiguration)
        switch buildConfiguration {
        case .debug:
            return Configuration(
                useMock: App.isUITesting,
                loggerConfig: OSLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .debug,
                    isDevelopmentInfoEnabled: false,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .internalTest:
            return Configuration(
                useMock: false,
                loggerConfig: OSLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .info,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .release:
            return Configuration(
                useMock: false,
                loggerConfig: OSLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .error,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .appStoreReview:
            return Configuration(
                useMock: true,
                loggerConfig: OSLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .warning,
                    isDevelopmentInfoEnabled: false,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        }
    }
}
