// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI
import TIM
import CMSServices
import UserSettingsServices
import LocalAuthentication
import Combine

final class SceneDelegate: UIResponder, UIWindowSceneDelegate {
    
    // MARK: - Dependencies
    @DI private var logger: Logger
    @DI private var timAuth: TIMAuth
    @DI private var activityMonitor: ActivityMonitor
    @DI private var timDataStorage: TIMDataStorage
    @DI private var userContext: UserContextProtocol
    @DI private var maternityStore: MaternityRepository
    
    // MARK: - Properties
    var window: UIWindow?
    private var cancelBag = CancelBag()
    
    // MARK: - Scene Lifecycle Methods
    func scene(_ scene: UIScene, willConnectTo session: UISceneSession, options connectionOptions: UIScene.ConnectionOptions) {
        guard let windowScene = scene as? UIWindowScene, !App.isUnitTesting else { return }
        
        setupInitialConfiguration(windowScene: windowScene)
        handleAppInitialState()
    }
    
    // MARK: - Initial Setup
    private func setupInitialConfiguration(windowScene: UIWindowScene) {
        UIColor.swizzleSystemBackground()
        
        if App.isUITesting {
            clearAllUserDataForUITests()
            UIView.setAnimationsEnabled(false)
        }
        
        let window = UIWindow(windowScene: windowScene)
        self.window = window
        window.makeKeyAndVisible()
        
        listenForNotifications()
    }
    
    private func handleAppInitialState() {
        // Prioritize authentication flow based on user's current state
        switch (userContext.hasHandledOnboardingFlow, userContext.hasUserRegistered) {
        case (false, _):
            // Incomplete onboarding - reset to onboarding
            resetToOnboarding()
        case (true, true):
            // Registered user - attempt TIM authentication
            handleTimAuthentication()
        case (true, false):
            // Non-registered user - handle local authentication
            handleLocalAuthentication()
        }
    }
    
    // MARK: - Authentication Handlers
    private func handleTimAuthentication() {
        changeRootViewController(UIHostingController(rootView: SessionLoadingView()), animated: false)
        
        timAuth.accessToken()
            .sink(receiveCompletion: { [weak self] completion in
                switch completion {
                case .failure(let error):
                    self?.handleAuthenticationFailure(error)
                case .finished:
                    break
                }
            }, receiveValue: { [weak self] accessToken in
                self?.logger.debug("Authenticated UserId: \(accessToken.userId)")
                self?.handleNotification(.showTabBarController, animated: false)
            })
            .store(in: &cancelBag)
        
        enableBackgroundMonitoringForTIM()
    }
    
    private func handleLocalAuthentication() {
        if userContext.hasEnabledLocalAuthentication {
            presentLocalAuthenticationFlow()
        } else {
            refreshDataAndPresentTabBarController()
        }
    }
    
    private func presentLocalAuthenticationFlow() {
        let storyboard = UIStoryboard(name: R.storyboard.launchScreen.name, bundle: nil)
        let launchScreen = storyboard.instantiateViewController(withIdentifier: "launchScreen")
        changeRootViewController(launchScreen, animated: false)
        
        authenticateLocally()
    }
    
    private func authenticateLocally() {
        let context = LAContext()
        guard context.canEvaluatePolicy(.deviceOwnerAuthentication, error: nil) else {
            resetToOnboarding()
            return
        }
        
        context.evaluatePolicy(.deviceOwnerAuthentication, localizedReason: Localizable.onboarding_login_touch()) { [weak self] success, _ in
            DispatchQueue.main.async {
                if success {
                    self?.userContext.setHasHandledOnboardingFlow(true)
                    self?.userContext.setIsAuthenticated(true)
                    self?.refreshDataAndPresentTabBarController()
                } else {
                    self?.presentLocalAuthenticationFlow()
                }
            }
        }
    }
    
    private func handleAuthenticationFailure(_ error: Error) {
        logger.info("Authentication failed: \(error.localizedDescription)")
        
        if let userId = timDataStorage.availableUserIds.first {
            handleNotification(.showLogin(userId: userId), animated: false)
        } else {
            resetToOnboarding()
        }
    }
    
    // MARK: - Utility Methods
    private func enableBackgroundMonitoringForTIM() {
        timAuth.enableBackgroundTimeout(durationSeconds: 300)
            .sink(receiveValue: { [weak self] _ in
                if let userId = self?.timDataStorage.availableUserIds.first {
                    self?.handleNotification(.showLogin(userId: userId), animated: true)
                } else {
                    self?.resetToOnboarding()
                }
            })
            .store(in: &cancelBag)
    }
    
    private func refreshDataAndPresentTabBarController() {
        maternityStore.requestUpdate()
            .sink(receiveCompletion: { [weak self] _ in
                self?.handleNotification(.showTabBarController, animated: false)
            }, receiveValue: { _ in })
            .store(in: &cancelBag)
    }
    
    // MARK: - Scene Delegate Methods
    func sceneDidBecomeActive(_ scene: UIScene) {
        SecurityCoverView.shared.detachFromWindow()
        
        guard !App.isTesting && timAuth.isLoggedIn else { return }
        activityMonitor.startTracking()
    }
    
    func sceneWillResignActive(_ scene: UIScene) {
        guard !App.isTesting else { return }
        
        activityMonitor.stopTracking()
        UserDefaults.standard.set(Date(), forKey: "LastOpened")
    }
    
    func sceneDidEnterBackground(_ scene: UIScene) {
        guard !App.isTesting else { return }
        SecurityCoverView.shared.attachToWindow()
    }
    
    func scene(_ scene: UIScene, openURLContexts URLContexts: Set<UIOpenURLContext>) {
        for url: URL in URLContexts.map(\.url) {
            timAuth.handleRedirect(url: url)
        }
    }
    
    // MARK: - Notification Handling
    private func listenForNotifications() {
        NotificationCenter.default
            .listen(for: Notification.Name.SceneDelegate.changeRootViewController)
            .sink(receiveValue: { [weak self] (message: NotificationMessage) in
                self?.handleNotification(message, animated: true)
            })
            .store(in: &cancelBag)
    }
    
    // MARK: - Root View Controller Management
    private func changeRootViewController(_ rootViewController: UIViewController, animated: Bool) {
        guard let window = window else {
            logger.critical("Failed to change rootViewController, window is nil")
            return
        }
        
        rootViewController.view.backgroundColor = ColorManager.bgNeutralPrimary.currentUIColor()
        window.rootViewController = rootViewController
        window.makeKeyAndVisible()
        
        if animated {
            UIView.transition(
                with: window,
                duration: 0.2,
                options: .transitionCrossDissolve,
                animations: {},
                completion: nil
            )
        }
    }
    
    // MARK: - Testing and Reset Methods
    private func clearAllUserDataForUITests() {
        timAuth.logout()
        timDataStorage.clearAllUsers()
        userContext.setHasHandledOnboardingFlow(false)
    }
    
    private func resetToOnboarding() {
        let onboarding = UIHostingController(rootView: OnboardingView())
        changeRootViewController(onboarding, animated: false)
    }
}

// MARK: - Notification Message Handling Extension
extension SceneDelegate {
    enum NotificationMessage: NotificationCenterValue {
        case beginRegisterFlow(userId: String)
        case showTabBarController
        case resetToOnboarding
        case showLogin(userId: String)
        case logout(userId: String)
        case showLoginWithInactivityAlert(userId: String)
    }
    
    private func handleNotification(_ notification: NotificationMessage, animated: Bool) {
        switch notification {
        case .beginRegisterFlow(let userId):
            let choosePin = UIHostingController(rootView: ChoosePinCodeView(userId: userId))
            changeRootViewController(choosePin, animated: animated)
            
        case .showTabBarController:
            changeRootViewController(TabBarController(), animated: animated)
            enableActivityMonitor()
            
        case .resetToOnboarding:
            let onboarding = UIHostingController(rootView: OnboardingView())
            changeRootViewController(onboarding, animated: animated)
            
        case .showLogin(let userId):
            showLogin(userId: userId, logoutTrigger: false)
            
        case .logout(let userId):
            showLogin(userId: userId, logoutTrigger: true, showInactivityAlert: false)
            
        case .showLoginWithInactivityAlert(let userId):
            showLogin(userId: userId, logoutTrigger: true, showInactivityAlert: true)
        }
    }
    
    private func showLogin(userId: String, logoutTrigger: Bool, showInactivityAlert: Bool = false) {
        timAuth.logout()
        activityMonitor.stopTracking()
        let loginView = UIHostingController(
            rootView: LoginView(
                userId: userId,
                presentedFromLogout: logoutTrigger
            )
        )
        changeRootViewController(loginView, animated: true)
    }
    
    private func enableActivityMonitor() {
        activityMonitor.setupForRootViewControllerChanges()
        activityMonitor.startTracking()
    }
}
