// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import CMSData
import CMSServices
import Combine
import ConfigurationServices
import ErrorData
import GMAppData

final class ChecklistRepositoryImpl: ChecklistRepository {
    @DI private var checklistContentService: ChecklistContentService
    @DI private var checklistStorageService: ChecklistStorageService

    func fetchChecklists() -> AnyPublisher<[Checklist], ServiceError> {
        if checklistStorageService.isEmpty() {
            let fromCms = checklistFromCMS()
            return saveToLocal(cms: fromCms)
        }
        return checklistFromLocal()
    }
    
    func checklistFromCMS() -> AnyPublisher<[Checklist], ServiceError> {
        checklistContentService
            .fetchChecklistContent()
            .map({ (content: [ChecklistContent]) -> [Checklist] in
                content.map { (content: ChecklistContent) -> Checklist in
                    Checklist.create(from: content)
                }
            })
            .eraseToAnyPublisher()
    }
    
    func checklistFromLocal() -> AnyPublisher<[Checklist], ServiceError> {
        checklistStorageService
            .fetchChecklists()
            .publisher.map { (storageModels: [ChecklistStorageModel]) in
                storageModels.map { (content: ChecklistStorageModel) -> Checklist in
                    Checklist.create(from: content)
                }
            }
            .eraseToAnyPublisher()
    }
    
    func saveToLocal(cms: AnyPublisher<[Checklist], ServiceError>) -> AnyPublisher<[Checklist], ServiceError> {
        cms.map({ [weak self] (result: [Checklist]) in
                self?.checklistStorageService.addChecklists(checklists: result)
                return result
            }
        )
        .mapToServiceError()
        .eraseToAnyPublisher()
    }
    
    func toggleChecklistItem(checklistId: Int, item: Checklist.Item) -> ServiceError? {
        checklistStorageService
            .toggleChecklistItem(checklistId: Int64(checklistId), item: item)

    }
    
    func deleteChecklistItem(checklistId: Int, item: Checklist.Item) -> ServiceError? {
        checklistStorageService
            .deleteChecklistItem(checklistId: Int64(checklistId), item: item)
    }
    
    func addChecklistItem(checklistId: Int, item: Checklist.Item) -> ServiceError? {
        checklistStorageService
            .addChecklistItem(checklistId: Int64(checklistId), item: item)
    }
    
    func updateChecklistItem(checklistId: Int, item: Checklist.Item) -> ServiceError? {
        checklistStorageService
            .updateChecklistItem(checklistId: Int64(checklistId), item: item)
    }
    
    func resetDB() -> ServiceError? {
        checklistStorageService
            .deleteAllData()
    }
    
    func clear() {
        resetDB()
    }
}
