// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import CMSData

struct Checklist: Identifiable, Equatable {

    let id: Int
    let title: String
    let description: String?
    var items: [Item]

    static func create(from content: ChecklistContent) -> Checklist {
        Checklist(
            id: content.id,
            title: content.title,
            description: content.text,
            items: content.items.map({ contentItem in
                Item.create(from: contentItem)
            })
        )
    }
    
    static func create(from content: ChecklistStorageModel) -> Checklist {
        Checklist(
            id: Int(content.id),
            title: content.title ?? "",
            description: content.text,
            items: content.wrappedItems.map({ element in
                Item.create(from: element)
            })
        )
    }

    struct Item: Identifiable, Equatable {
        let id: String
        var title: String
        var isChecked: Bool

        static func create(from content: ChecklistItemContent) -> Item {
            Item(
                id: content.id,
                title: content.title,
                isChecked: false
            )
        }
        
        static func create(from content: ChecklistItemStorageModel) -> Item {
            Item(
                id: content.id ?? "",
                title: content.title ?? "",
                isChecked: content.isChecked
            )
        }
    }
}

extension Checklist {
    static func == (lhs: Checklist, rhs: Checklist) -> Bool {
        lhs.id == rhs.id && lhs.description == rhs.description && lhs.title == rhs.title && lhs.items == rhs.items
    }
}

extension Checklist.Item {
    static func == (lhs: Checklist.Item, rhs: Checklist.Item) -> Bool {
        lhs.id == rhs.id && lhs.title == rhs.title && lhs.isChecked == rhs.isChecked
    }
}

extension ChecklistStorageModel {
    
    public var wrappedItems: [ChecklistItemStorageModel] {
        let set = items as? Set<ChecklistItemStorageModel> ?? []
        return set.sorted {
            $0.title ?? "" > $1.title ?? ""
        }
    }
}

extension Array where Element == Checklist {
    func customSorted() -> [Checklist] {
        let customOrder = ["Fødetasken", "Tøj til baby", "Det første udstyr"]
        
        return self.sorted { (first, second) -> Bool in
            guard let firstIndex = customOrder.firstIndex(of: first.title),
                  let secondIndex = customOrder.firstIndex(of: second.title) else {
                return false
            }
            return firstIndex < secondIndex
        }
    }
}
