// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import SwiftUI
import MaternityServices
import ConfigurationServices
import GMAppData
import UserSettingsServices
import ErrorData
import CoreData

final class MaternityRepositoryImpl: ObservableObject, MaternityRepository {
    @DI private var logger: Logger
    @DI private var maternityStorageService: MaternityStorageService

    @Published private(set) var selectedMaternity: SelectedMaternity?

    static let shared = MaternityRepositoryImpl()

    private init() { }

    func changeSelectedMaternity(_ maternity: SelectedMaternity) -> AnyPublisher<Void, Never> {
        if selectedMaternity == nil {
            return maternityStorageService.saveMaternity(maternity: maternity.toRepoModel())
                .receiveOnMain()
                .handleEvents(receiveOutput: { [weak self] (maternity: MaternityStorageModel) in
                    let repoModel = maternity.toRepoModel()
                    self?.selectedMaternity = repoModel.toSelectedMaternity()
                    NotificationCenter.default
                        .post(notification: .TabBarController.changeOfSelectedMaternity, value: "")
                })
                .mapToVoid()
                .replaceError(with: Void())
                .eraseToAnyPublisher()
        }
        return maternityStorageService.updateMaternity(maternity: maternity.toRepoModel())
            .receiveOnMain()
            .handleEvents(receiveOutput: { [weak self] (maternity: MaternityStorageModel?) in
                let repoModel = maternity?.toRepoModel()
                self?.selectedMaternity = repoModel?.toSelectedMaternity()
            })
            .mapToVoid()
            .replaceError(with: Void())
            .eraseToAnyPublisher()
    }
    
    func changeDateOfBirthForSelectedMaternity(_ date: Date?) -> AnyPublisher<Void, ServiceError> {
        guard let selectedMaternity = selectedMaternity else {
            return Just(Void())
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        let maternityy = selectedMaternity.toRepoModel(dateOfBirth: date)
        return maternityStorageService
            .updateMaternity(maternity: maternityy)
            .receiveOnMain()
            .handleEvents(receiveOutput: { [weak self] (maternity: MaternityStorageModel?) in
                let repoModel = maternity?.toRepoModel()
                self?.selectedMaternity = repoModel?.toSelectedMaternity()
            })
            .mapToVoid()
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func requestUpdate() -> AnyPublisher<Void, ServiceError> {
        maternityStorageService.fetchMaternity()
            .receiveOnMain()
            .handleEvents(receiveOutput: { (maternity: MaternityStorageModel?) in
                let repoModel = maternity?.toRepoModel()
                self.selectedMaternity = repoModel?.toSelectedMaternity()
            })
            .mapToVoid()
            .eraseToAnyPublisher()
    }

    func clear() {
        selectedMaternity = nil
        maternityStorageService.deleteMaternity()
    }
}
