// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import CMSData

extension CMSData.Category : Identifiable {
    var numberOfArticlesText: String {
        let itemsCount: Int = articles.count
        return itemsCount == 1 ?
                Localizable.articles_count_singular(itemsCount.description) :
                Localizable.articles_count_plural(itemsCount.description)
    }

    static func mockList() -> [CMSData.Category] {
        (1...3).map({ category in
            CMSData.Category(
                id: category,
                name: "Test \(category)",
                slug: "mock",
                description: String.loremIpsum(.small),
                iconUrl: URL(string: "https://cataas.com/cat")!,
                articles: (1...3).map({ article in
                    Article(
                        id: article,
                        title: "Test Article \(article)",
                        lastUpdated: Date(),
                        description: String.loremIpsum(.small),
                        content: String.loremIpsum(.small),
                        searchOrder: Bool.random() ? article : nil,
                        sections: (1...2).map({ section in
                            ArticleSection(
                                id: section,
                                title: "Test Section \(section)",
                                content: "<p><b>Hej</v> Hej hej hej</p>",
                                searchOrder: Bool.random() ? section : nil
                            )
                        })
                    )
                })
            )
        })
    }
}

extension Sequence where Element == CMSData.Category {
    
    func searchResults() -> [ArticleSearchResult] {
        var results: [ArticleSearchResult] = []
        
        for element in self {
            for article in element.articles {
                if let searchOrder = article.searchOrder {
                    results.append(
                        ArticleSearchResult(
                            title: article.title,
                            description: article.description,
                            subtitle: element.name,
                            iconUrl: element.iconUrl,
                            article: article,
                            searchOrder: searchOrder,
                            articleSectionId: nil
                        )
                    )
                }

                for section in article.sections {
                    if let searchOrder = section.searchOrder {
                        results.append(
                            ArticleSearchResult(
                                title: section.title,
                                description: section.content.strippingHTML().nilIfEmpty,
                                subtitle: "\(element.name) • \(article.title)",
                                iconUrl: element.iconUrl,
                                article: article,
                                searchOrder: searchOrder,
                                articleSectionId: section.id
                            )
                        )
                    }
                }
            }
        }
        let sortedResults = results.sorted(by: \.searchOrder)
        return sortedResults
    }
    
    var hasSearchResults: Bool { !searchResults().isEmpty }
    var numberOfSearchResults: Int { searchResults().count }
}
