// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData
import CMSData
import UIData

protocol ObservationTableDataSet {
    var fetusSources: [ObservationTableMeasurementSource]? { get }
    var motherSource: ObservationTableMeasurementSource? { get }
    func generateValueColumns(navigateToActivity: @escaping ((String) -> Void)) -> [[DataTableCell]]
}

extension ObservationTableDataSet {
    var sources: [ObservationTableMeasurementSource] {
        [
            (fetusSources ?? []),
            [ motherSource ].compactMap({ $0 })
        ].flatMap({ $0 })
    }

    var uniqueListOfSourceActivities: [ObservationTableActivity] {
        sources
            .flatMap({ $0.activities })
            .filterForUniqueActivities()
            .sorted(by: \.sortDate) { (date1: Date, date2: Date) -> Bool in
                date1 > date2
            }
    }

    func toDataTableColumnSet(dictionary: CMSDictionary, navigateToActivity: @escaping ((String) -> Void)) -> DataTableColumnSet {
        DataTableColumnSet(
            titleColumn: generateTitleColumn(dictionary: dictionary),
            valueColumns: generateValueColumns(navigateToActivity: navigateToActivity)
        )
    }

    private func generateTitleColumn(dictionary: CMSDictionary) -> [DataTableCell] {
        sources.map { (source) -> [DataTableCell] in
            titleCells(for: source, dictionary: dictionary)
        }.flatMap({ $0 })
    }

    func generateValueColumns<
        Fetus: ObservationTableMeasurementSource & ObservationTableMeasurementRowsSource,
        Mother: ObservationTableMeasurementSource & ObservationTableMeasurementRowsSource
    >(fetuses: [Fetus]?, mother: Mother?, navigateToActivity: @escaping ((String) -> Void)) -> [[DataTableCell]] {
        var valueColumns: [[DataTableCell]] = []

        // Iterate over all activity ID's to create columns for all activities
        for activity in uniqueListOfSourceActivities {
            var valueColumnForActivity: [DataTableCell] = []

            var i: Int = 0 // Only used to generate unique cell ids as fallback
            for fetus in fetuses ?? [] {
                let sectionId = (fetus.id ?? fetus.title) ?? "fetus_\(i)"
                let fetusRow = fetus.rows.rowForActivity(activity: activity)
                let fetusValues = Fetus.Row.MeasurementType.valueColumn(sectionId: sectionId, activity: activity, generator: fetusRow, navigateToActivity: navigateToActivity)
                valueColumnForActivity.append(contentsOf: fetusValues)
                i += 1
            }

            let motherRow = (mother ?? Mother.empty()).rows.rowForActivity(activity: activity)
            let motherValues = Mother.Row.MeasurementType.valueColumn(sectionId: "mother", activity: activity, generator: motherRow, navigateToActivity: navigateToActivity)
            valueColumnForActivity.append(contentsOf: motherValues)

            valueColumns.append(valueColumnForActivity)
        }
        return valueColumns
    }

    private func titleCells(for source: ObservationTableMeasurementSource, dictionary: CMSDictionary) -> [DataTableCell] {
        var titleColumn: [DataTableCell] = []
        let sectionId = (source.id ?? source.title) ?? UUID().uuidString
        titleColumn.append(
            DataTableCell(
                id: sectionId,
                value: source.title ?? Localizable.common_missing_data(),
                type: .boldText
            )
        )
        
        let measurementTitles = source.measurementTypes.map { (measurementType) in
            DataTableCell(
                id: "\(sectionId)_\(measurementType.rawValue)",
                value: measurementType.title,
                type: .potentialDictionaryCellType(title: measurementType.title, dictionary: dictionary, fallback: measurementType.defaultCellType)
            )
        }
        titleColumn.append(contentsOf: measurementTitles)
        return titleColumn
    }
}
