// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import SwiftUI

protocol ScreenTrackable {
    /// Override this to set custom name for screen tracking, otherwise the protocol extension will use the class name
    var screenTrackingName: String { get }
}

extension ScreenTrackable {
    var screenTrackingName: String {
        String(describing: type(of: self))
    }
}

extension View {
    @ViewBuilder
    func screenTracking(_ screenTrackingName: String, isEnabled: Bool = true) -> some View {
        self
            .modifier(ScreenTrackingViewModifier(screenTrackingName: screenTrackingName, isEnabled: isEnabled))
    }
}

private struct ScreenTrackingViewModifier: ViewModifier {
    @DI var logger: Logger
    let screenTrackingName: String
    var isEnabled: Bool = true

    /// Only used to log non fatal issues if the appearance events are spamming
    @State private var lastAppearDate: Date?

    func body(content: Content) -> some View {
        content
            .protectedAppearanceEvents(
                onAppear: {
                    guard isEnabled else {
                        return
                    }

                    // If the appear event happens more than once within 100 ms, then post a non-fatal issue! This should not be happening!
                    if let timeSinceLastAppear = lastAppearDate?.timeIntervalSinceNow, timeSinceLastAppear >= -0.1 {
                        logger.error("It looks like the appearance events are spamming very fast for \(screenTrackingName)!")
                    }
                    lastAppearDate = Date()
                    // Screen tracking started
                },
                onDisappear: {
                    guard isEnabled else {
                        return
                    }

                    // Screen tracking stopped
                }
            )
    }
}
