// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import CoreData
import Foundation
import GMAppData
import Combine
import ErrorData
import DependencyInjection
import Logger

class ChecklistStorageServiceImpl: ChecklistStorageService {
    @Inject private var logger: LoggerProtocol
    
    static let shared: ChecklistStorageService = ChecklistStorageServiceImpl()
    private let context: NSManagedObjectContext = LocalStorageController().container.viewContext
    
    func isEmpty () -> Bool {
        let data: [NSFetchRequestResult]? = try? context.fetch(ChecklistStorageModel.fetchRequest())
        return data?.isEmpty ?? true
    }
    
    func fetchChecklists() -> Result<[ChecklistStorageModel], ServiceError> {
        tryForResult {
            try context.fetch(ChecklistStorageModel.fetchRequest())
        }.mapError { (error: Error) in
            ServiceError.unexpectedError(error)
        }
    }
    
    func addChecklists(checklists: [Checklist]) -> ServiceError? {
        checklists.createIn(context: context)
        return saveContextOrError()
    }
    
    func updateChecklistItem(
        checklistId: Int64,
        item: Checklist.Item
    ) -> ServiceError? {
        updateListItem(checklistId: checklistId, item: item) { (foundItem: ChecklistItemStorageModel) in
            foundItem.updateWith(item)
        }
    }
    
    func addChecklistItem(
        checklistId: Int64,
        item: Checklist.Item
    ) -> ServiceError? {
        let existingCheckListResult: Result<ChecklistStorageModel?, ServiceError> = findChecklistBy(id: checklistId)
        var existingCheckList: ChecklistStorageModel?
        switch existingCheckListResult {
        case .failure(let error): return error
        case .success(let result): existingCheckList = result
        }
        
        if let existingCheckList = existingCheckList {
            let newItem = item.createIn(context: context)
            existingCheckList.addItem(newItem: newItem)
        } else {
            return ServiceError.unexpectedError(nil)
        }
        return saveContextOrError()
    }
    
    func deleteChecklistItem(checklistId: Int64, item: Checklist.Item) -> ServiceError? {
            updateListItem(checklistId: checklistId, item: item) { (model: ChecklistItemStorageModel) in
            context.delete(model)
        }
    }
    
    func toggleChecklistItem(checklistId: Int64, item: Checklist.Item) -> ServiceError? {
            updateListItem(checklistId: checklistId, item: item) { (model: ChecklistItemStorageModel) in
            model.isChecked = !model.isChecked
        }
    }
    
    private func updateListItem(
        checklistId: Int64,
        item: Checklist.Item,
        updateAction: (ChecklistItemStorageModel) -> Void
    ) -> ServiceError? {
        let foundItem: ChecklistItemStorageModel?
        switch findChecklistItemBy(checklistId: checklistId, itemId: item.id) {
        case .failure(let serviceError): return serviceError
        case .success(let found): foundItem = found
        }
        guard let foundItem = foundItem else {
            return nil
        }
        updateAction(foundItem)
        return saveContextOrError()
    }
    
    private func findChecklistItemBy(checklistId: Int64, itemId: String) -> Result<ChecklistItemStorageModel?, ServiceError> {
        let checklistResult: ChecklistStorageModel?
        let findListResult = findChecklistBy(id: checklistId)
        switch findListResult {
        case .failure(let serviceError): return Result.failure(serviceError)
        case .success(let success): checklistResult = success
        }
        
        return Result.success(checklistResult?.findItemBy(id: itemId))
    }
    
    private func findChecklistBy(id: Int64) -> Result<ChecklistStorageModel?, ServiceError> {
        let lists: [ChecklistStorageModel]
        let listsResult: Result<[ChecklistStorageModel], ServiceError> = fetchChecklists()
        switch listsResult {
        case .failure(let serviceError): return Result.failure(serviceError)
        case .success(let success): lists = success
        }
        return Result.success(
            lists.first { (list: ChecklistStorageModel) in
                list.id == id
            }
        )
    }
    
    private func saveContextOrError() -> ServiceError? {
        do {
            logger.debug("Saving checklist item on device...")
            try context.save()
            logger.debug("Checklist item was saved on device!")
            return nil
        } catch let error as NSError {
            logger.error("Failed to save checklist item on device! \(error.localizedDescription)")
            return ErrorData.ServiceError.unexpectedError(error)
        }
    }
    
    func deleteAllData() -> ServiceError? {
        do {
            let entites = try context.fetch(ChecklistStorageModel.fetchRequest())
            
            for entity in entites {
                context.delete(entity)
            }
            return saveContextOrError()

        } catch let error {
            return ServiceError.unexpectedError(error)
        }
    }
}

extension Array where Element == Checklist {
    func createIn(context: NSManagedObjectContext) {
        forEach { (item: Checklist) in
            let checklistStorageModel = ChecklistStorageModel(context: context)
            checklistStorageModel.id = Int64(item.id)
            checklistStorageModel.title = item.title
            checklistStorageModel.text = item.description
            checklistStorageModel.items = NSSet(
                array: item.items.map({ item in
                    item.createIn(context: context)
                })
            )
        }
    }
}

extension Checklist.Item {
    func createIn(context: NSManagedObjectContext) -> ChecklistItemStorageModel {
        let checklistItemStorageModel = ChecklistItemStorageModel(context: context)
        checklistItemStorageModel.id = id
        checklistItemStorageModel.title = title
        checklistItemStorageModel.isChecked = isChecked
        return checklistItemStorageModel
    }
}

extension ChecklistStorageModel {
    func findItemBy(id: String) -> ChecklistItemStorageModel? {
        wrappedItems.first { (item: ChecklistItemStorageModel) in
            item.id == id
        }
    }
    
    func addItem(newItem: ChecklistItemStorageModel) {
        let setOrEmpty: NSSet = items ?? NSSet()
        items = setOrEmpty.adding(newItem) as NSSet
    }
}

extension ChecklistItemStorageModel {
    func updateWith(_ other: Checklist.Item) {
        self.id = other.id
        self.title = other.title
        self.isChecked = other.isChecked
    }
}

func tryForResult<T>(action: () throws -> T) -> Result<T, Error> {
    do {
        return Result.success(try action())
    } catch let error {
        return Result.failure(error)
    }
}

public extension NSManagedObject {

    convenience init(context: NSManagedObjectContext) {
        let name = String(describing: type(of: self))
        let entity = NSEntityDescription.entity(forEntityName: name, in: context)!
        self.init(entity: entity, insertInto: context)
    }
}
