// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import CoreData
import Foundation
import GMAppData
import Combine
import ErrorData
import DependencyInjection
import Logger

class MaternityStorageServiceImpl: MaternityStorageService {
    @Inject private var logger: LoggerProtocol
    
    static let shared = MaternityStorageServiceImpl()
    private let context = LocalStorageController().container.viewContext
    
    func fetchMaternity() -> AnyPublisher<MaternityStorageModel?, ServiceError> {
        var storageModel: MaternityStorageModel?
        
        do {
            logger.debug("Fetching maternity stored on device...")
            let items = try context.fetch(MaternityStorageModel.fetchRequest())
            
            if let item = items.first {
                logger.debug("Maternity was fetched!")
                storageModel = item
            } else {
                logger.debug("No maternity was fetched from device!")
            }
        } catch let error as NSError {
            logger.error("Failed to fetch maternity stored on device! \(error.localizedDescription)")
        }
        return Just(storageModel)
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func saveMaternity(maternity: MaternityRepoModel) -> AnyPublisher<MaternityStorageModel, ServiceError> {
        let storageModel = MaternityStorageModel(context: context)
        storageModel.dateOfBirth = maternity.dateOfBirth
        storageModel.dateOfDelivery = maternity.expectedDateOfDelivery
        
        do {
            logger.debug("Saving maternity on device...")
            try context.save()
            
        } catch let error as NSError {
            logger.error("Failed to save maternity on device! \(error.localizedDescription)")
        }
        
        logger.debug("Maternity was saved!")
        return Just(storageModel)
            .setFailureType(to: ServiceError.self)
            .eraseToAnyPublisher()
    }
    
    func deleteMaternity() {
        do {
            logger.debug("Fetching maternity stored on device to delete...")
            let maternityEntites = try context.fetch(MaternityStorageModel.fetchRequest())
            
            for entity in maternityEntites {
                context.delete(entity)
            }
            do {
                logger.debug("Deleting maternity stored on device...")
                try context.save()
                
            } catch let error as NSError {
                logger.error("Failed to delete maternity on device! \(error.localizedDescription)")
            }
        } catch let error as NSError {
            logger.error("Failed to fetch maternity stored on device to delete! \(error.localizedDescription)")
        }
        logger.debug("Maternity was deleted!")
    }
    
    func updateMaternity(maternity: MaternityRepoModel) -> AnyPublisher<MaternityStorageModel?, ServiceError> {
        var storageModelResult: MaternityStorageModel?
        
        do {
            logger.debug("Fetching maternity stored on device to update...")
            let storageModel = try context.fetch(MaternityStorageModel.fetchRequest()).first
            
            if let storageModel = storageModel {
                storageModel.dateOfBirth = maternity.dateOfBirth
                storageModel.dateOfDelivery = maternity.expectedDateOfDelivery
                storageModelResult = storageModel
            }
            
            do {
                logger.debug("Updating maternity stored on device...")
                try context.save()
                
            } catch let error as NSError {
                logger.error("Failed to update maternity on device! \(error.localizedDescription)")
            }
            
            logger.debug("Maternity was updated!")
            return Just(storageModelResult)
                .mapToServiceError()
                .eraseToAnyPublisher()
            
        } catch let error as NSError {
            logger.error("Failed to fetch maternity stored on device to update! \(error.localizedDescription)")
            return Just(storageModelResult)
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
    }
    
    private func createMaternityStorageModel(maternity: MaternityRepoModel) {
        let storageModel = MaternityStorageModel(context: context)
        storageModel.dateOfBirth = maternity.dateOfBirth
        storageModel.dateOfDelivery = maternity.expectedDateOfDelivery
    }
}

extension MaternityStorageModel {
    func toRepoModel() -> MaternityRepoModel {
        MaternityRepoModel(dateOfBirth: dateOfBirth, expectedDateOfDelivery: dateOfDelivery!)
    }
}
