// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation

struct MaternityRepoModel {
    let dateOfBirth: Date?
    let expectedDateOfDelivery: Date?
}

extension MaternityRepoModel {
    
    private func calculateOffsetDate(_ date: Date) -> Date? {
        
        let offsetDate = date.date(byAdding: .day, value: -280)?.startOfDay
        return offsetDate
    }
    
    private func calculateMonthsIntoPregnancy(_ offsetDate: Date?) -> Double {
        guard let daysIntoPregnancy = calculateDaysIntoPregnancy(offsetDate: offsetDate)
        else { return 0 }
        
        return Double(daysIntoPregnancy) / 30.42
    }
    
    private func calculateDaysIntoPregnancy(offsetDate: Date?) -> Int? {
        guard let offsetDate = offsetDate else { return nil }
        let maxDays = 300
        
        let calendar = Calendar(identifier: .gregorian)
        
        if let daysInto = calendar.dateComponents([.day], from: offsetDate, to: Date()).day {
            return daysInto >= maxDays ? maxDays : daysInto
        }
        return nil
    }
    
    private func calculateGestationAge(offsetDate: Date?) -> String? {
        guard let daysIntoPregnancy = calculateDaysIntoPregnancy(offsetDate: offsetDate),
              let week = calculateWeek(offsetDate: offsetDate)
        else { return nil }
        
        return "\(week - 1)+\(daysIntoPregnancy % 7)"
    }
    
    private func calculateTrimester(offsetDate: Date?) -> Int {
        guard let daysIntoPregnancy = calculateDaysIntoPregnancy(offsetDate: offsetDate) else { return 1 }
        
        if daysIntoPregnancy < (12 * 7) {
            return 1
        } else if daysIntoPregnancy < (28 * 7) {
            return 2
        } else {
            return 3
        }
    }
    
    private func calculateWeek(offsetDate: Date?) -> Int? {
        guard let daysIntoPregnancy = calculateDaysIntoPregnancy(offsetDate: offsetDate) else { return nil }
        let maxWeek = 43
        let week = (daysIntoPregnancy / 7) + 1
        return week >= maxWeek
        ? maxWeek
        : week
    }
    
    func toSelectedMaternity() -> SelectedMaternity? {
        guard let expectedDateOfDelivery = expectedDateOfDelivery else { return nil }
        let offsetDate = calculateOffsetDate(expectedDateOfDelivery)
        
        return SelectedMaternity(
            dateOfBirth: self.dateOfBirth,
            offsetDate: offsetDate,
            dateOfDelivery: self.expectedDateOfDelivery,
            gestationAge: calculateGestationAge(offsetDate: offsetDate),
            trimester: calculateTrimester(offsetDate: offsetDate),
            monthsIntoPregnancy: calculateMonthsIntoPregnancy(offsetDate),
            week: calculateWeek(offsetDate: offsetDate)
        )
    }
}
