// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import UserSettingsServices
import MaternityServices
import ConfigurationServices
import GMAppData
import ErrorData
import Combine
import HTTPNetworking
import TIM

final class UserContext : UserContextProtocol {
    @DI private var timAuth: TIMAuth
    @DI private var timStorage: TIMDataStorage
    @DI private var userSettings: UserSettingsService
    @DI private var logger: Logger
    @DI private var maternityRepo: MaternityRepository
    @DI private var checklistRepo: ChecklistRepository

    var hasHandledOnboardingFlow: Bool {
        userSettings.get(key: .hasHandledOnboarding) ?? false
    }
    
    var hasEnabledLocalAuthentication: Bool {
        userSettings.get(key: .hasEnabledAuthentication) ?? false
    }
    
    var isLocalAuthenticated: Bool {
        userSettings.get(key: .isAuthenticated) ?? false
      }

    var swipeToDeleteTipCount: Int {
        userSettings.get(key: .swipeToDeleteTipSeenCount) ?? 0
    }
    
    var userId: String? {
        timAuth.refreshToken?.userId
    }
    
    var hasUserRegistered: Bool {
        !timStorage.availableUserIds.isEmpty
    }
    
    var isTIMAuthenticated: Bool {
        timAuth.isLoggedIn
    }
    
    func logout() {
        guard let userId = userId else {
            resetForUndefinedUserState()
            return
        }

        timAuth.logout()
        clearEnvironmentStoreStates()
        NotificationCenter.default.post(
            notification: Notification.Name.SceneDelegate.changeRootViewController,
            value: SceneDelegate.NotificationMessage.logout(userId: userId)
        )
    }

    func resetAllUserData() {
        self.clearStoredUserSettings()
        self.clearEnvironmentStoreStates()
        self.resetToOnboarding()
    }

    func setHasHandledOnboardingFlow(_ handled: Bool) {
        userSettings.save(handled, for: .hasHandledOnboarding)
    }
    
    func sethasEnabledAuthentication(_ enabled: Bool) {
        userSettings.save(enabled, for: .hasEnabledAuthentication)
    }
    
    func setIsAuthenticated(_ authenticated: Bool) {
        userSettings.save(authenticated, for: .isAuthenticated)
    }

    
    func incrementSwipeToDeleteTipCount() {
        if swipeToDeleteTipCount <= 10 {
            userSettings.save(swipeToDeleteTipCount + 1, for: .swipeToDeleteTipSeenCount)
        }
    }
    
    func disableSwipeToDeleteTip() {
        userSettings.save(11, for: .swipeToDeleteTipSeenCount)
    }

    func clearStoredUserSettings() {
        userSettings.remove(key: .hasHandledOnboarding)
        userSettings.remove(key: .swipeToDeleteTipSeenCount)
        userSettings.remove(key: .hasEnabledAuthentication)
    }

    func clearEnvironmentStoreStates() {
        let userDataStores: [UserDataStore] = [
            self.maternityRepo,
            self.checklistRepo
        ]
        userDataStores.forEach({ $0.clear() })
    }

    private func resetForUndefinedUserState() {
        logger.critical("Tried to logout, but did not have a userId. Resetting all registered users and resetting to onboarding!")
        userSettings.remove(key: .selectedEpisodeOfCareId)
        timStorage.clearAllUsers()
        setHasHandledOnboardingFlow(false)
        clearEnvironmentStoreStates()
        clearStoredUserSettings()
        resetToOnboarding()
    }
    
    private func resetToOnboarding() {
        NotificationCenter.default.post(
            notification: Notification.Name.SceneDelegate.changeRootViewController,
            value: SceneDelegate.NotificationMessage.resetToOnboarding
        )
    }
}
