// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import SwiftUI

struct AlertBottomView: View {
    let state: AlertState
    let title: String?
    let description: String
    let showCloseIcon: Bool
    let isSticky: Bool

    init(
        state: AlertState,
        title: String? = nil,
        description: String,
        showCloseIcon: Bool,
        isSticky: Bool = false
    ) {
        self.state = state
        self.title = title
        self.description = description
        self.showCloseIcon = showCloseIcon
        self.isSticky = isSticky
    }
    
    var body: some View {
        ZStack {
            HStack(alignment: .top, spacing: Metrics.Spacing.space12) {
                        Image(R.image.icons.basic.info.name)
                            .resizable()
                            .renderingMode(.template)
                            .foregroundColor(state.iconColor)
                            .smallIconStyle()
                        
                            HStack(spacing: Metrics.Spacing.space0) {
                                VStack(alignment: .leading, spacing: Metrics.Spacing.space4) {
                                    if let title = title {
                                        Text(title)
                                            .applyTextStyle(fontStyle: FontStyle.headlineXXS)
                                            .lineLimit(2)
                                        .foregroundColor(state.textColor)
                                    }
                                    Text(description)
                                        .applyTextStyle(fontStyle: FontStyle.textXS)
                                        .multilineTextAlignment(.leading)
                                        .lineLimit(.max)
                                }
                                Spacer()
                                if showCloseIcon {
                                    Image(R.image.icons.basic.close.name)
                                        .resizable()
                                        .renderingMode(.template)
                                        .foregroundColor(state.closeIconColor)
                                        .xsmallIconStyle()
                                }
                            }
                    }
                    .padding()
                    .background(state.backgroundColor)
                .clipShape(RoundedTopCornersShape(radius: Metrics.Spacing.space12))
            
        }
        .offset(y: 1)
        .background(state.borderColor)
        .clipShape(RoundedTopCornersShape(radius: Metrics.Spacing.space12))
    }
}

struct AlertStickyCardView_Previews: PreviewProvider {
    static var previews: some View {
        VStack(spacing: 16) {
            AlertBottomView(
                state: AlertState.info,
                title: "Info title",
                description: "Description of the info",
                showCloseIcon: true
            )
            AlertBottomView(
                state: AlertState.alert,
                title: nil,
                description: "Description of the alert",
                showCloseIcon: true
            )
            AlertBottomView(
                state: AlertState.error,
                title: "Error title",
                description: "Description of the error",
                showCloseIcon: true
            )
            AlertBottomView(
                state: AlertState.success,
                title: "Success title",
                description: "Description of the success",
                showCloseIcon: true
            )
        }
        .padding()
        .background(Color.gray.opacity(0.0).edgesIgnoringSafeArea(.all))
    }
}
