// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct GeneralCardView: View {
    let image: Image?
    let title: String
    let description: String
    let tag: String?
    let showChevron: Bool
    let titleFontStyle: FontStyle
    let descriptionFontSyle: FontStyle
    let imageColor: Color?

    init(
        image: Image? = nil,
        title: String,
        description: String,
        tag: String? = nil,
        showChevron: Bool,
        titleFontStyle: FontStyle = FontStyle.headlineXS,
        descriptionFontSyle: FontStyle = FontStyle.textS,
        imageColor: Color? = nil

    ) {
        self.image = image
        self.title = title
        self.description = description
        self.tag = tag
        self.showChevron = showChevron
        self.titleFontStyle = titleFontStyle
        self.descriptionFontSyle = descriptionFontSyle
        self.imageColor = imageColor
    }
    
    var body: some View {
        BaseCardView {
            HStack(alignment: .top, spacing: Metrics.Spacing.space16) {
                if let image = image {
                    image
                        .resizable()
                        .foregroundColor(imageColor)
                        .largeIconStyle()
                }
                
                VStack(alignment: .leading, spacing: Metrics.Spacing.space0) {
                    HStack(spacing: Metrics.Spacing.space0) {
                        Text(title)
                            .applyTextStyle(
                                fontStyle: titleFontStyle,
                                textColor: ColorManager.textPrimary.currentColor()
                            )
                            .multilineTextAlignment(.leading)
                            .lineLimit(2)
                        Spacer()
                        if showChevron {
                            ChevronView(direction: .right)
                        }
                    }
                    .padding(.bottom, Metrics.Spacing.space8)
                    
                    Text(description)
                        .applyTextStyle(
                            fontStyle: descriptionFontSyle,
                            textColor: ColorManager.textPrimary.currentColor()
                        )
                        .multilineTextAlignment(.leading)
                        .lineLimit(3)
                    
                    if let tag = tag {
                        Text(tag)
                            .applyTextStyle(
                                fontStyle: FontStyle.textXS,
                                textColor: ColorManager.textSecondary.currentColor()
                            )
                            .padding(.top, Metrics.Spacing.space4)
                    }
                }
            }
            .padding()
        }
        .applySmallShadow()
    }
}

struct GeneralCardView_Previews: PreviewProvider {
    static var previews: some View {
        VStack(spacing: Metrics.Spacing.space20) {
            GeneralCardView(
                image: Image(systemName: "photo"),
                title: "Very very very long cardtitle",
                description: "This is a multiline description for the content of the general card view",
                tag: "Card tag",
                showChevron: true
            )
            GeneralCardView(
                image: Image(MoreView.ListItem.support.image.name),
                title: "Card title",
                description: "Card description",
                tag: "Card tag",
                showChevron: true,
                imageColor: .red
            )
            GeneralCardView(
                image: nil,
                title: "Card title",
                description: "Card description",
                showChevron: true
            )
            GeneralCardView(
                image: nil,
                title: "Card title",
                description: "Card description",
                showChevron: false
            )
        }
        .padding()
        .background(Color.gray.opacity(0.1).edgesIgnoringSafeArea(.all))
    }
}
