import SwiftUI
import Charts

struct MeasurementsChart: View {
    let chartValues: [ChartValue]
    let chartNormalValues: ChartNormalValues
    let chartXData: AxisData
    let chartYData: AxisData

    @State private var selectedValue: ChartValue?
    @State private var tapLocation: CGPoint = .zero
    @State private var arrowDirection: TooltipArrowDirection = .downMiddle

    var body: some View {
        Chart {
            chartMarks
        }
        .chartYAxis {
            AxisMarks(position: .leading, values: chartYData.values) { value in
                AxisGridLine(stroke: GridLineStyle.stroke(for: value.index))
                    .foregroundStyle(ColorManager.borderChart.currentColor())
                AxisValueLabel {
                    yAxisLabel(for: value)
                }
            }
        }
        .chartXScale(
            domain: chartXData.values.first!...chartXData.values.last! + 2,
            range: .plotDimension(
                startPadding: Metrics.Spacing.space0,
                endPadding: Metrics.Spacing.space16
            )
        )
        .chartXAxis {
            AxisMarks(values: chartXData.values) { value in
                AxisTick(length: 8, stroke: StrokeStyle(lineWidth: 1))
                    .foregroundStyle(ColorManager.borderChart.currentColor())
                AxisValueLabel {
                    xAxisLabel(for: value)
                }
            }
        }
        .chartOverlay { proxy in
            GeometryReader { geo in
                Rectangle()
                    .fill(Color.clear)
                    .contentShape(Rectangle())
                    .coordinateSpace(name: "chartArea") // <- coordinate space setup
                    .gesture(
                        DragGesture(minimumDistance: 0)
                            .onEnded { gesture in
                                let location = gesture.location // <- no convert needed
                                if let xValue: Double = proxy.value(atX: location.x) {
                                    if let nearest = chartValues.min(by: {
                                        abs($0.x - xValue) < abs($1.x - xValue)
                                    }) {
                                        selectedValue = nearest
                                        tapLocation = location
                                        arrowDirection = calculateArrowDirection(from: location, in: geo.size)
                                    }
                                }
                            }
                    )
            }
        }
    }

    // MARK: - Chart Marks
    private var chartMarks: some ChartContent {
        ForEach(chartValues) { value in
            PointWithTooltipMark(
                value: value,
                chartXUnit: chartXData.unit,
                chartYUnit: chartYData.unit,
                isSelected: selectedValue?.id == value.id,
                arrowDirection: arrowDirection
            )
        }
    }

    // MARK: - Tooltip Direction Logic
    private func calculateArrowDirection(from point: CGPoint, in screenSize: CGSize) -> TooltipArrowDirection {
        let thirdWidth = screenSize.width / 3
        let thirdHeight = screenSize.height / 3

        let column = Int(point.x / thirdWidth)
        let row = Int(point.y / thirdHeight)

        switch (row, column) {
        case (0, 0): return .upLeft
        case (0, 1): return .upMiddle
        case (0, 2): return .upRight
        case (1, 0), (2, 0): return .downLeft
        case (1, 1), (2, 1): return .downMiddle
        case (1, 2), (2, 2): return .downRight
        default: return .downMiddle
        }
    }

    // MARK: - Axis Labels
    @ViewBuilder
    private func yAxisLabel(for value: AxisValue) -> some View {
        if value.index != 0, let yValue = value.as(Double.self) {
            Text("\(yValue, specifier: "%.1f") \(chartYData.unit)")
                .applyTextStyle(
                    fontStyle: .textXSStrong,
                    textColor: ColorManager.textPrimary.currentColor()
                )
        }
    }

    @ViewBuilder
    private func xAxisLabel(for value: AxisValue) -> some View {
        if let xValue = value.as(Int.self) {
            let isLast = value.index == chartXData.values.count - 1
            let labelText = isLast ? "\(xValue)\n\(chartXData.unit)" : "\(xValue)"
            CenteredAxisLabel(labelText: labelText)
        }
    }
}

// MARK: - Tooltip + Point Renderer

private struct PointWithTooltipMark: ChartContent {
    let value: ChartValue
    let chartXUnit: String
    let chartYUnit: String
    let isSelected: Bool
    let arrowDirection: TooltipArrowDirection

    var body: some ChartContent {
        PointMark(
            x: .value(chartXUnit, value.x),
            y: .value(chartYUnit, value.y)
        )
        .opacity(0)
        .annotation(position: .overlay) {
            ChartPointMarkSymbol()
                .applySmallShadow()
        }
        .annotation(position: .overlay) {
            if isSelected {
                let formattedText = "Uge \(Int(value.x)), \(String(format: "%.1f", value.y)) \(chartYUnit)"
                TooltipBubble(
                    text: formattedText,
                    arrowDirection: arrowDirection,
                    backgroundColor: .purple,
                    textColor: .white
                )
            }
        }
    }
}

// MARK: - Preview

struct MeasurementsChart_Previews: PreviewProvider {
    static var previews: some View {
        MeasurementsChart(
            chartValues: ChartValue.mocklist(),
            chartNormalValues: ChartNormalValues.mock(),
            chartXData: AxisData.mockXAxis(),
            chartYData: AxisData.mockYAxis()
        )
        .padding()
    }
}
