import SwiftUI

enum TooltipArrowDirection {
    case upLeft, upMiddle, upRight
    case downLeft, downMiddle, downRight
}

struct TooltipBubble: View {
    let text: String
    let arrowDirection: TooltipArrowDirection
    let backgroundColor: Color
    let textColor: Color
    
    @State private var bubbleWidth: CGFloat = 0
    
    var body: some View {
        VStack(spacing: 0) {
            if arrowDirection.isArrowOnTop {
                arrow
            }

            Text(text)
                .font(.system(size: 14, weight: .semibold))
                .foregroundColor(textColor)
                .padding(.horizontal, 12)
                .padding(.vertical, 8)
                .background(
                    GeometryReader { geo in
                        RoundedRectangle(cornerRadius: 12)
                            .fill(backgroundColor)
                            .preference(key: BubbleWidthPreferenceKey.self, value: geo.size.width)
                    }
                )
                .onPreferenceChange(BubbleWidthPreferenceKey.self) { bubbleWidth = $0 }

            if !arrowDirection.isArrowOnTop {
                arrow
            }
        }
    }

    private var arrow: some View {
        Triangle()
            .fill(backgroundColor)
            .frame(width: 12, height: 6)
            .rotationEffect(arrowDirection.isArrowOnTop ? .zero : .degrees(180))
            .offset(x: arrowOffset(for: bubbleWidth))
            .frame(height: 6)
    }

    private func arrowOffset(for width: CGFloat) -> CGFloat {
        switch arrowDirection {
        case .upLeft, .downLeft:
            return -(width / 2) + 18 // adjust based on padding
        case .upMiddle, .downMiddle:
            return 0
        case .upRight, .downRight:
            return (width / 2) - 18 // adjust based on padding
        }
    }
}

// MARK: - Direction Helpers

private extension TooltipArrowDirection {
    var isArrowOnTop: Bool {
        switch self {
        case .upLeft, .upMiddle, .upRight:
            return true
        case .downLeft, .downMiddle, .downRight:
            return false
        }
    }
    
    var alignment: Alignment {
        switch self {
        case .upLeft, .downLeft:
            return .leading
        case .upMiddle, .downMiddle:
            return .center
        case .upRight, .downRight:
            return .trailing
        }
    }
}

// MARK: - Triangle Shape

struct Triangle: Shape {
    func path(in rect: CGRect) -> Path {
        var path = Path()
        path.move(to: CGPoint(x: rect.midX, y: rect.minY)) // top
        path.addLine(to: CGPoint(x: rect.maxX, y: rect.maxY)) // bottom right
        path.addLine(to: CGPoint(x: rect.minX, y: rect.maxY)) // bottom left
        path.closeSubpath()
        return path
    }
}

// MARK: - PreferenceKey for width

private struct BubbleWidthPreferenceKey: PreferenceKey {
    static var defaultValue: CGFloat = 0
    static func reduce(value: inout CGFloat, nextValue: () -> CGFloat) {
        value = nextValue()
    }
}

// MARK: - Preview

struct TooltipBubble_Previews: PreviewProvider {
    static var previews: some View {
        VStack(spacing: 30) {
            TooltipBubble(text: "Down Middle", arrowDirection: .downMiddle, backgroundColor: .purple, textColor: .white)
            TooltipBubble(text: "Down Left", arrowDirection: .downLeft, backgroundColor: .purple, textColor: .white)
            TooltipBubble(text: "Down Right", arrowDirection: .downRight, backgroundColor: .purple, textColor: .white)
            TooltipBubble(text: "Up Middle", arrowDirection: .upMiddle, backgroundColor: .purple, textColor: .white)
            TooltipBubble(text: "Up Left", arrowDirection: .upLeft, backgroundColor: .purple, textColor: .white)
            TooltipBubble(text: "Up Right", arrowDirection: .upRight, backgroundColor: .purple, textColor: .white)
        }
        .padding()
        .background(Color(.systemBackground))
    }
}
