// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI

final class CustomNumberPadKeyboardViewController: SUIViewContainerViewController<UIView> {
    private lazy var customInputViewController = NumberPadInputViewController(isBiometricKeyEnabled: isBiometricKeyEnabled)
    var keyEventActivated: ((NumberPadKeyboardKeyType) -> Void)? {
        didSet {
            customInputViewController.keyEventActivated = keyEventActivated
        }
    }

    var isBiometricKeyEnabled: Bool = true

    override var inputViewController: UIInputViewController? {
        customInputViewController
    }
}

private final class NumberPadInputViewController: UIInputViewController {
    @DI var logger: Logger

    let isBiometricKeyEnabled: Bool
    var keyEventActivated: ((NumberPadKeyboardKeyType) -> Void)?

    init(isBiometricKeyEnabled: Bool) {
        self.isBiometricKeyEnabled = isBiometricKeyEnabled
        super.init(nibName: nil, bundle: nil)
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        inputView = NumberPadInputView(
            isBioKeyEnabled: isBiometricKeyEnabled,
            keyPressed: keyEvent
        )
    }

    private func keyEvent(_ key: NumberPadKeyboardKeyType) {
        switch key {
        case .number:
            guard let value = key.value else {
                return
            }
            textDocumentProxy.insertText(value)
        case .delete:
            textDocumentProxy.deleteBackward()
        default:
            break
        }

        keyEventActivated?(key)
    }
}

private final class NumberPadInputView: UIInputView {
    private var keyPressed: (NumberPadKeyboardKeyType) -> Void
    private let isBioKeyEnabled: Bool

    required init(isBioKeyEnabled: Bool, keyPressed: @escaping (NumberPadKeyboardKeyType) -> Void) {
        self.keyPressed = keyPressed
        self.isBioKeyEnabled = isBioKeyEnabled
        super.init(frame: .zero, inputViewStyle: .default)
        setup()
    }

    required init?(coder: NSCoder) {
        fatalError("You have to provide keyPressed event")
    }

    private func setup() {
        let vc = UIHostingController(
            rootView: NumberPadKeyboard(
                isBioKeyEnabled: isBioKeyEnabled,
                keyPressed: keyPressed
            )
        )
        vc.view.translatesAutoresizingMaskIntoConstraints = false
        vc.view.backgroundColor = ColorManager.bgNeutralPrimary.currentUIColor()
        self.addSubview(vc.view, withEdgeInsets: .zero)

        let safeAreaBottom = UIApplication.shared.windows.first(where: { $0.isKeyWindow })?.safeAreaInsets.bottom ?? 0
        self.frame = CGRect(
            x: 0,
            y: 0,
            width: UIScreen.main.bounds.width,
            height: Metrics.Sizes.numberKeyboardHeight + Metrics.Margins.numberKeyboardBottom + safeAreaBottom
        )
    }
}
