// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import LocalAuthentication
import TIM
import TIMEncryptedStorage
import Combine
import ErrorData
import UIKit

extension LoginView {
    final class ViewModel : ObservableObject, PinCodeLoginViewModel {
        @DI var logger: Logger
        @DI private var timAuth: TIMAuth
        @DI private var localAuth: LocalAuthenticationContext
        //@DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
        @DI var timStorage: TIMDataStorage
        @DI var userContext: UserContextProtocol

        var cancelBag = CancelBag()

        let userId: String
        let presentedFromLogout: Bool

        @Published var pinCode: String = ""
        var pinCodePublisher: Published<String>.Publisher { $pinCode }

        @Published var isLoading: Bool = false

        @Published var wrongCode: Bool = false

        @Published var biometricLoginWasActivated: Bool = false
        @Published var shouldResetWrongPinCodeState: Bool = false
        var shouldResetWrongPinCodeStatePublisher: Published<Bool>.Publisher { $shouldResetWrongPinCodeState }

        @Published var showKeyboard: Bool = true

        @Published var loginInvalidated: Bool = false
        @Published var genericError: Bool = false
        @Published var sessionExpired: Bool = false
        @Published var clientTimeIsOff: Bool = false
        @Published var failedToLoadInitialDataError: Error?
        @Published var criticalUnexpectedError: Bool = false

        @Published var forgotPassword: Bool = false

        var isBiometricLoginAvailable: Bool {
            localAuth.isAuthenticationAvailable &&
                timStorage.hasBiometricAccessForRefreshToken(userId: userId)
        }

        var loginText: String {
            let bioName = localAuth.authentiocationName
            if bioName.isEmpty {
                return Localizable.login_text_no_biometrics_available()
            } else {
                return Localizable.login_text(bioName)
            }
        }

        init(userId: String, presentedFromLogout: Bool) {
            self.userId = userId
            self.presentedFromLogout = presentedFromLogout
            self.observeChangesForView()
        }

        private func observeChangesForView() {
            observeChanges()

            $biometricLoginWasActivated
                .sink { [weak self] (activated) in
                    if activated && self?.isLoading == false {
                        self?.loginWithBio()
                    }
                }
                .store(in: &cancelBag)
        }

        func handleCompletedPIN(_ pin: String) {
            isLoading = true
            showKeyboard = false
            timAuth.loginWithPassword(userId: userId, password: pin, storeNewRefreshToken: true)
                .mapError(LoginError.timError)
//                .flatMap({ _ in
//                    self.fetchInitialData()
//                        .mapError(LoginError.failedToGetInitialData)
//                })
//                .receiveOnMain()
                .sink(
                    receiveCompletion: handleLoginCompletion,
                    receiveValue: {  _ in }
                )
                .store(in: &cancelBag)
        }

        private func loginWithBio() {
            timAuth.loginWithBiometricId(
                userId: userId,
                storeNewRefreshToken: true,
                willBeginNetworkRequests: { [weak self] in
                    self?.isLoading = true
                    self?.showKeyboard = false
                })
                .mapError(LoginError.timError)
//                .flatMap({ _ in
//                    self.fetchInitialData()
//                        .mapError(LoginError.failedToGetInitialData)
//                })
//                .receiveOnMain()
                .sink(
                    receiveCompletion: handleLoginCompletion,
                    receiveValue: {  _ in }
                )
                .store(in: &cancelBag)
        }
        
        private func handleLoginCompletion(completion: Subscribers.Completion<LoginError>) {
            switch completion {
            case .failure(let error):
                switch error {
                case .failedToGetInitialData:
                    logger.debug("Failed to load initial user data!")
                    failedToLoadInitialDataError = error
                    pinCode = "" // reset
              //      maternityEnvironmentStore.clear() // Clear data - something went wrong while requesting update.
                case .timError(let timError):
                    handleTimError(timError)
                }
            case .finished:
                logger.info("Login completed!")
                NotificationCenter.default.post(
                    notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                    value: SceneDelegate.NotificationMessage.showTabBarController
                )
            }
            isLoading = false
        }

//        private func fetchInitialData() -> AnyPublisher<Void, ServiceError> {
//         //   maternityEnvironmentStore.requestUpdate()
//            promise(.()) // Immediately succeeds with Void
//
//        }
    }
}

enum LoginError : Error, LocalizedError {
    case timError(TIMError)
    case failedToGetInitialData(ServiceError)

    var errorDescription: String? {
        switch self {
        case .timError(let timError): return timError.localizedDescription
        case .failedToGetInitialData(let serviceError): return serviceError.localizedDescription
        }
    }
}
