// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import TIM
import TIMEncryptedStorage

/// This protocol is designed for views with ViewModels that needs to validate the PIN code with TIM and then perform some action afterwards.
protocol PinCodeLoginViewModel: AnyObject {
    var pinCodeLength: UInt { get }

    var pinCode: String { get set }
    var pinCodePublisher: Published<String>.Publisher { get }

    var isLoading: Bool { get set }
    var wrongCode: Bool { get set }

    var shouldResetWrongPinCodeState: Bool { get set }
    var shouldResetWrongPinCodeStatePublisher: Published<Bool>.Publisher { get }

    var showKeyboard: Bool { get set }

    var loginInvalidated: Bool { get set }
    var genericError: Bool { get set }
    var sessionExpired: Bool { get set }
    var clientTimeIsOff: Bool { get set }
    var criticalUnexpectedError: Bool { get set }

    var cancelBag: CancelBag { get set }

    var logger: Logger { get }

    var timStorage: TIMDataStorage { get }
    var userContext: UserContextProtocol { get }

    func handleCompletedPIN(_ pin: String)
}

extension PinCodeLoginViewModel {
    var pinCodeLength: UInt {
        Metrics.Lengths.pinCode
    }

    func observeChanges() {
        pinCodePublisher
            .sink { [weak self] (pinCode) in
                guard let self = self else {
                    return
                }
                if pinCode.length == self.pinCodeLength, !self.isLoading {
                    self.handleCompletedPIN(pinCode)
                }
            }
            .store(in: &cancelBag)

        shouldResetWrongPinCodeStatePublisher
            .sink { [weak self] (reset) in
                if reset {
                    self?.wrongCode = false
                    self?.pinCode = ""
                    self?.shouldResetWrongPinCodeState = false
                }
            }
            .store(in: &cancelBag)
    }

    //swiftlint:disable:next cyclomatic_complexity
    func handleTimError(_ timError: TIMError) {
        switch timError {
        case .storage(let storageError):
            switch storageError {
            case .encryptedStorageFailed:
                if storageError.isWrongPassword() {
                    logger.debug("Wrong PIN")
                    wrongCode = true
                    showKeyboard = true
                } else if storageError.isKeyLocked() {
                    logger.debug("Wrong PIN was provided three times. The Login is invalidated.")
                    loginInvalidated = true
                } else if storageError.isBiometricFailedError() {
                    logger.info("Bio login cancelled or failed.")
                    shouldResetWrongPinCodeState = true //Reset pin code state
                } else if case .encryptedStorageFailed(let encryptionError) = storageError, !storageError.isKeyServiceError() {
                    handleEncryptedStorageError(encryptionError)
                    criticalUnexpectedError = true
                } else {
                    // Could be "no internet" errors, etc.
                    logger.info("Failed to login with storage error: \(storageError.localizedDescription)")
                    genericError = true
                }
            case .incompleteUserDataSet:
                logger.error("Incomplete user data set: \(storageError.localizedDescription)")
                hardResetTIM()
                criticalUnexpectedError = true
            }
        case .auth(let authError):
            switch authError {
            case .refreshTokenExpired:
                sessionExpired = true
            case .failedToValidateIDToken:
                clientTimeIsOff = true
            default:
                logger.error("Failed to login: \(authError.localizedDescription)")
                genericError = true
            }
        }
    }

    func resetToOnboarding() {
        NotificationCenter.default.post(
            notification: NSNotification.Name.SceneDelegate.changeRootViewController,
            value: SceneDelegate.NotificationMessage.resetToOnboarding
        )
    }

    @ViewBuilder
    func alertViewsForErrorStates() -> some View {
        VStack { }
            .alertOnBackground(isPresented: Binding(get: { self.sessionExpired }, set: { [weak self] v in self?.sessionExpired = v }), alert: {
                Alert(
                    title: Text(Localizable.common_alert_title_error()),
                    message: Text(Localizable.login_error_refresh_token_expired()),
                    dismissButton: .default(Text(Localizable.common_ok()), action: { [weak self] in
                        self?.resetToOnboarding()
                    })
                )
            })
            .alertOnBackground(isPresented: Binding(get: { self.clientTimeIsOff }, set: { [weak self] v in self?.clientTimeIsOff = v }), alert: {
                Alert(
                    title: Text(Localizable.common_alert_title_error()),
                    message: Text(Localizable.login_error_idtoken()),
                    dismissButton: .default(Text(Localizable.common_ok()), action: { [weak self] in
                        self?.shouldResetWrongPinCodeState = true
                        self?.showKeyboard = true
                    })
                )
            })
            .alertOnBackground(isPresented: Binding(get: { self.loginInvalidated }, set: { [weak self] v in self?.loginInvalidated = v }), alert: {
                Alert(
                    title: Text(Localizable.common_alert_title_error()),
                    message: Text(Localizable.login_error_key_locked()),
                    dismissButton: .default(Text(Localizable.common_ok()), action: { [weak self] in
                        self?.resetToOnboarding()
                    })
                )
            })
            .alertOnBackground(isPresented: Binding(get: { self.genericError }, set: { [weak self] v in self?.genericError = v }), alert: {
                Alert(
                    title: Text(Localizable.common_alert_title_error()),
                    message: Text(Localizable.common_alert_message_failed_to_load_data()),
                    dismissButton: .default(Text(Localizable.common_ok()), action: { [weak self] in
                        self?.shouldResetWrongPinCodeState = true
                        self?.showKeyboard = true
                    })
                )
            })
            .alertOnBackground(isPresented: Binding(get: { self.criticalUnexpectedError }, set: { [weak self] v in self?.criticalUnexpectedError = v }), alert: {
                Alert(
                    title: Text(Localizable.common_alert_title_error()),
                    message: Text(Localizable.login_error_critical()),
                    dismissButton: .default(Text(Localizable.common_ok()), action: { [weak self] in
                        self?.resetToOnboarding()
                    })
                )
            })
    }

    private func handleEncryptedStorageError(_ encryptedStorageError: TIMEncryptedStorageError) {
        switch encryptedStorageError {
        case .keyServiceFailed:
            break // This might just be internet issues or maintenance on the KeyService - let it go for now!
        case .failedToDecryptData,
             .failedToEncryptData,
             .invalidEncryptionKey,
             .invalidEncryptionMethod,
             .secureStorageFailed,
             .unexpectedData:
            // Report as error - this is unexpected!
            logger.error("Failed to retrieve encrypted data, invalidating session: \(encryptedStorageError)")
            hardResetTIM()
        }
    }

    /// When shit hits the fan and this is the only way out 💩🤯
    private func hardResetTIM() {
//        guard let userId = userContext.userId else {
//            timStorage.clearAllUsers()
//            return
//        }
//        timStorage.clear(userId: userId)
    }
}
