import SwiftUI

struct AppointmentsView: View, ScreenTrackable {
    @StateObject private var viewModel: AppointmentsViewModel
    
    @State private var didFinishInitialScrolling = false
    @State private var scrollOffset: CGFloat = 0
    @State private var infoViewBannerHeight: CGFloat = 0
    
    private let scrollDelayMilliseconds: Int = 500
    private let scrollId: String = "plannedActivities"
    private let scrollCoordinateSpace: String = "scroll"
        
    private var shouldAutoScroll: Bool {
        !viewModel.statusSet.planned.isEmpty
    }
    
    init(viewModel: AppointmentsViewModel = AppointmentsViewModel()) {
        _viewModel = StateObject(wrappedValue: viewModel)
    }
    
    var body: some View {
        NavigationView {
            GeometryReader { geo in
                VStack(spacing: Metrics.Spacing.space0) {
                    ScrollView {
                        DataLoadingView(
                            state: viewModel.state,
                            contentHeightForNoDataState: geo.size.height - infoViewBannerHeight,
                            retryAction: { viewModel.fetchData() },
                            content: {
                                viewContent(safeAreaTop: geo.safeAreaInsets.top)
                            }
                        )
                    }
                    .coordinateSpace(name: scrollCoordinateSpace)
                    AlertBottomView(
                        state: .info,
                        description: Localizable.appointment_info(),
                        showCloseIcon: false,
                        isSticky: true
                    )
                    .onSizeChanged(InfoViewSizeReaderKey.self, perform: { value in
                        infoViewBannerHeight = value.height
                    }
                    )
                }
            }
            .screenTracking(screenTrackingName)
            .navigationBarTitle(Text(Localizable.bottom_navigation_title_appointments()), displayMode: .automatic)
        }
        .accentColor(ColorManager.textPrimary.currentColor())
        .navigationViewStyle(StackNavigationViewStyle())
        .onAppear(perform: {
            viewModel.fetchData(force: true)
        })
    }
    
    @ViewBuilder
    private func viewContent(safeAreaTop: CGFloat) -> some View {
        ScrollViewReader { reader in
            scrollViewContent
                .onAppear {
                    if shouldAutoScroll {
                        self.scrollToFirstFutureActivityIfNeeded(reader: reader)
                    }
                }
        }
    }
    
    @ViewBuilder
    private var scrollViewContent: some View {
        VStack(spacing: Metrics.Spacing.space0) {
            if !viewModel.statusSet.finished.isEmpty {
                finishedContent
            }
            if !viewModel.statusSet.planned.isEmpty {
                plannedContent
            }
        }
    }
    
    @ViewBuilder
    private var finishedContent: some View {
        VStack(spacing: Metrics.Spacing.space0) {
            finishedHeader
            
            ActivitiesTimeline(
                entryPrefix: "finished",
                activities: viewModel.statusSet.finished,
                backgroundColor: ColorManager.bgNeutralPrimary.currentColor(),
                dotOpacity: 0.4
            )
            
            finishedFooter
        }
    }
    
    @ViewBuilder
    private var plannedContent: some View {
        VStack(spacing: Metrics.Spacing.space0) {
            plannedHeader
                .id(scrollId)
            
            ActivitiesTimeline(
                entryPrefix: "planned",
                activities: viewModel.statusSet.planned,
                backgroundColor: .clear,
                dotOpacity: 1.0
            )
        }
        .background(
            LinearGradient(
                gradient: Gradient(
                    colors: [
                        ColorManager.iconDecorativePurpleHeart.currentColor().opacity(0.1),
                        ColorManager.bgNeutralPrimary.currentColor(),
                        ColorManager.bgNeutralPrimary.currentColor(),
                        ColorManager.bgNeutralPrimary.currentColor(),
                        ColorManager.bgNeutralPrimary.currentColor()
                    ]),
                startPoint: .top,
                endPoint: .bottom
            )
            
        )
        .clipShape(
            RoundedRectangle(
                cornerRadius: Metrics.Radius.radius16,
                style: .continuous
            ))
        .background(ColorManager.bgNeutralPrimary.currentColor())
    }

    @ViewBuilder
    private var finishedHeader: some View {
        HStack {
            VStack(alignment: .leading, spacing: Metrics.Spacing.space0) {
                Text(R.string.localizable.previous_appointments_title())
                    .applyTextStyle(
                        fontStyle: .headlineS,
                        textColor: ColorManager.textPrimary.currentColor()
                    )
            }
            .padding(.horizontal, Metrics.Spacing.space16)
            .padding(.bottom, Metrics.Spacing.space24)
            .padding(.top, Metrics.Spacing.space32)
            Spacer()
        }
        .background(ColorManager.bgNeutralPrimary.currentColor())
    }

    @ViewBuilder
    private var finishedFooter: some View {
        HStack {
            VerticalLineView()
                .frame(height: Metrics.Spacing.space16)
                .padding(.leading)
            Spacer()
        }
        .background(ColorManager.bgNeutralPrimary.currentColor())
    }

    @ViewBuilder
    private var plannedHeader: some View {
            HStack(spacing: Metrics.Spacing.space0) {
                VerticalLineView()
                
                Text(R.string.localizable.future_appointments_title())
                            .applyTextStyle(
                                fontStyle: .headlineS,
                                textColor: ColorManager.textPrimary.currentColor()
                            )
                            .padding(.horizontal, Metrics.Spacing.space24)
                            .padding(.bottom, Metrics.Spacing.space24)
                            .padding(.top, Metrics.Spacing.space24)
                Spacer()

                    }
            .padding(.leading)
            .background(.clear)
        }

    private func scrollToFirstFutureActivityIfNeeded(reader: ScrollViewProxy) {
        guard !didFinishInitialScrolling else { return }

        DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(scrollDelayMilliseconds)) {
            withAnimation {
                reader.scrollTo(scrollId, anchor: .top)
            }
        }
        didFinishInitialScrolling = true
    }
}

private struct InfoViewSizeReaderKey: SizeReaderKey {
    static var defaultValue: CGSize = .zero
}

#if DEBUG
struct AppointmentsView_Previews: PreviewProvider {
    static var previews: some View {
        AppointmentsView(
            viewModel: AppointmentsViewModel.preview
        )
    }
}
#endif
