// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import MaternityServices
import GMAppData
import SwiftUI

class AppointmentsViewModel: ObservableObject {
    @DI private var userContext: UserContext
    @DI private var carePlanService: CarePlanService
    @DI var maternityRepo: MaternityRepository
    
    @Published var state = DataLoadingState.initial
    @Published var statusSet: ActivityStatusSet = .empty()
    
    private var cancelBag = CancelBag()
    
    var dateOfBirth: Date? {
        maternityRepo.selectedMaternity?.dateOfBirth
    }
    
    var week: Int? {
        maternityRepo.selectedMaternity?.week
    }
    
    func fetchData(force: Bool = false, onRefreshComplete: @escaping RefreshComplete = { }) {
        guard state.allowsDataFetch || force else {
            if state.allowsDataFetch {
                state = .noSelectedMaternityEmpty()
            }
            return
        }
        
        state = force && state.allowRefresh ? .refreshing : .loading
        carePlanService
            .fetchCarePlan()
            .map { carePlan -> ActivityStatusSet in
                carePlan?.activities?.statusSetForMaternity(
                    week: self.week ?? 1,
                    dateOfBirth: self.dateOfBirth) ?? .empty()
            }
            .receiveOnMain()
            .sink { [weak self] completion in
                switch completion {
                case .failure(let error):
                    self?.state = .error(error)
                case .finished:
                    if self?.statusSet.isEmpty == true {
                        self?.state = .empty(Localizable.careplan_activities_empty_state_title(), Localizable.careplan_activities_empty_state_description())
                    } else {
                        self?.state = .data
                    }
                }
                
                onRefreshComplete()
            } receiveValue: { [weak self] statusSet in
                self?.statusSet = statusSet
            }
            .store(in: &cancelBag)
    }
}

#if DEBUG
extension AppointmentsViewModel {
    static var preview: AppointmentsViewModel {
        let viewModel = AppointmentsViewModel()
        viewModel.statusSet = loadPreviewCarePlan()
            .activities?.statusSetForWeek(15) ?? .empty()
        viewModel.state = .data
        return viewModel
    }
}
#endif
