// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData
import UIData

struct PreviousPregnanciesView: View, ScreenTrackable, DictionaryPresenter {
    enum Section {
        case births
        case abortions
    }

    @ObservedObject private var viewModel = ViewModel()
    @State private var selectedSegment = Section.births

    var body: some View {
        BottomContainerCardView {
            VStack(spacing: 0) {
                Picker(selection: $selectedSegment, label: Text("")) {
                    Text(Localizable.journal_section_previous_births_tab_title()).tag(Section.births)
                    Text(Localizable.journal_section_previous_abortions_tab_title()).tag(Section.abortions)
                }.pickerStyle(SegmentedPickerStyle())
                .padding(.horizontal)

                switch selectedSegment {
                case .births: birthsBody
                case .abortions: abortionsBody
                }
            }
        }
        .padding(.top)
        .onAppear(perform: { viewModel.fetchData() })
        .navigationBarTitle(Text(Localizable.journal_section_around_the_pregnancy_past_pregnancies_item()), displayMode: .inline)
        .screenTracking(screenTrackingName)
    }

    private var birthsBody: some View {
        PreviousPregnanciesTabView(state: viewModel.birthState, retryAction: { viewModel.fetchData(force: true) }) {
            ForEach(viewModel.births) { (birth: Birth) in
                VStack(alignment: .leading) {
                    Text(Localizable.journal_section_previous_births_pregnancy_title(birth.year))
                        .applyTextStyle(fontStyle: .h1)
                        .padding(.bottom)
                    
                    ForEach(birth.table.indices) { (row: Int) in
                        VStack(alignment: .leading) {
                            if birth.table.count > 1 {
                                Text(Localizable.journal_section_previous_births_child_title("\(row + 1)"))
                                    .applyTextStyle(fontStyle: .h3)
                            }
                            
                            TwoColumnDataTable(
                                rows: birth.table[row].map(mapToCell)
                            )
                            
                            // If not last row, then show divider
                            if row != birth.table.count - 1 {
                                DefaultDivider()
                                    .padding(.vertical)
                            }
                        }
                        .id(row)
                    }
                    
                    ForEach(birth.info) { (keyValueModel: KeyValueModel) in
                        KeyValueView(
                            title: keyValueModel.title,
                            primaryValue: keyValueModel.primaryValue,
                            secondaryValue: keyValueModel.secondaryValue,
                            dictionaryAction:  keyValueModel.dictionaryValue == nil ? nil : {
                                showDictionaryView(
                                    title: keyValueModel.title,
                                    text: keyValueModel.dictionaryValue ?? Localizable.common_missing_data()
                                )
                            })
                            .padding(.bottom)
                            .id(keyValueModel.id)
                    }


                }
                .padding(.bottom)
                .id(birth.id)
            }
        }
    }

    private var abortionsBody: some View {
        PreviousPregnanciesTabView(state: viewModel.abortionState, retryAction: { viewModel.fetchData() }) {
            VStack(alignment: .leading) {
                ForEach(viewModel.abortions.indices) { (i) in
                    let abortion: Abortion = viewModel.abortions[i]
                    Text(Localizable.journal_section_previous_abortions_abortion_title(
                        abortion.table[0].primaryValue ?? "-")
                    )
                    .applyTextStyle(fontStyle: .h1)

                    TwoColumnDataTable(
                        rows: abortion.table.map(mapToCell)
                    )
                    .padding(.bottom)
                }
            }
            .padding(.bottom)
        }
    }

    private func mapToCell(_ keyValueModel: KeyValueModel) -> [DataTableCell] {
        [
            DataTableCell(
                id: "title-\(keyValueModel.id)",
                value: keyValueModel.title,
                type: .potentialDictionaryCellType(
                    title: keyValueModel.title,
                    dictionaryText: keyValueModel.dictionaryValue
                )
            ),

            DataTableCell(
                id: "value-\(keyValueModel.id)",
                value: keyValueModel.primaryValue ?? Localizable.common_missing_data(),
                type: .text
            )
        ]
    }
}

struct PreviousPregnanciesView_Previews: PreviewProvider {
    static var previews: some View {
        PreviousPregnanciesView()
    }
}
