// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Combine
import ErrorData
import MaternityServices
import UIData

extension JournalScansView {
    class ViewModel: ObservableObject, JournalTableContainerViewModel {
        @Published var state: DataLoadingState = .initial
        @Published var otherResults: [KeyValueModel] = []
        @Published var titleColumn: DataTableColumn = DataTableColumn(width: 0, cells: [])
        @Published var valueColumns: [DataTableColumn] = []
        @Published var shouldNavigateToActivity: Bool = false
        var activityIdForNavigation: String?

        @DI private var observationTableService: ObservationTableService
        @DI private var scanningResultsService: ScanningResultsService
        @DI private var maternityEnvironmentStore: MaternityRepository

        private var cancelBag = CancelBag()

        func fetchData(force: Bool = false, onRefreshComplete: @escaping RefreshComplete = { }) {
//            guard let maternityId = maternityEnvironmentStore.selectedMaternity?.episodeOfCareIdentifier else {
//                state = .noSelectedMaternityEmpty()
//                return
//            }
            guard force || state.allowsDataFetch else {
                return
            }

            state = force && state.allowRefresh ? .refreshing : .loading
            scanningResultsService
                .fetchOtherResults()
                .zip(observationTableService.fetchScanTable(maternityId: "12345", navigateToActivity: self.navigateToActivity))
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] (completion) in
                        switch completion {
                        case .failure(let error):
                            self?.state = .error(error)
                        case .finished:
                            self?.state = !(self?.titleColumn.cells.isEmpty ?? true) || !(self?.otherResults.isEmpty ?? true) ? .data :
                                .empty(
                                    Localizable.common_alert_title_error(),
                                    Localizable.common_alert_message_generic()
                                )
                        }
                        onRefreshComplete()
                    },
                    receiveValue: { [weak self] (otherResults: [KeyValueModel], tableDataSet: DataTableColumnSet) in
                        self?.titleColumn = DataTableColumn(width: 140, cells: tableDataSet.titleColumn)
                        self?.valueColumns = tableDataSet.valueColumns.map({ DataTableColumn(width: 100, cells: $0) })
                        self?.otherResults = otherResults
                    }
                ).store(in: &cancelBag)
        }
    }

    class PreviewViewModel: ViewModel {
        override func fetchData(force: Bool = false, onRefreshComplete: @escaping RefreshComplete = { }) {
            state = .data
            otherResults = [KeyValueModel(title: "Preview", primaryValue: "Test")]
            titleColumn = ScrollableDataTable.previewTitleColumn()
            valueColumns = ScrollableDataTable.previewColumns()
        }
    }
}
