// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI
import GMAppData

final class JournalFullscreenTablePresenter {
    private (set) var isPresented: Bool = false
    private var currentlyPresentedViewController: UIViewController?

    // Have to be a singleton to avoid getting recreated as a new instance
    static let shared = JournalFullscreenTablePresenter()

    private init() {

    }

    func present(titleColumn: DataTableColumn, valueColumns: [DataTableColumn]) {
        guard !valueColumns.isEmpty else {
            return
        }
        let safeAreaInsets = UIApplication.shared.windows.first(where: { $0.isKeyWindow })?.safeAreaInsets ?? .zero
        let leadingPadding: CGFloat
        let trailingPadding: CGFloat

        switch UIDevice.current.orientation {
        case .landscapeLeft:
            leadingPadding = safeAreaInsets.top
            trailingPadding = safeAreaInsets.bottom
        default: // .landscapeRight is preferred so we are going to right if we are forced in landscape from portrait...
            leadingPadding = safeAreaInsets.bottom
            trailingPadding = safeAreaInsets.top
        }

        let vc = SUILandscapeModalViewController {
            JournalMeasurementScrollableTable(
                valueColumns: valueColumns,
                titleColumn: titleColumn,
                tableWidth: UIScreen.main.bounds.height, // Using height since we are in landscape
                leadingColumnPadding: leadingPadding,
                trailingColumnPadding: trailingPadding,
                isFullScreen: true,
                fullScreenRequestsDismiss: dismiss
            )
        }
        vc.modalTransitionStyle = .crossDissolve
        currentlyPresentedViewController = vc
        isPresented = true
        App.topViewController?.present(vc, animated: true, completion: nil)
    }

    func dismiss() {
        currentlyPresentedViewController?.dismiss(animated: true, completion: { [weak self] in
            self?.isPresented = false
            self?.currentlyPresentedViewController = nil
        })
    }
}

//swiftlint:disable:next type_name
struct JournalFullscreenTableAutoPresenterViewModifier: ViewModifier {
    let titleColumn: DataTableColumn
    let valueColumns: [DataTableColumn]
    let presenter: JournalFullscreenTablePresenter
    @State private var isEnabled: Bool = false

    private var currentOrientation: UIDeviceOrientation {
        UIDevice.current.orientation
    }

    private var orientationChangePublisher: NotificationCenter.Publisher {
        NotificationCenter.default.publisher(for: UIDevice.orientationDidChangeNotification)
    }
    
    func body(content: Content) -> some View {
        content
            .onDisappear(perform: {
                self.isEnabled = false
            })
            .onAppear(perform: {
                self.isEnabled = true
            }) // Workaround to make it listen correctly ... 😬
            .onReceive(orientationChangePublisher, perform: { _ in
                guard isEnabled else {
                    return // nope!
                }
                if currentOrientation.isLandscape && !presenter.isPresented {
                    presenter.present(titleColumn: titleColumn, valueColumns: valueColumns)
                } else if currentOrientation == .portrait && presenter.isPresented { //Ignore upside down
                    presenter.dismiss()
                }
            })
    }
}
