// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData

struct JournalMeasurementScrollableTable: View {
    let valueColumns: [DataTableColumn]
    let titleColumn: DataTableColumn
    let tableWidth: CGFloat
    let isFullScreen: Bool
    let fullScreenRequestsDismiss: (() -> Void)?
    let leadingColumnPadding: CGFloat
    let trailingColumnPadding: CGFloat

    var scrollProgress: Double {
        max(
            0,
            Double(-scrollOffset.x) / (Double(scrollContentSize.width) - Double(scrollViewSize.width))
        )
    }

    @State private var scrollOffset: CGPoint = .zero
    @State private var scrollContentSize: CGSize = .zero
    @State private var scrollViewSize: CGSize = .zero

    init(
        valueColumns: [DataTableColumn],
        titleColumn: DataTableColumn,
        tableWidth: CGFloat,
        leadingColumnPadding: CGFloat = 0,
        trailingColumnPadding: CGFloat = 0,
        isFullScreen: Bool = false,
        fullScreenRequestsDismiss: (() -> Void)? = nil) {
        self.valueColumns = valueColumns
        self.titleColumn = titleColumn
        self.tableWidth = tableWidth
        self.leadingColumnPadding = leadingColumnPadding
        self.trailingColumnPadding = trailingColumnPadding
        self.isFullScreen = isFullScreen
        self.fullScreenRequestsDismiss = fullScreenRequestsDismiss
    }

    var body: some View {
        VStack(spacing: 0) {
            if scrollContentSize.width > scrollViewSize.width {
                ProgressBar(progress: scrollProgress)
            } else {
                DefaultDivider()
            }
            if isFullScreen {
                ZStack {
                    ScrollView(.vertical) {
                        table()
                    }
                    VStack(alignment: .trailing) {
                        HStack {
                            Spacer()
                            Button(action: fullScreenRequestsDismiss ?? { }, label: {
                                Circle()
                                    .fill(.red)
                                    .applySmallShadow()
                                    .overlay(
                                        Image(R.image.icons.basic.close.name)
                                            .renderingMode(.original)
                                            .resizable()
                                            .frame(size: 18))
                            })
                            .frame(size: 40)
                            .padding()
                        }
                        Spacer(minLength: 0)
                    }
                }
            } else {
                table()
            }
        }
    }

    @ViewBuilder
    private func table() -> some View {
        ScrollableDataTable(
            valueColumns: valueColumns,
            titleColumnRows: titleColumn,
            width: tableWidth,
            leadingColumnPadding: leadingColumnPadding,
            trailingColumnPadding: trailingColumnPadding,
            scrollOffset: $scrollOffset,
            scrollContentSize: $scrollContentSize,
            scrollViewSize: $scrollViewSize
        )
    }
}

struct JournalMeasurementScrollableTable_Previews: PreviewProvider {
    static var previews: some View {
        GeometryReader { geo in
            JournalMeasurementScrollableTable(
                valueColumns: ScrollableDataTable.previewColumns(),
                titleColumn: ScrollableDataTable.previewTitleColumn(),
                tableWidth: geo.size.width,
                isFullScreen: true
            )
        }
    }
}
