// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import CMSData
import Combine
import CMSServices

struct ArticleDetailView: View, ScreenTrackable {
    @StateObject var viewModel = ArticleDetailViewModel()

    let article : Article?
    let highlightSectionId: Int?
    
    @State private var idPublisher = PassthroughSubject<Int, Never>()
    @State private var urlPublisher = PassthroughSubject<String, Never>()

    @State private var didStartHighlightScroll: Bool = false
    @State private var didAppear: Bool = false

    @State private var isArticleDescriptionLoading: Bool = true
    @State private var articleDescriptionHeight: CGFloat = 50
    
    @State private var showSafari: Bool = false
    @State private var externalUrl: String = ""
        
    private let dateFormatter: DateFormatter

    init(article: Article?, highlightSectionId: Int? = nil) {
        self.article = article
        self.highlightSectionId = highlightSectionId
        self.dateFormatter = DateFormatter()
        self.dateFormatter.timeStyle = .none
        self.dateFormatter.dateStyle = .medium
    }
    
    var body: some View {
        ScrollView {
            ScrollViewReader { reader in
                content
                    .onAppear(perform: {
                        scrollToHighlightedSectionIfNeeded(reader: reader)
                    })
            }
            NavigationLink(
                destination: ArticleDetailView(
                    article: viewModel.linkedArticle?.article ?? article,
                    highlightSectionId: viewModel.linkedArticle?.sectionId
                ),
                isActive: $viewModel.handleInternalLink,
                label: { EmptyView() })
            
        }
        .background(ColorManager.bgNeutralPrimary.currentColor())
        .navigationBarHidden(false)
        .navigationBarBackButtonHidden(false)
        .navigationBarTitle(Text(article?.title ?? ""), displayMode: .inline)
        .screenTracking(screenTrackingName)
        .onAppear(perform: {
            self.didAppear = true
        })
        .onReceive(idPublisher, perform: { articleId in
            self.viewModel.fetchData(id: articleId)
        })
        .onReceive(urlPublisher, perform: { url in
            externalUrl = url
        })
        .onChange(of: externalUrl, perform: { url in
            if url.isEmpty == false { showSafari = true } // Only show Safari if the URL is not empty.
        })
        .onChange(of: showSafari, perform: { show in
            if show == false { externalUrl = "" } // Reset the binding when closing Safari so we can trigger onChange again with the same url if we want.
        })
        .fullScreenCover(isPresented: $showSafari, content: {
            SFSafariViewWrapper(url: URL(string: externalUrl)!)
        })
    }

    private var content: some View {
        VStack(alignment: .leading, spacing: Metrics.Spacing.space16) {
            if let article = article {
                Group {
                    Group {
                        if let content = article.content {
                            ZStack {
                                HTMLViewWithInternalLinking(
                                    idPublisher: $idPublisher,
                                    urlPublisher: $urlPublisher,
                                    dynamicHeight: $articleDescriptionHeight,
                                    isLoading: $isArticleDescriptionLoading,
                                    htmlFilePath: R.file.articleContentHtml()!,
                                    html: content,
                                    fadeInOnLoad: true,
                                    scrollEnabled: false)
                                
                                if isArticleDescriptionLoading {
                                    HStack {
                                        Spacer()
                                        ActivityIndicatorView(isAnimating: isArticleDescriptionLoading,
                                                              style: .medium,
                                                              hidesWhenStopped: true)
                                        Spacer()
                                    }
                                }
                            }
                            .frame(height: articleDescriptionHeight)
                            .padding(.vertical)
                        } else if let description = article.description { // Fallback to normal description, if there is not content: https://jira.nspop.dk/browse/DGAF-2209
                            Text(description)
                                .applyTextStyle(
                                    fontStyle: .textM,
                                    textColor: ColorManager.textPrimary.currentColor()
                                )
                                .padding(.vertical)
                        }
                    }
                    
                    ForEach(article.sections) { section in
                        ArticleSectionCard(
                            articleSection: section,
                            autoExpand: article.sections.count == 1 || highlightSectionId == section.id,
                            showExpandButton: article.sections.count > 1,
                            idPublisher: $idPublisher,
                            urlPublisher: $urlPublisher
                        )
                        .id(section.id)
                    }
                }
                .padding(.horizontal)
                // Using group to apply padding, instead of directly on the VStack, since iOS 15 scrolling is completely broken when setting padding on VStack... 🤯
            } 
        }
        .padding(.vertical)
    }
    
    private func scrollToHighlightedSectionIfNeeded(reader: ScrollViewProxy) {
        guard let highlightedId = highlightSectionId, !didStartHighlightScroll else { return }

        DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(1000)) {
            withAnimation {
                reader.scrollTo(highlightedId, anchor: .top)
            }
        }
        didStartHighlightScroll = true
    }
}

struct ArticleDetailView_Previews: PreviewProvider {
    static var previews: some View {
        NavigationView {
            ArticleDetailView(
                article: CMSData.Category.mockList().first!.articles.first!
            )
        }
    }
}
