// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import Combine
import CMSData

struct ArticleCategoryCard: View {
    let title: String
    let description: String?
    let iconUrl: URL?
    let tag: String?

    init(title: String, description: String?, iconUrl: URL?, tag: String?) {
        self.title = title
        self.description = description
        self.iconUrl = iconUrl
        self.tag = tag
    }

    init(category: CMSData.Category) {
        self.init(
            title: category.name,
            description: category.description,
            iconUrl: category.iconUrl,
            tag: category.numberOfArticlesText
        )
    }
    
    var body: some View {
        BaseCardView {
            HStack(alignment: .top, spacing: Metrics.Spacing.space16) {
                if let url = iconUrl {
                    URLImage(
                        url: iconUrl ?? URL(fileURLWithPath: ""),
                        imageView: { uiImage in
                            Image(uiImage: uiImage)
                                .renderingMode(.original)
                                .resizable()
                                .aspectRatio(contentMode: .fit)
                                .frame(size: 50 - 20)
                                .clipped()
                        },
                        placeholderView: {
                            ActivityIndicatorView(
                                isAnimating: true,
                                style: .medium,
                                hidesWhenStopped: true
                            )
                        },
                        errorView: { _ in
                            Circle().fill(Color(UIColor.lightGray))
                        }
                    )
                }
                
                VStack(alignment: .leading, spacing: Metrics.Spacing.space0) {
                    HStack(spacing: Metrics.Spacing.space0) {
                        Text(title)
                            .applyTextStyle(
                                fontStyle: FontStyle.headlineXS,
                                textColor: ColorManager.textPrimary.currentColor()
                            )
                            .multilineTextAlignment(.leading)
                            .lineLimit(2)
                        Spacer()
                        ChevronView(direction: .right)
                        
                    }
                    .padding(.bottom, Metrics.Spacing.space8)
                    
                    if let description = description {
                        Text(description)
                            .applyTextStyle(
                                fontStyle: FontStyle.textS,
                                textColor: ColorManager.textPrimary.currentColor()
                            )
                            .multilineTextAlignment(.leading)
                            .lineLimit(3)
                    }
                    
                    if let tag = tag {
                        Text(tag)
                            .applyTextStyle(
                                fontStyle: FontStyle.textXS,
                                textColor: ColorManager.textSecondary.currentColor()
                            )
                            .padding(.top, Metrics.Spacing.space4)
                    }
                }
            }
            .padding()
        }
        .applySmallShadow()
    }
}

struct ArticleCategoryCard_Previews: PreviewProvider {

    static var previews: some View {
        ScrollView {
            VStack {
                ForEach(CMSData.Category.mockList()) { category in
                    ArticleCategoryCard(category: category)
                }
            }
            .padding()
        }
        .background(ColorManager.bgNeutralPrimary.currentColor())
    }
}
