// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import CMSData
import Combine

struct ArticleSectionCard: View {
    let articleSection: ArticleSection
    let autoExpand: Bool
    let showExpandButton: Bool

    @Binding var idPublisher: PassthroughSubject<Int, Never>
    @Binding var urlPublisher: PassthroughSubject<String, Never>
    
    @State private var webViewHeight: CGFloat = 50
    @State private var isExpanded = false
    @State private var didAutoExpand = false
    @State private var isLoading = true

    var body: some View {
        ExpandableBaseCardView(
            isExpanded: $isExpanded,
            showExpandButton: showExpandButton,
            isExpandButtonEnabled: !isLoading) {
                VStack(alignment: .leading, spacing: Metrics.Spacing.space0) {
                Text(articleSection.title)
                        .applyTextStyle(
                            fontStyle: .headlineXS,
                            textColor: ColorManager.textPrimary.currentColor()
                        )

                contentView
            }
            .onReceive(Just(isLoading), perform: { loading in
                // Make sure that we only do this one time
                if !loading && autoExpand && !didAutoExpand {
                    self.isExpanded = true
                    self.didAutoExpand = true
                }
            })
        }
    }

    @ViewBuilder private var contentView: some View {
        ZStack {
            HTMLViewWithInternalLinking(
                idPublisher: $idPublisher,
                urlPublisher: $urlPublisher,
                dynamicHeight: $webViewHeight,
                isLoading: $isLoading,
                htmlFilePath: R.file.articleContentHtml()!,
                html: articleSection.content,
                fadeInOnLoad: false,
                scrollEnabled: false
            )
            .background(ColorManager.bgSurfaceNeutralPrimary.currentColor())
            if !isExpanded {
                Rectangle()
                    .fill(Color.clear)
                    .background(LinearGradient(gradient: Gradient(colors: [
                        ColorManager.bgSurfaceNeutralPrimary.currentColor(),
                        ColorManager.bgSurfaceNeutralPrimary.currentColor().opacity(0.0)
                    ]), startPoint: .bottom, endPoint: .top))
            }
            
            if isLoading {
                HStack {
                    Spacer()
                    ActivityIndicatorView(
                        isAnimating: isLoading,
                        style: .medium,
                        hidesWhenStopped: true
                    )
                    Spacer()
                }
            }
        }
        .frame(height: isExpanded ? webViewHeight : 50)
    }
}

//struct ArticleCard_Previews: PreviewProvider {
//    static var previews: some View {
//        ArticleSectionCard(
//            articleSection: Category.mockList().first!.articles.first!.sections.first!,
//            autoExpand: false
//        )
//    }
//}
