// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import ErrorData
import Combine

struct ChecklistDetailItemRow: View {
    @State var item: Checklist.Item

    @State var showDeleteAlert: Bool = false
    @State var isLoading: Bool = false
    @State var isSwiped: Bool = false
    @State var offset: CGFloat = 0

    @State private var cancelBag = CancelBag()
    
    @State var isEditing: Bool = false
    @FocusState private var isFocused: Bool
    
    var toggleItemEvent: (Checklist.Item) -> Void
    var deleteItemEvent: (Checklist.Item) -> Void
    var updateItemEvent: (Checklist.Item) -> Void

    private func toggleItem() {
        updateItemEvent(item)
        item.isChecked.toggle()
        toggleItemEvent(item)
    }
    
    private func deleteItem() {
        offset = -1000
        deleteItemEvent(item)
    }
    
    var body: some View {
        ZStack {
            LinearGradient(
                gradient: .init(
                    colors: [ColorManager.borderNeutralPrimary.currentColor()]
                ),
                startPoint: .leading,
                endPoint: .trailing
            )
                .cornerRadius(Metrics.Sizes.cardCornerRadius)
            
            HStack {
                Spacer()
                Button {
                    showDeleteAlert = true
                } label: {
                    Image(R.image.icons.basic.trash.name)
                        .frame(width: 50, height: 50)
                }
            }
            
            BaseCardView {
                HStack(spacing: Metrics.Spacing.space0) {
                    VStack(alignment: .leading) {
                        HStack {
                            Checkbox(
                                isChecked: Binding(
                                    get: { item.isChecked },
                                    set: { _ in toggleItem() }
                                )
                            )
                            .opacity(isLoading ? 0.0 : 1.0)
                            .allowsHitTesting(!isLoading)
                            .overlay(
                                ActivityIndicatorView(
                                    isAnimating: isLoading,
                                    color: ColorManager.iconInteractivePrimary.currentUIColor(),
                                    style: .medium,
                                    hidesWhenStopped: true
                                )
                                .allowsHitTesting(false)
                            )
                            if isEditing {
                                ChecklistEditItemRow(
                                    item: $item,
                                    isEditing: $isEditing,
                                    submitAction: updateItemEvent
                                )
                                .padding([.leading])
                                .frame(minHeight: 80)
                                Spacer()
                            } else {
                                Text(item.title)
                                    .applyTextStyle(
                                        fontStyle: .headlineXS,
                                        textColor: ColorManager.textPrimary.currentColor()
                                    )
                                    .lineLimit(.max)
                                    .onTapGesture {
                                        isEditing = true
                                    }
                                    .padding([.leading])
                                    .frame(minHeight: 80)
                                    Spacer()

                            }
                        }
                    }
                    .padding(.horizontal)
                }
            }
            .offset(x: offset)
            .gesture(DragGesture().onChanged(self.onChanged(value:)).onEnded(self.onEnded(value:)))
        }
        .alertOnBackground(isPresented: $showDeleteAlert, alert: {
            Alert(
                title: Text(Localizable.checklist_remove_item_alert_title()),
                message: Text(Localizable.checklist_remove_item_alert_description()),
                primaryButton: .default(
                    Text(Localizable.common_remove()),
                    action: { deleteItem() }
                ),
                secondaryButton: .cancel(
                    Text(Localizable.common_cancel()),
                    action: { offset = 0}
                )
            )
        })
    }
    
    func onChanged(value: DragGesture.Value) {
        if value.translation.width < 0 {
            if isSwiped {
                offset = value.translation.width - 50
            } else {
                offset = value.translation.width
            }
        }
    }
    
    func onEnded(value: DragGesture.Value) {
        if value.translation.width < 0 {
            if -value.translation.width > (UIScreen.main.bounds.width * 0.70) {
                offset = -1000
                deleteItemEvent(item)
            } else if -offset > 50 {
                isSwiped = true
                offset = -50
                
            } else {
                isSwiped = false
                offset = 0
            }
        } else {
            isSwiped = false
            offset = 0
        }
    }
}

struct ChecklistDetailItemRow_Previews: PreviewProvider {
    static var previews: some View {
        ChecklistDetailItemRow(item: Checklist.Item(id: "123", title: "test item", isChecked: true),
                               toggleItemEvent: {_ in },
                               deleteItemEvent: {_ in},
                               updateItemEvent: {_ in})
    }
}
