// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import Combine
import ErrorData
import UserSettingsServices


class ChecklistDetailViewModel: ObservableObject {
    @DI private var checklistRepository: ChecklistRepository
    @DI private var userContext: UserContextProtocol
    
    @Published var checklist: Checklist
    @Published var isInfoTextExpanded: Bool = false
    @Published var alertError: Error?
    @Published var swipeToDeleteTipCount: Int = 0
    @Published var isEditing: Bool = false
    
    var checklistId: Int {
        checklist.id
    }
    
    init(checklist: Checklist) {
        self.checklist = checklist
        self.swipeToDeleteTipCount = userContext.swipeToDeleteTipCount
    }
    
    func toggleCheckItem(_ item: Checklist.Item) -> ServiceError? {
        guard let listIndex = checklist.items.firstIndex(where: { $0.id == item.id }) else { return nil }
        
        checklist.items[listIndex].isChecked.toggle()
        return checklistRepository.toggleChecklistItem(checklistId: checklistId, item: item)
    }
    
    func deleteCheckItem(_ item: Checklist.Item) -> ServiceError? {
        guard let listIndex = checklist.items.firstIndex(where: { $0.id == item.id }) else { return nil }
        
        checklist.items.remove(at: listIndex)
        return checklistRepository.deleteChecklistItem(checklistId: checklistId, item: item)
    }
    
    func addCheckItem(_ item: Checklist.Item) -> ServiceError? {
        checklist.items.append(item)
        return checklistRepository.addChecklistItem(checklistId: checklistId, item: item)
    }
    
    func updateCheckItem(_ item: Checklist.Item) -> ServiceError? {
        guard let listIndex = checklist.items.firstIndex(where: { $0.id == item.id }) else { return nil }
        
        checklist.items[listIndex] = item
        return checklistRepository.updateChecklistItem(checklistId: checklistId, item: item)
    }
    
    func dismissSwipeToDeleteTip() {
        withAnimation {
            swipeToDeleteTipCount = 11
            userContext.disableSwipeToDeleteTip()
        }
    }
    
    func incrementSwipeToDeleteTip() {
        swipeToDeleteTipCount += 1
        userContext.incrementSwipeToDeleteTipCount()
    }
}
