// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI

struct KnowledgeContentView: View, ScreenTrackable {
    @ObservedObject var viewModel = KnowledgeContentViewModel()
    @StateObject var checklistViewModel = ChecklistSectionViewModel()

    let preferredNavigationBarDisplayMode: NavigationBarItem.TitleDisplayMode
    private var searchBarProvider: SearchBarProvider?
        
    init(preferredNavigationBarDisplayMode: NavigationBarItem.TitleDisplayMode) {
        self.preferredNavigationBarDisplayMode = preferredNavigationBarDisplayMode
        self.searchBarProvider = SearchBarProvider(
            searchQuery: $viewModel.searchQuery,
            searchFieldHasFocus: $viewModel.searchFieldHasFocus
        )
    }
    
    var body: some View {
        DataLoadingView(
            state: viewModel.state,
            animateStateChanges: true,
            retryAction: {
                if viewModel.isSearching {
                    self.viewModel.searchForCurrentQuery()
                } else {
                    self.viewModel.fetchData()
                }
            },
            content: {
                ScrollView {
                    if viewModel.isSearching && !viewModel.searchQuery.isEmpty {
                        KnowledgeViewSearchResultsView(
                            searchQuery: viewModel.searchQuery,
                            searchCategories: viewModel.searchResults
                        )
                        .padding()
                    } else {
                        VStack(spacing: Metrics.Spacing.space0) {
                            ChecklistSectionView(viewModel: checklistViewModel)
                            KnowledgeViewCategoriesSection(categories: viewModel.categories)
                        }
                    }
                }
            }
        )
        .overlay(
            ViewControllerResolver { (parent) in
                parent.navigationItem.searchController = searchBarProvider?.searchController
                parent.navigationItem.hidesSearchBarWhenScrolling = false
            }.frame(width: 0, height: 0)
        )
        .navigationBarTitle(
            Text(Localizable.bottom_navigation_title_knowledge()),
            displayMode: preferredNavigationBarDisplayMode
        )
        .navigationBarHidden(false)
        .navigationBarBackButtonHidden(false)
        .protectedAppearanceEvents(onAppear: { viewModel.fetchData() })
        .screenTracking(screenTrackingName, isEnabled: !viewModel.isSearching) // Should not track when updating because of searching
    }
}

struct KnowledgeContentView_Previews: PreviewProvider {
    static var previews: some View {
        KnowledgeContentView(preferredNavigationBarDisplayMode: .automatic)
    }
}
