// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import SwiftUI
import CMSServices
import CMSData
import ErrorData

final class KnowledgeContentViewModel: ObservableObject {
    @DI private var articlesService: ArticlesService
    @DI var userContext: UserContextProtocol
    @DI private var logger: Logger
    private var cancelBag: CancelBag = CancelBag()
    
    @Published var categories: [CMSData.Category] = []
    @Published private(set) var state: DataLoadingState = .initial
    
    @Published var searchResults: [SearchResultDomain] = []
    @Published var searchQuery: String = ""
    @Published var searchFieldHasFocus: Bool = false
    var hasData: Bool {
        !categories.isEmpty
    }
    var isSearching: Bool {
        !searchQuery.isEmpty || searchFieldHasFocus
    }
    
    init() {
        listenForSearchEvents()
    }
    
    func fetchData(force: Bool = false, onRefreshComplete: @escaping RefreshComplete = { }) {
        guard force && !isSearching || state.allowsDataFetch && !isSearching else {
            return
        }
        
        state = force && state.allowRefresh ? .refreshing : .loading
        
        articlesService.fetchArticleCategories()
            .receiveOnMain()
            .sink(
                receiveCompletion: { [weak self] completion in
                    switch completion {
                    case .failure(let error):
                        self?.state = .error(error)
                    default: break
                    }
                    
                    onRefreshComplete()
                },
                receiveValue: { [weak self] categories in
                    self?.categories = categories
                    self?.updateStateForData()
                })
            .store(in: &cancelBag)
    }
    
    private func listenForSearchEvents() {
        // Reduce duplicates and typing speed for direct searchQuery input
        let publisher = $searchQuery
            .removeDuplicates()
            .debounce(for: 0.2, scheduler: DispatchQueue.main)
            .eraseToAnyPublisher()
        
        search(queryPublisher: publisher)
    }
    
    /// Force searches for current query (designed for when in error state)
    func searchForCurrentQuery() {
        search(queryPublisher: Just(searchQuery).eraseToAnyPublisher())
    }
    
    private func updateStateForData() {
        self.state = hasData ? .data :
            .empty(
                Localizable.common_alert_title_error(),
                Localizable.common_alert_message_failed_to_load_data()
            )
    }
    
    /// Perform search based on a query publisher
    private func search(queryPublisher: AnyPublisher<String, Never>) {
        queryPublisher
            .receiveOnMain()
            .handleEvents(receiveOutput: { (query) in
                if query.isEmpty && self.state != .loading {
                    self.searchResults = []
                    self.updateStateForData()
                }
            })
            .filter({ !$0.isEmpty })
            .setFailureType(to: ServiceError.self)
            .flatMap({ (query: String) -> AnyPublisher<[SearchResultDomain], ServiceError> in
                self.state = .loading
                return self.articlesService.search(query: query)
                    .receiveOnMain()
                // Catch errors to avoid death of publisher ☠️
                    .catch({ [weak self] (error: ServiceError) -> Empty<[SearchResultDomain], ServiceError> in
                        self?.state = .error(error)
                        return Empty()
                    })
                    .eraseToAnyPublisher()
            })
            .replaceError(with: [])
            .sink(receiveValue: { [weak self] (results) in
                self?.searchResults = results
                // The user might cancel the search (=> searchQuery.isEmpty) during load - then fallback to data view
                if results.isEmpty && self?.searchQuery.isEmpty == false {
                    self?.state = .empty(Localizable.search_no_results_title(), Localizable.search_no_results_description())
                } else {
                    self?.state = .data
                }
            })
            .store(in: &cancelBag)
    }
}
