// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import CMSData
import Combine
import ErrorData
import UIKit

class SettingsPageViewModel: ObservableObject {
    let title: String
    @Published var htmlContent: String = ""
    @Published var htmlViewHeight: CGFloat = 0.0
    @Published var state: DataLoadingState = .initial
    @Published var isWebViewLoading: Bool = false
    @Published var error: Error?
    
    let pageFetcher: AnyPublisher<Page, ServiceError>
    
    @DI private var userContext: UserContextProtocol
    private var cancelBag = CancelBag()
    
    init(title: String, pageFetcher: AnyPublisher<Page, ServiceError>) {
        self.title = title
        self.pageFetcher = pageFetcher
    }
    
    func fetchData() {
        guard state.allowsDataFetch else {
            return
        }
        state = .loading
        pageFetcher
            .receiveOnMain()
            .sink { [weak self] (completion) in
                switch completion {
                case .failure(let error):
                    self?.state = .error(error)
                case .finished: break
                }
            } receiveValue: { [weak self] (page) in
                self?.htmlContent = page.content
                self?.state = .data
            }
            .store(in: &cancelBag)
        
    }
    
    func removeConsent() {
        userContext.resetAllUserData()
    }
}

#if DEBUG
extension SettingsPageViewModel {
    static func preview() -> SettingsPageViewModel {
        SettingsPageViewModel(
            title: "Test",
            pageFetcher: Just(
                Page(
                    title: "Test",
                    content: "Test",
                    excerpt: nil,
                    slug: nil
                )
            )
            .setFailureType(to: ServiceError.self)
            .eraseToAnyPublisher()
        )
    }
}
#endif
