// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import Combine
import CMSData
import ErrorData

struct SettingsView: View, ScreenTrackable {
    @StateObject private var viewModel = SettingsViewModel()

    private let rowHeight: CGFloat = Metrics.Sizes.navigationRowHeight

    var body: some View {
        ScrollView {
            CompatibleVStack {
                CircleImageHeaderView(
                    title: viewModel.title,
                    text: Localizable.more_settings_description(),
                    imageUrl: nil,
                    image: Image(R.image.icons.more.settings.name)
                )
                    .padding(.bottom)

                // MARK: - Privacy
                Section(
                    header: sectionHeader(title: Localizable.settings_privacy_title()),
                    content: {
                        SettingsSectionContentCard {
                            navigationRowForPage(
                                title: Localizable.settings_data_protection_title(),
                                description: Localizable.settings_data_protection_description(),
                                publisher: viewModel.pagesServices.fetchPrivacyPolicy(),
                                insertDivider: false
                            )
                        }
                    }
                )
                
                // MARK: - Security
                    Section(
                        header: sectionHeader(title: Localizable.settings_security_title()),
                        content: {
                            SettingsSectionContentCard {
                                toggleRow(
                                    title: Localizable.settings_protect_app_title(),
                                    description: nil,
                                    binding: Binding(get: { viewModel.isAuthentiocationToggleSwitchOn }, set: { on in
                                        viewModel.isAuthentiocationToggleSwitchOn = on
                                        if !viewModel.isAuthenticationAvailable {
                                            viewModel.showAuthenticationUnavailableAlert = true
                                        } else if on {
                                            viewModel.showEnableAuthenticationAlert = true
                                        } else {
                                            viewModel.showDisableAuthenticationAlert = true
                                        }
                                    }),
                                    insertDivider: false
                                )
                            }
                        }
                    )
                // MARK: - About the app
                Section(
                    header: sectionHeader(title: Localizable.settings_about_the_app_title()),
                    content: {
                        SettingsSectionContentCard {
                            navigationRowForPage(
                                title: Localizable.settings_read_about_the_app_title(),
                                description: nil,
                                publisher: viewModel.pagesServices.fetchAboutTheApp()
                            )
                            navigationRowForPage(
                                title: Localizable.settings_who_is_behind_the_app_title(),
                                description: nil,
                                publisher: viewModel.pagesServices.fetchBehindTheApp()
                            )
                            navigationRow(
                                title: Localizable.settings_consent_title(),
                                description: nil,
                                destination: SettingsConsentPageView(
                                    viewModel: SettingsPageViewModel(
                                        title: Localizable.settings_consent_title(),
                                        pageFetcher: viewModel.pagesServices.fetchConsent()
                                    )
                                ),
                                insertDivider: false
                            )
                        }
                    }
                )

                Text(Localizable.settings_version(viewModel.appVersion, viewModel.appBuildNumber))
                    .applyTextStyle(
                        fontStyle: .textXS,
                        textColor: ColorManager.textSecondary.currentColor()
                        )
            }
            .padding()
        }
        .navigationBarTitle(Text(viewModel.title), displayMode: .inline)
        .screenTracking(screenTrackingName)
        .onAppear(perform: {
            viewModel.updateValues()
        })
        .alertOnBackground(isPresented: $viewModel.showEnableAuthenticationAlert, alert: {
            enableAuthenticationAlert()
        })
        
        .alertOnBackground(isPresented: $viewModel.showDisableAuthenticationAlert, alert: {
            disableAuthenticationAlert()
        })
        .alertOnBackground(isPresented: $viewModel.showAuthenticationUnavailableAlert, alert: {
            authenticationNotAvailableAlert()
        })
    }

    @ViewBuilder
    private func navigationRowForPage(title: String, description: String?, publisher: AnyPublisher<Page, ServiceError>, insertDivider: Bool = true) -> some View {
        navigationRow(
            title: title,
            description: description,
            destination: SettingsPageView(
                viewModel: SettingsPageViewModel(
                    title: title,
                    pageFetcher: publisher
                )
            ),
            insertDivider: insertDivider
        )
    }

    @ViewBuilder
    private func navigationRow<T: View>(title: String, description: String?, destination: T, insertDivider: Bool = true) -> some View {
        NavigationLink(
            destination: destination,
            label: {
                ListItemNavigationView(
                    title: title,
                    description: description
                )
                .frame(minHeight: rowHeight)
            })
        if insertDivider {
            DefaultDivider()
        }
    }

    @ViewBuilder
    private func sectionHeader(title: String) -> some View {
        HStack {
            Text(title)
                .applyTextStyle(fontStyle: .headlineM)
            Spacer(minLength: 0)
        }
    }

    @ViewBuilder
    private func toggleRow(title: String, description: String?, binding: Binding<Bool>, insertDivider: Bool = true) -> some View {
        ListItemToggleView(
            title: title,
            description: description,
            isOn: binding
        )
        .frame(minHeight: rowHeight)
        if insertDivider {
            DefaultDivider()
        }
    }

    private func enableAuthenticationAlert() -> Alert {
        Alert(
            title: Text(Localizable.settings_protect_app_prompt_add_title()),
            message: Text(Localizable.settings_protect_app_prompt_add_description(viewModel.authentiocationName)),
            primaryButton: .default(
                Text(Localizable.ios_alert_login_cta_use(viewModel.authentiocationName)),
                action: {
                    viewModel.enableAuthentication { success in
                        if success {
                            viewModel.InitiateAuthenticationEnabling()
                        }
                    }
                }),
            secondaryButton: .cancel(
                Text(Localizable.common_cancel()),
                action: {
                    withAnimation(.default) {
                        viewModel.isAuthentiocationToggleSwitchOn = viewModel.isAuthentiocationEnabled // reset
                    }
                })
        )
    }
        
    private func disableAuthenticationAlert() -> Alert {
        Alert(
            title: Text(Localizable.settings_protect_app_prompt_remove_title()),
            message: Text(Localizable.settings_protect_app_prompt_remove_description(viewModel.authentiocationName)),
            primaryButton: .default(
                Text(Localizable.common_remove()),
                action: {
                    viewModel.disableAuthentication { success in
                        if success {
                            viewModel.InitiateAuthenticationEnabling()
                        }
                    }
                }),
            secondaryButton: .cancel(
                Text(Localizable.common_cancel()),
                action: {
                    withAnimation(.default) {
                        viewModel.isAuthentiocationToggleSwitchOn = viewModel.isAuthentiocationEnabled // reset
                    }
                })
        )
    }
    
    private func authenticationNotAvailableAlert() -> Alert {
        Alert(
            title: Text(Localizable.settings_protect_app_enroll_dialog_title()),
            message: Text(Localizable.settings_protect_app_enroll_dialog_description()),
            primaryButton: .default(
                Text(Localizable.settings_protect_app_enroll_dialog_confirm_button_title()),
                action: {
                    viewModel.isAuthentiocationToggleSwitchOn = viewModel.isAuthentiocationEnabled // reset
                    UIApplication.shared.open(URL(string: UIApplication.openSettingsURLString)!, options: [:], completionHandler: nil)
                }),
            secondaryButton: .cancel(
                Text(Localizable.common_regret()),
                action: {
                    withAnimation(.default) {
                        viewModel.isAuthentiocationToggleSwitchOn = viewModel.isAuthentiocationEnabled // reset
                    }
                })
        )
    }
}

struct SettingsView_Previews: PreviewProvider {
    static var previews: some View {
        SettingsView()
    }
}
