// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData

struct OverviewView: View, ScreenTrackable {
    @DI private var logger: Logger
    
    @StateObject var viewModel = OverviewViewModel()
    @State private var scrollOffset: CGPoint = .zero
    
    var navigationBarHidden: Bool { true }
    
    var body: some View {
        NavigationView {
            ScrollView {
                VStack(spacing: Metrics.Spacing.space16) {
                    welcomeHeaderView
                        .padding(Metrics.Spacing.space0)
                    headerView
                        .padding(.bottom, Metrics.Spacing.space24)
                    if viewModel.showArticlesSection {
                        DataLoadingView(
                            state: viewModel.state,
                            retryAction: { viewModel.fetchData() },
                            content: {
                                VStack(spacing: Metrics.Spacing.space20) {
                                    if viewModel.showCurrentWeekArticles {
                                        OverviewKnowledgeSectionView(
                                            title: viewModel.currentRelevantArticleTitle,
                                            actionTitle: Localizable.overview_find_more_knowledge_button(),
                                            action: {
                                                NotificationCenter.default.post(notification: .TabBarController.changeTab, value: TabBarController.ChangeTabNotificationMessage.knowledge)
                                            },
                                            categories: viewModel.currentRelevantArticleResults
                                        )
                                    }
                                    if viewModel.showNextWeekArticles {
                                        OverviewKnowledgeSectionView(
                                            title: Localizable.overview_relevant_next_week_title(),
                                            actionTitle: nil,
                                            action: nil,
                                            categories: viewModel.nextRelevantArticleResults
                                        )
                                        .padding(.top, Metrics.Spacing.space8)
                                    }
                                }
                            }
                        )
                    }
                }
                .padding(Metrics.Spacing.space16)
                .background(
                    VStack(spacing: Metrics.Spacing.space0) {
                        BubblesBackgroundView(size: UIScreen.main.bounds.size)
                        Spacer()
                    }
                )
                NavigationLink(
                    destination: KnowledgeContentView(preferredNavigationBarDisplayMode: .inline)
                        .navigationBarHidden(false)
                        .navigationBarBackButtonHidden(false),
                    isActive: $viewModel.pushKnowledgeSection,
                    label: {
                        EmptyView()
                    }
                )
            }
            .overlay(
                GeometryReader { geo in
                    VStack(spacing: Metrics.Spacing.space0) {
                        VisualBlurEffectView(style: .systemChromeMaterial)
                            .frame(height: geo.safeAreaInsets.top)
                        DefaultDivider()
                        Spacer(minLength: 0)
                    }
                    .opacity(Double(max(min(-scrollOffset.y / geo.safeAreaInsets.top, 1.0), 0.0)))
                    .edgesIgnoringSafeArea(.top)
                }
            )
            .navigationBarTitle("", displayMode: .inline)
            .navigationBarHidden(navigationBarHidden)
            .navigationBarBackButtonHidden(navigationBarHidden)
            .screenTracking(screenTrackingName)
            .onAppear(perform: { viewModel.fetchData() })
            .sheetOnBackground(
                isPresented: $viewModel.presentSetDateOfBirthModal,
                content: {
                    OverviewSetDateOfBirthView(
                        viewModel: viewModel.datefBirthViewModel
                    )
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentUpdateDateOfBirthModal,
                content: {
                    OverviewUpdateDateOfBirthView(
                        viewModel: viewModel.datefBirthViewModel
                    )
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentSetDateOfDeliveryModal,
                content: {
                    OverviewSetDateOfDeliveryView(
                        viewModel: viewModel.dateOfDeliveryViewModel,
                        title: Localizable.overview_pregnancy_select_expected_delivery_date_title(),
                        description: Localizable.overview_select_expected_delivery_date_description(),
                        primaryButtonTitle: Localizable.overview_pregnancy_select_expected_delivery_date_button_title(),
                        secondaryButtonTitle: Localizable.overview_pregnancy_calculate_expected_delivery_date_button_title(),
                        secondaryButtonAction: {
                            viewModel.presentCalculateDateOfDeliveryModal = true
                        }
                    )
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentUpdateDateOfDeliveryModal,
                content: {
                    OverviewUpdateDateOfDeliveryView(
                        viewModel: viewModel.dateOfDeliveryViewModel,
                        action: {
                            viewModel.didPressEndPregnancy()
                        }
                    )
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentCalculateDateOfDeliveryModal,
                content: {
                    OverviewCalculateDateOfDeliveryView(
                        viewModel: viewModel.calculateDateOfDeliveryViewModel
                    )
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentEndPregnancyEarlyModal,
                content: {
                    EndPregnancyEarlyView()
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentEndPregnancyModal,
                content: {
                    EndPregnancyView(
                        action: { viewModel.presentSetDateOfBirthModal = true }
                    )
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentStartNewPregnancyModal,
                content: {
                    OverviewSetDateOfDeliveryView(
                        viewModel: viewModel.dateOfDeliveryViewModel,
                        title: Localizable.create_new_pregnancy_title(),
                        description: Localizable.overview_select_expected_delivery_date_description(),
                        primaryButtonTitle: Localizable.create_new_pregnancy_btn_title(),
                        secondaryButtonTitle: Localizable.overview_pregnancy_calculate_expected_delivery_date_button_title(),
                        secondaryButtonAction: {
                            viewModel.presentCalculateDateOfDeliveryModal = true
                        }
                    )
                }
            )
            .alertOnBackground(isPresented: $viewModel.presentNewPregnancyAlert, alert: {
                Alert(
                    title: Text(Localizable.common_are_you_sure()),
                    message: Text(Localizable.overview_start_new_pregnancy_alert_description()),
                    primaryButton: .default(Text(Localizable.common_create()), action: {viewModel.startNewPregnancy()}),
                    secondaryButton: .cancel(Text(Localizable.common_regret()))
                )
            })
        }
        .accentColor(ColorManager.textPrimary.currentColor())
        .navigationViewStyle(StackNavigationViewStyle())
        .onChange(of: viewModel.week) { _ in
            viewModel.fetchData(force: true)
        }
    }
    
    @ViewBuilder
    private var welcomeHeaderView: some View {
        OverviewWelcomeHeaderView(
            title: viewModel.navigationBarTitle,
            subtitle: viewModel.navigationBarSubtitle,
            action: viewModel.headerViewButtonAction
        )
    }
    
    @ViewBuilder
    private var headerView: some View {
        switch viewModel.pregnancyState {
        case .notPregnant:
            OverviewNotPregnantHeaderView(
                presentTermModal: $viewModel.presentSetDateOfDeliveryModal
            )
        case .pregnant:
            OverviewPregnantHeaderView(
                monthsIntoPregnancy: viewModel.montsIntoPregnancy,
                dueDate: viewModel.dueDate,
                month: viewModel.month,
                week: viewModel.week,
                gestationAge: viewModel.gestationAge,
                trimester: viewModel.trimester,
                numberOfFetuses: viewModel.numberOfFetuses,
                endPregnancyButtonPressed: $viewModel.presentSetDateOfBirthModal,
                updatePregnancyButtonPressed: $viewModel.presentUpdateDateOfDeliveryModal
            )
        case .postPregnant:
            if let date = viewModel.dateOfBirth {
                OverviewPostPregnantHeaderView(
                    dateOfBirth: date,
                    numberOfFetuses: 1,
                    presentUpdateDateOfBirthModal: $viewModel.presentUpdateDateOfBirthModal
                )
            } else {
                EmptyView()
                    .onAppear(perform: {
                        logger.error("Overview is requesting header view for `.postPregnang`, but does not have a birth date.")
                    })
            }
        }
    }
    
    struct OverviewView_Previews: PreviewProvider {
        static var previews: some View {
            OverviewView(
                viewModel: OverviewPreviewViewModel()
            )
        }
    }
}
