// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import MaternityServices
import GMAppData
import CMSServices
import CMSData
import ErrorData
import Combine

class OverviewViewModel: ObservableObject {
    private var cancelBag = CancelBag()
    @DI private var maternityRepo: MaternityRepository
    @DI private var articlesService: ArticlesService
    @DI private var userContext: UserContextProtocol
    
    @Published var state: DataLoadingState = .initial
    
    @Published var presentSetDateOfBirthModal: Bool = false
    @Published var presentUpdateDateOfBirthModal: Bool = false
    
    @Published var presentEndPregnancyModal: Bool = false
    @Published var presentEndPregnancyEarlyModal: Bool = false
    
    @Published var presentStartNewPregnancyModal: Bool = false
    
    @Published var presentSetDateOfDeliveryModal: Bool = false
    @Published var presentUpdateDateOfDeliveryModal: Bool = false
    @Published var presentCalculateDateOfDeliveryModal: Bool = false
    
    @Published var pushKnowledgeSection: Bool = false
    @Published var shouldReloadData: Bool = false
    
    @Published var presentNewPregnancyAlert: Bool = false
    
    @Published var currentRelevantArticleResults: [CMSData.Category] = []
    @Published var nextRelevantArticleResults: [CMSData.Category] = []
    
    var navigationBarTitle: String {
        Localizable.overview_welcome_message_unauthenticated()
    }
    
    var navigationBarSubtitle: String {
        Localizable.bottom_navigation_title_overview()
    }
    
    var hasSelectedMaterinity: Bool {
        maternityRepo.selectedMaternity != nil
    }
    
    var headerViewButtonAction: (() -> Void)? {
        dateOfBirth != nil ? { self.presentNewPregnancyAlert = true } : nil
    }
    
    private var currentWeekForArticles: Int {
        guard let week = maternityRepo.selectedMaternity?.week else {
            return 1
        }
        return week
    }
    
    let datefBirthViewModel = UpdateDateOfBirthViewModel()
    let dateOfDeliveryViewModel = UpdateDateOfDeliveryViewModel()
    
    let calculateDateOfDeliveryViewModel = OverviewCalculateDateOfDeliveryViewModel()
    
    var pregnancyState: OverviewPregnancyState {
        if maternityRepo.selectedMaternity == nil {
            return .notPregnant
        } else if dateOfBirth != nil {
            return .postPregnant
        } else {
            return .pregnant
        }
    }
    
    var numberOfFetuses: Int { 1 } // TODO will v2 support multiple fetuses?
    
    var trimester: Int { maternityRepo.selectedMaternity?.trimester ?? 0 }
    
    var dateOfBirth: Date? { maternityRepo.selectedMaternity?.dateOfBirth }
    
    var gestationAge: String {
        maternityRepo.selectedMaternity?.gestationAge ?? Localizable.common_missing_data()
    }
    
    var week: Int {
        maternityRepo.selectedMaternity?.week ?? 0
    }
    
    var month: Int {
        Int(maternityRepo.selectedMaternity?.monthsIntoPregnancy ?? 0) + 1
    }
    
    var dueDate: Date? {
        maternityRepo.selectedMaternity?.dateOfDelivery
    }
    
    var montsIntoPregnancy: Double {
        maternityRepo.selectedMaternity?.monthsIntoPregnancy ?? 0
    }
    
    var showArticlesSection: Bool {
        showCurrentWeekArticles || showNextWeekArticles
    }
    
    var showCurrentWeekArticles: Bool {
        currentRelevantArticleResults.hasSearchResults
    }
    
    var showNextWeekArticles: Bool {
        nextRelevantArticleResults.hasSearchResults && pregnancyState != .notPregnant
    }
    
    var currentRelevantArticleTitle: String {
        pregnancyState == .notPregnant ? Localizable.overview_selected_topics_title() : Localizable.overview_relevant_right_now_title()
    }
    
    init() {
        listenForReloadEvents()
    }
    
    private func listenForReloadEvents() {
        datefBirthViewModel.$didSave
            .sink { [weak self] reload in
                if reload {
                    self?.fetchData(force: true)
                }
            }
            .store(in: &cancelBag)
        
        dateOfDeliveryViewModel.$didSave
            .sink { [weak self] reload in
                if reload {
                    self?.fetchData(force: true)
                }
            }
            .store(in: &cancelBag)
        
        calculateDateOfDeliveryViewModel.$didSave
            .sink { [weak self] reload in
                if reload {
                    self?.fetchData(force: true)
                }
            }
            .store(in: &cancelBag)
    }
    
    func fetchData(force: Bool = false, onRefreshComplete: @escaping RefreshComplete = { }) {
        guard (state.allowsDataFetch && !state.isEmpty) || force else {
            return
        }
        
        state = force && state.allowRefresh ? .refreshing : .loading
        
        let categoriesRequest: AnyPublisher<CMSData.RecommendedCategories, ServiceError>
        
        switch pregnancyState {
        case .notPregnant:
            let weekNumber = Int.random(in: 1...10)
            categoriesRequest = articlesService.fetchArticleCategories(for: weekNumber)
        case .postPregnant:
            let maximumOffset = 8
            let dateOfBirthWeekOffset = dateOfBirth?.weekNumberAfterBirthDate() ?? 1
            categoriesRequest = dateOfBirthWeekOffset > maximumOffset
            ? articlesService.fetchAfterBirthArticleCategories(for: maximumOffset)
            : articlesService.fetchAfterBirthArticleCategories(for: dateOfBirthWeekOffset)
        case .pregnant:
            let maximumWeekforArticles = 43
            categoriesRequest = currentWeekForArticles > 43
            ? articlesService.fetchArticleCategories(for: maximumWeekforArticles)
            : articlesService.fetchArticleCategories(for: currentWeekForArticles)
        }
        
        categoriesRequest
            .receiveOnMain()
            .sink(
                receiveCompletion: { [weak self] completion in
                    switch completion {
                    case .failure(let error):
                        self?.state = .error(error)
                    case .finished:
                        self?.state = .data
                    }
                    onRefreshComplete()
                },
                receiveValue: { [weak self] (categories) in
                    self?.currentRelevantArticleResults = categories.currentWeek
                    self?.nextRelevantArticleResults = categories.nextWeek ?? []
                }
            )
            .store(in: &cancelBag)
    }
    
    func didPressEndPregnancy() {
        let week = maternityRepo.selectedMaternity?.week ?? 0
        if week > 22 {
            return presentEndPregnancyModal = true
        }
        return presentEndPregnancyEarlyModal = true
    }
    
    func startNewPregnancy() {
        userContext.clearEnvironmentStoreStates()
        presentStartNewPregnancyModal = true
    }
}

final class OverviewPreviewViewModel: OverviewViewModel {
    override func fetchData(force: Bool = false, onRefreshComplete: @escaping RefreshComplete = { }) {
        currentRelevantArticleResults = Array(CMSData.Category.mockList().prefix(2))
        nextRelevantArticleResults = [ CMSData.Category.mockList().last! ]
        state = .data
    }
}

