// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import SwiftUI

final class UpdateDateOfBirthViewModel: ObservableObject {
    @DI private var maternityRepo: MaternityRepository
    @DI private var logger: Logger

    private var cancelBag = CancelBag()

    @Published var date: Date!
    @Published var isLoadingReset: Bool = false
    @Published var isLoadingSave: Bool = false
    @Published var error: Error?
    @Published var showResetAlert: Bool = false
    @Published var didSave: Bool = false

    var dateOfBirth: Date? {
        maternityRepo.selectedMaternity?.dateOfBirth
    }

    init() {
        self.reset()
    }

    func lowerBound() -> Date {
        maternityRepo.selectedMaternity?.offsetDate ?? Date()
    }

    func upperBound() -> Date {
        let maximulDateOfDelivery = maternityRepo.selectedMaternity?.dateOfDelivery?.date(byAdding: .day, value: 14) ?? Date()
        let today = Date()
        return today > maximulDateOfDelivery ? maximulDateOfDelivery : today
    }

    func dateOfBirthClosedRange() -> ClosedRange<Date> {
        lowerBound()...upperBound()
    }

    func reset() {
        if let date = dateOfBirth {
            self.date = date
        } else {
            self.date = Date()
        }
        self.didSave = false
        self.isLoadingSave = false
        self.isLoadingReset = false
        self.showResetAlert = false
        self.error = nil
    }

    func saveSelectedDate() {
        isLoadingSave = true
        saveDate(date: date)
    }

    func resetDate() {
        isLoadingReset = true
        saveDate(date: nil)
    }

    func saveDate(date: Date?) {
        maternityRepo
            .changeDateOfBirthForSelectedMaternity(date)
            .receiveOnMain()
            .sink(
                receiveCompletion: { [weak self] completion in
                    switch completion {
                    case .failure(let error):
                        self?.error = error
                        self?.didSave = false
                    case .finished:
                        self?.didSave = true
                    }
                    self?.isLoadingReset = false
                    self?.isLoadingSave = false
                },
                receiveValue: { _ in }
            )
            .store(in: &cancelBag)
    }
}
