// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Combine
import ConfigurationServices
import ErrorData
import GMAppData
import SwiftUI

final class OverviewCalculateDateOfDeliveryViewModel: ObservableObject {
    @DI private var maternityRepo: MaternityRepository
    @DI private var logger: Logger
    
    @Published var date: Date!
    @Published var isLoadingSave: Bool = false
    @Published var error: Error?
    @Published var didSave: Bool = false
    
    private var cancelBag = CancelBag()
    
    var primaryButtonText: String {
        Localizable.overview_pregnancy_calculate_expected_delivery_date_button_title()
    }
    
    var navigationBarTitleText: String {
        Localizable.overview_pregnancy_calculate_expected_delivery_date_title()
    }
    
    var descriptionText: String {
        Localizable.overview_select_calculated_expected_due_date_description()
        
    }
    
    var lowerBound: Date { Date().date(byAdding: .day, value: -280) ?? Date() }
    
    var upperBound: Date { Date() }
    
    var dateOfDeliveryClosedRange: ClosedRange<Date> { lowerBound...upperBound }
    
    init() {
        self.reset()
    }
    
    func reset() {
        self.date = Date()
        self.didSave = false
        self.isLoadingSave = false
        self.error = nil
    }
    
    func saveSelectedDate() {
        isLoadingSave = true
        saveDate(date: date.date(byAdding: .day, value: 280))
    }
    
    private func saveDate(date: Date?) {
        guard let date = date else { return }
        let maternity = SelectedMaternity(dateOfDelivery: date)
        
        maternityRepo
            .changeSelectedMaternity(maternity)
            .receiveOnMain()
            .sink(
                receiveCompletion: { [weak self] completion in
                    switch completion {
                    case .failure(let error):
                        self?.error = error
                        self?.didSave = false
                    case .finished:
                        self?.didSave = true
                    }
                    self?.isLoadingSave = false
                },
                receiveValue: { _ in }
            )
            .store(in: &cancelBag)
    }
}
