// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct OverviewSetDateOfDeliveryView: View, ScreenTrackable {
    @ObservedObject var viewModel: UpdateDateOfDeliveryViewModel
    @Environment(\.presentationMode) var presentationMode
    
    let title: String
    let description: String
    let primaryButtonTitle: String
    let secondaryButtonTitle: String
    let secondaryButtonAction: (() -> Void)
    
    init(
        viewModel: UpdateDateOfDeliveryViewModel,
        title: String,
        description: String,
        primaryButtonTitle: String,
        secondaryButtonTitle: String,
        secondaryButtonAction: @escaping () -> Void
    ) {
        self.viewModel = viewModel
        self.title = title
        self.description = description
        self.primaryButtonTitle = primaryButtonTitle
        self.secondaryButtonTitle = secondaryButtonTitle
        self.secondaryButtonAction = secondaryButtonAction
    }
    
    var body: some View {
        NavigationView {
            VStack(
                alignment: .leading,
                spacing: Metrics.Spacing.space12
            ) {
                Text(description)
                    .applyTextStyle(fontStyle: .textM)
                    .lineLimit(.max)
                GeneralDatePicker(
                    selectedDate: $viewModel.date,
                    closedRange: viewModel.dateOfDeliveryClosedRange()
                )
                Spacer()
                ButtonPrimary(
                    title: primaryButtonTitle,
                    action: {
                        viewModel.saveSelectedDate()
                    }
                )
                ButtonPrimarySubtle(
                    title: secondaryButtonTitle,
                    action: {
                            presentationMode.wrappedValue.dismiss()
                            secondaryButtonAction()
                    }
                )
                .padding(.bottom, Metrics.Spacing.space8)
            }
            .padding()
            .navigationBarTitle(
                Text(title),
                displayMode: .inline
            )
            .navigationBarItems(
                trailing: CloseModalButton(
                    action: {
                        presentationMode.wrappedValue.dismiss()
                    }
                )
            )
            .screenTracking(screenTrackingName)
            .onReceive(viewModel.$didSave, perform: { didSave in
                if didSave {
                    presentationMode.wrappedValue.dismiss()
                }
            })
            .alert(for: $viewModel.error)
            .protectedAppearanceEvents(
                onAppear: viewModel.reset,
                onDisappear: viewModel.reset
            )
        }
        .accentColor(ColorManager.iconPrimary.currentColor())
    }
}

struct OverviewSetDateOfDeliveryView_Previews: PreviewProvider {
    static var previews: some View {
        OverviewSetDateOfDeliveryView(
            viewModel: UpdateDateOfDeliveryViewModel(),
            title: Localizable.overview_pregnancy_select_expected_delivery_date_title(),
            description: Localizable.overview_select_expected_delivery_date_description(),
            primaryButtonTitle: Localizable.overview_pregnancy_select_expected_delivery_date_button_title(),
            secondaryButtonTitle: Localizable.overview_pregnancy_calculate_expected_delivery_date_button_title(),
            secondaryButtonAction: { }
        )
    }
}
