// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import Rswift

struct OverviewPregnancyProgressView: View {
    let monthsIntoPregnancy: Double
    let beginAnimation: Bool
    let numberOfFetuses: Int
    private let targetMonths: Double = 9
    
    private let padding: CGFloat = 5
    private let lineWidth: CGFloat = 15
    
    private var fetusScale: CGFloat {
        guard numberOfFetuses > 1 else {
            return 1.0
        }
        return max(1.0 - CGFloat(numberOfFetuses - 1) * 0.25, 0.5)
    }
    
    var body: some View {
        ZStack {
            CircularProgressBar(
                progress: monthsIntoPregnancy / targetMonths,
                lineWidth: lineWidth,
                padding: padding,
                backgroundColor: ColorManager.bgSurfaceNeutralSecondary.currentColor()
            )
            InnerShadowCircleView {
                ZStack {
                    LinearGradient(
                        gradient: Gradient(
                            colors: [
                                R.color.babyGradientTop.color(),
                                R.color.babyGradientBottom.color()
                            ]),
                        startPoint: .top,
                        endPoint: .bottom
                    )
                    .clipShape(Circle())
                    Group {
                        ForEach(0 ..< numberOfFetuses) { _ in
                            Image(currentImage.name)
                                .resizable()
                                .aspectRatio(contentMode: .fit)
                                .scaleEffect(fetusScale)
                                .modifier(
                                    FloatingAnimation(
                                        radius: 20.0 + (CGFloat(numberOfFetuses - 1) * 10.0), // 10 extra for every extra fetus
                                        pct: beginAnimation ? 1.0 : 0.0
                                    )
                                    .ignoredByLayout()
                                )
                            // This animation causes the simulator to go into 100% CPU usage when the view isn't present on the screen. I can't find a fix for it, but everything works as expected on a real iOS device...
                        }
                    }
                    .animation(beginAnimation ? Animation.linear(duration: 80.0).repeatForever(autoreverses: true) : nil, value: beginAnimation)
                    .rotationEffect(Angle(degrees: beginAnimation ? 70 : 0), anchor: .center)
                    .animation(beginAnimation ? Animation.linear(duration: 20.0).repeatForever(autoreverses: true) : nil, value: beginAnimation)
                    
                }
                .clipShape(Circle())
            }
            .padding(lineWidth + padding + 3) // plus minor space for the inner circle
        }
        .accessibility(
            label:
                Text(Localizable.overview_baby_circle_accessibility_label(Int(monthsIntoPregnancy / targetMonths * 100)))
        )
    }
    
    private var currentImage: Rswift.ImageResource {
        switch monthsIntoPregnancy {
        case 0..<1.0:
            return R.image.baby.month0p5
        case 1.0..<1.5:
            return R.image.baby.month1
        case 1.5..<2.0:
            return R.image.baby.month1p5
        case 2.0..<2.5:
            return R.image.baby.month2
        case 2.5..<3.0:
            return R.image.baby.month2p5
        case 3.0..<3.5:
            return R.image.baby.month3
        case 3.5..<4.0:
            return R.image.baby.month3p5
        case 4.0..<4.5:
            return R.image.baby.month4
        case 4.5..<5.0:
            return R.image.baby.month4p5
        case 5.0..<5.5:
            return R.image.baby.month5
        case 5.5..<6.0:
            return R.image.baby.month5p5
        case 6.0..<6.5:
            return R.image.baby.month6
        case 6.5..<7.0:
            return R.image.baby.month6p5
        case 7.0..<7.5:
            return R.image.baby.month7
        case 7.5..<8.0:
            return R.image.baby.month7p5
        case 8.0..<8.5:
            return R.image.baby.month8
        case 8.5..<9.0:
            return R.image.baby.month8p5
        default:
            return R.image.baby.month9
        }
    }
}

struct OverviewPregnancyProgressView_Previews: PreviewProvider {
    static var previews: some View {
        VStack {
            OverviewPregnancyProgressView(monthsIntoPregnancy: 1, beginAnimation: true, numberOfFetuses: 0)
            OverviewPregnancyProgressView(monthsIntoPregnancy: 1, beginAnimation: true, numberOfFetuses: 3)
            OverviewPregnancyProgressView(monthsIntoPregnancy: 9, beginAnimation: true, numberOfFetuses: 4)
            Spacer()
            
        }.padding()
    }
}
