// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import TIM
import LocalAuthentication

extension ChoosePinCodeView {

    final class ViewModel : ObservableObject {
        @DI private var logger: Logger
        @DI private var timStorage: TIMDataStorage
        @DI private var timAuth: TIMAuth
        @DI private var localAuth: LocalAuthenticationContext
        @DI private var userContext: UserContextProtocol

        private var cancelBag = CancelBag()

        let userId: String
        let pinCodeLength: UInt = Metrics.Lengths.pinCode

        @Published var pinCode: String = ""
        @Published var isLoading: Bool = false
        @Published var failedToSetPinCode: Bool = false
        @Published var askToEnableTouchID: Bool = false
        @Published var failedToLoadInitialDataError: Error?

        @Published var showKeyboard: Bool = true

        init(userId: String) {
            self.userId = userId

            $pinCode.sink { [weak self] (pinCode) in
                guard let self = self else {
                    return
                }
                if pinCode.length == self.pinCodeLength, !self.isLoading {
                    self.saveRefreshToken(pin: pinCode)
                }
            }
            .store(in: &cancelBag)
        }

        private func saveRefreshToken(pin: String) {
            guard let refreshToken = timAuth.refreshToken else {
                failedToSetPinCode = true
                return
            }
            isLoading = true
            showKeyboard = false
            timStorage.storeRefreshToken(refreshToken, withNewPassword: pin)
                .receiveOnMain()
                .sink(receiveCompletion: { [weak self] (completion) in
                    switch completion {
                    case .failure(let error):
                        self?.failedToSetPinCode = true
                        self?.logger.error("Failed to store refresh token with new pin: \(error.localizedDescription)")
                    case .finished:
                        if self?.localAuth.isAuthenticationAvailable == true {
                            self?.askToEnableTouchID = true
                        } else {
                            self?.finishRegisterFlow()
                        }
                        self?.logger.debug("Successfully stored refresh token with new pin code.")
                    }
                    self?.isLoading = false
                }, receiveValue: { _ in })
            .store(in: &cancelBag)
        }

        func enableBio() {
            isLoading = true
            showKeyboard = false
            
            // TODO Save this on deveice

            timStorage.enableBiometricAccessForRefreshToken(
                password: pinCode,
                userId: userId
            )
            .receiveOnMain()
            .sink(
                receiveCompletion: { [weak self] (completion) in
                    // We do not set `isLoading = false`, since finishRegisterFlow will continue loading.
                    switch completion {
                    case .failure(let error):
                        self?.logger.error("Failed to store refresh token with biometric access: \(error.localizedDescription)")
                        // Continue anyway, we don't have any good way to handle a fallback for this.
                        self?.finishRegisterFlow()
                    case .finished:
                        self?.finishRegisterFlow()
                    }
                },
                receiveValue: { _ in }
            ).store(in: &cancelBag)
        }

        func finishRegisterFlow() {
            // User is done with the onboarding
            self.userContext.setHasHandledOnboardingFlow(true)

            // Now start loading the necessary data to continue.
            
            // TODO call warm-up endpoint here
            NotificationCenter.default.post(
                notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                value: SceneDelegate.NotificationMessage.showTabBarController
            )
        }

        func resetToLogin() {
            guard let userId = timAuth.refreshToken?.userId else {
                resetToOnboarding()
                return
            }
            NotificationCenter.default.post(
                notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                value: SceneDelegate.NotificationMessage.logout(userId: userId)
            )
        }

        func resetToOnboarding() {
            NotificationCenter.default.post(
                notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                value: SceneDelegate.NotificationMessage.resetToOnboarding
            )
        }
    }
}
