// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import Foundation
import SwiftUI
import UserSettingsServices

final class TabBarController: UITabBarController {
    private var cancelBag = CancelBag()
    private var popper: TabBarControllerPopper?

    // swiftlint:disable:next function_body_length
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.popper = TabBarControllerPopper(self)
        self.delegate = self.popper
        
        let overviewViewControler = UIHostingController(rootView: OverviewView())
        overviewViewControler.tabBarItem = tabbarItem(
            title: Localizable.bottom_navigation_title_overview(),
            image: R.image.icons.tabbar.overview(),
            selectedImage: R.image.icons.tabbar.overviewSelected(),
            accessibilityId: "tabbar_overview"
        )
                
        let journalViewController = UIHostingController(rootView: JournalView())
        journalViewController.tabBarItem = tabbarItem(
            title: Localizable.bottom_navigation_title_journal(),
            image: R.image.icons.tabbar.journal(),
            selectedImage: R.image.icons.tabbar.journalSelected(),
            accessibilityId: "tabbar_journal"
        )

        let knowledgeViewController = UIHostingController(rootView: KnowledgeView())
        knowledgeViewController.tabBarItem = tabbarItem(
             title: Localizable.bottom_navigation_title_knowledge(),
            image: R.image.icons.tabbar.knowledge(),
            selectedImage: R.image.icons.tabbar.knowledgeSelected(),
            accessibilityId: "tabbar_knowledge"
        )

        let appointmentViewController = UIHostingController(rootView: AppointmentsView())
        appointmentViewController.tabBarItem = tabbarItem(
            title: Localizable.bottom_navigation_title_appointments(),
            image: R.image.icons.tabbar.appointments(),
            selectedImage: R.image.icons.tabbar.appointmentsSelected(),
            accessibilityId: "tabbar_appointments"
        )

        let moreViewController = UIHostingController(rootView: MoreView())
        moreViewController.tabBarItem = tabbarItem(
            title: Localizable.bottom_navigation_title_more(),
            image: R.image.icons.tabbar.more(),
            selectedImage: R.image.icons.tabbar.moreSelected(),
            accessibilityId: "tabbar_more"
        )
        viewControllers = [
            overviewViewControler,
            appointmentViewController,
            journalViewController,
            knowledgeViewController,
            moreViewController
        ]
        
        viewControllers?.forEach({ (vc) in
            tabBar.tintColor = ColorManager.bgSurfaceNeutralPrimary.currentUIColor()
            vc.extendedLayoutIncludesOpaqueBars = true
        })
        tabBar.accessibilityIdentifier = "tab_bar"
        tabBar.tintColor = ColorManager.iconInteractivePrimary.currentUIColor()
        tabBar.unselectedItemTintColor = ColorManager.iconSecondary.currentUIColor()
        tabBar.barTintColor = ColorManager.bgSurfaceNeutralPrimary.currentUIColor()

        listenForNotifications()
    }

    private func tabbarItem(title: String, image: UIImage?, selectedImage: UIImage?, accessibilityId: String) -> UITabBarItem {
        let item = UITabBarItem(
            title: title,
            image: image,
            selectedImage: selectedImage
        )
        item.accessibilityIdentifier = accessibilityId
        item.badgeColor = ColorManager.iconDecorativePurpleHeart.currentUIColor()
        item.setBadgeTextAttributes(
            [
                NSAttributedString.Key.font: FontStyle.textXXSStrong.scaledUIFont() as Any,
                NSAttributedString.Key.foregroundColor: UIColor.white as Any
            ],
            for: .normal
        )
        return item
    }

    private func listenForNotifications() {
        NotificationCenter.default
            .listen(for: .TabBarController.changeTab)
            .sink(receiveValue: handleChangeTabNotification)
            .store(in: &cancelBag)

        NotificationCenter.default
            .listen(for: .TabBarController.changeOfSelectedMaternity)
            .sink(receiveValue: handleChangeOfSelectedMaternityNotification)
            .store(in: &cancelBag)
    }

    private func handleChangeTabNotification(_ notificationValue: ChangeTabNotificationMessage) {
        switch notificationValue {
        case .knowledge:
            selectedIndex = 2
        }
    }

    private func handleChangeOfSelectedMaternityNotification(_ notificationValue: String) {
        // Reset tabs for maternity change
        viewControllers?
            .flatMap({ $0.children.compactMap({ $0 as? UINavigationController }) })
            .forEach({ navCon in
                navCon.popToRootViewController(animated: false)
            })
    }

    private func presentOkayAlert(title: String, message: String) {
        let alert = UIAlertController(
            title: title,
            message: message,
            preferredStyle: .alert
        )
        alert.addAction(.init(title: Localizable.common_okay(), style: .default, handler: nil))
        present(alert, animated: true, completion: nil)
    }
}

extension TabBarController {
    enum ChangeTabNotificationMessage : NotificationCenterValue {
        case knowledge
    }
}

extension String: NotificationCenterValue { }

private class TabBarControllerPopper: NSObject, UITabBarControllerDelegate {
    private var previouslySelectedIndex: Int

    init(_ tabBarController: TabBarController) {
        self.previouslySelectedIndex = tabBarController.selectedIndex
    }

    func tabBarController(_ tabBarController: UITabBarController, didSelect viewController: UIViewController) {
        if previouslySelectedIndex == tabBarController.selectedIndex {
            popToRootOrScrollUp(on: viewController)
        }

        previouslySelectedIndex = tabBarController.selectedIndex
    }

    private func popToRootOrScrollUp(on viewController: UIViewController) {
        let nvc = navigationController(for: viewController)
        let popped = nvc?.popToRootViewController(animated: true)

        if (popped ?? []).isEmpty {
            let rootViewController = nvc?.viewControllers.first ?? viewController
            if let scrollView = firstScrollView(in: rootViewController.view ?? UIView()) {
                let preservedX = scrollView.contentOffset.x
                let y = -scrollView.adjustedContentInset.top
                scrollView.setContentOffset(CGPoint(x: preservedX, y: y), animated: true)
            }
        }
    }

    private func navigationController(for viewController: UIViewController) -> UINavigationController? {
        for child in viewController.children {
            if let nvc = viewController as? UINavigationController {
                return nvc
            } else if let nvc = navigationController(for: child) {
                return nvc
            }
        }
        return nil
    }

    func firstScrollView(in view: UIView) -> UIScrollView? {
        for subview in view.subviews {
            if let scrollView = view as? UIScrollView {
                return scrollView
            } else if let scrollView = firstScrollView(in: subview) {
                return scrollView
            }
        }

        return nil
    }
}
