// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData
import UIData

struct DataTableCellView: View {
    let cell: DataTableCell
    let rowNumber: Int
    let height: CGFloat
    let width: CGFloat
    let leadingPadding: CGFloat
    let trailingPadding: CGFloat
    let colorOddRows: Bool

    private var shouldColorRow: Bool {
        colorOddRows ? rowNumber % 2 != 0 : rowNumber % 2 == 0
    }

    init(cell: DataTableCell, rowNumber: Int, height: CGFloat, width: CGFloat, leadingPadding: CGFloat = 0, trailingPadding: CGFloat = 0, colorOddRows: Bool = true) {
        self.cell = cell
        self.rowNumber = rowNumber
        self.height = height
        self.width = width
        self.leadingPadding = leadingPadding
        self.trailingPadding = trailingPadding
        self.colorOddRows = colorOddRows
    }

    var body: some View {
        VStack(alignment: .leading, spacing: 0) {
            Spacer()
            HStack(spacing: 0) {
                switch cell.type {
                case .text:
                    Text(cell.value)
                        .modifier(
                            DataTableCellViewModifier(
                                fontStyle: .textM,
                                foregroundColor: ColorManager.textPrimary.currentColor(),
                                rowNumber: rowNumber
                            )
                        )
                    
                case .boldText:
                    Text(cell.value)
                        .modifier(
                            DataTableCellViewModifier(
                                fontStyle: .headlineXS,
                                foregroundColor: ColorManager.textPrimary.currentColor(),
                                rowNumber: rowNumber
                            )
                        )

                case .primaryButton(let action):
                    button(title: cell.value, color: ColorManager.iconDecorativePurpleHeart.currentColor(), action: action)
                case .infoButton(let action):
                    button(title: cell.value, color: R.color.aqua70.color(), action: action)
                }
                Spacer(minLength: 0)
            }
            .padding(.leading, leadingPadding)
            .padding(.trailing, trailingPadding)

            Spacer()
        }
        .frame(
            minHeight: (height == .infinity ? 0 : height), // .infinity is not allowed as minHeight
            maxHeight: height //Using maxHeight to allow use of `.infinity`
        )
        .frame(
            minWidth: (width == .infinity ? 0 : width), // .infinity is not allowed as minWidth
            maxWidth: width //Using maxWidth to allow use of `.infinity`
        )
        .frame(minHeight: 64)
        .background(shouldColorRow ? ColorManager.bgTableCell.currentColor() : Color.clear)
    }

    private func button(title: String, color: Color, action: @escaping () -> Void) -> some View {
        Button(title, action: action)
            .modifier(
                DataTableCellViewModifier(
                    fontStyle: .headlineXS,
                    foregroundColor: color,
                    rowNumber: rowNumber
                )
            )
    }
}

private struct DataTableCellViewModifier: ViewModifier {
    let fontStyle: FontStyle
    let foregroundColor: Color
    let rowNumber: Int

    func body(content: Content) -> some View {
        content
            .fixedSize(horizontal: false, vertical: true) // To make iOS 13 happy
            .font(fontStyle.scaledFont())
            .foregroundColor(foregroundColor)
            .padding(10)
            .overlay(DataTableCellDetermineHeight(row: rowNumber))
    }
}

struct DataTableCellView_Previews: PreviewProvider {
    static var previews: some View {
        ScrollView {
            VStack(spacing: 0) {
                DataTableCellView(
                    cell: DataTableCell(
                        id: "id",
                        value: String.loremIpsum(.small),
                        type: .boldText
                    ),
                    rowNumber: 0,
                    height: 400,
                    width: 100
                )
                DataTableCellView(
                    cell: DataTableCell(
                        id: "id",
                        value: "YAY",
                        type: .infoButton({ })
                    ),
                    rowNumber: 1,
                    height: 20,
                    width: 100
                )
                DataTableCellView(
                    cell: DataTableCell(
                        id: "id",
                        value: "YAY",
                        type: .text
                    ),
                    rowNumber: 2,
                    height: 70,
                    width: 100
                )
            }
            .border(Color.pink, width: 2)
        }
    }
}
