// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData
import UIData

struct ScrollableDataTable: View {
    let valueColumns: [DataTableColumn]
    let titleColumnRows: DataTableColumn
    let width: CGFloat
    let leadingColumnPadding: CGFloat
    let trailingColumnPadding: CGFloat

    @Binding var scrollOffset: CGPoint
    @Binding var scrollContentSize: CGSize
    @Binding var scrollViewSize: CGSize

    @State private var calculatedHeightForRow: [Int: CGFloat] = [:]

    private let dividerWidth: CGFloat = 1

    var body: some View {
            HStack(alignment: .top, spacing: 0) {
                buildColumn(
                    titleColumnRows.cells,
                    width: titleColumnRows.width + leadingColumnPadding,
                    leadingPadding: leadingColumnPadding
                )
                ScrollViewOffset(
                    scrollOffset: $scrollOffset,
                    scrollContentSize: $scrollContentSize,
                    axes: .horizontal,
                    showsIndicators: false
                ) {
                    HStack(alignment: .top, spacing: 0) {
                        ForEach(valueColumns.indices, id: \.self) { columnNumber in
                            let rowValuesForColumn = valueColumns[columnNumber]
                            buildColumn(
                                rowValuesForColumn.cells,
                                width: widthForValueColumn(columnWidth: rowValuesForColumn.width, columnNumber: columnNumber),
                                trailingPadding: trailingValuePadding(columnNumber: columnNumber)
                            )
                            .tag(columnNumber)
                            DefaultDivider()
                                .frame(width: dividerWidth)
                        }
                    }
                }
                .overlay(
                    HStack {
                        LinearGradient(
                            gradient: Gradient(
                                colors: [
                                    ColorManager.borderNeutralPrimary.currentColor(),
                                    ColorManager.borderNeutralPrimary.currentColor().opacity(0)
                                    ]
                            ),
                            startPoint: .leading,
                            endPoint: .trailing
                        )
                        .frame(width: 10)
                        Spacer()
                    }
                )
                .onSizeChanged(ScrollViewSizeReaderKey.self) { (size) in
                    scrollViewSize = size
                }
            }
            .frame(width: width)
    }

    private func buildColumn(_ cells: [DataTableCell], width: CGFloat, leadingPadding: CGFloat = 0, trailingPadding: CGFloat = 0) -> some View {
        VStack(alignment: .leading, spacing: 0) {
            ForEach(cells.indices, id: \.self) { rowNumber in
                let cell = cells[rowNumber]
                DataTableCellView(
                    cell: cell,
                    rowNumber: rowNumber,
                    height: calculatedHeightForRow[rowNumber] ?? 0,
                    width: width,
                    leadingPadding: leadingPadding,
                    trailingPadding: trailingPadding
                )
                .onPreferenceChange(DataTableCellDetermineHeight.Key.self) { result in
                    calculatedHeightForRow[result.row] = max(
                        calculatedHeightForRow[result.row] ?? 0,
                        result.height
                    )
                }
                .tag(cell.id)
            }
        }
    }

    /// This function will return the greatest value of these two scenarios:
    ///     1. (tableWidth - paddings - columnWidth - (dividerWidth * numberOfValueColumns)) / numberOfValueColumns
    ///     So the idea is to get the space that is left for the value columns, and then split it into the number of value columns we have to achieve fill effect for the table columns even if there is too few to fill the width.
    ///
    ///     2. columnWidth - dividerWidth
    ///     If there are enough columns to fill the width, then this value will be the greatest value and the system will take this for all columns.
    ///
    /// Finally the trailing padding is added if it is the last column.
    private func widthForValueColumn(columnWidth: CGFloat, columnNumber: Int) -> CGFloat {
        max(
            (width - titleColumnRows.width - leadingColumnPadding - trailingColumnPadding - (dividerWidth * CGFloat(valueColumns.count))) / CGFloat(valueColumns.count),
            columnWidth - dividerWidth
        ) + trailingValuePadding(columnNumber: columnNumber)
    }

    private func trailingValuePadding(columnNumber: Int) -> CGFloat {
        columnNumber == (valueColumns.count - 1) ? trailingColumnPadding : 0
    }

    private static var previewRows: Int = 5
    private static let previewTitles = [
        "Date",
        "Week",
        "Estimated Due Date",
        "Fetus Position",
        "Estimated Fetal Weight",
        "Fetal Activity",
        "Heartbeat",
        "Fetal Position in Ultrasound",
        "Chorionicity",
        "Chromosomal Abnormalities",
        "Ultrasound Weight",
        "Ultrasound Weight Deviation"
    ]

    private static let previewValues = [
        ["10-10-2023", "14+3", "15-09-2023", "Front", "450g", "-", "-", "-", "-", "-", "-", "-"],
        ["29-12-2023", "17+1", "15-09-2023", "Top", "600g", "-", "-", "-", "-", "-", "-", "-"],
        ["07-02-2024", "23+6", "22-07-2024", "Side", "900g", "-", "-", "-", "-", "-", "-", "-"],
        ["15-04-2024", "29+5", "14-07-2024", "Bottom", "1200g", "-", "-", "-", "-", "-", "-", "-"],
        ["20-05-2024", "33+2", "19-06-2024", "Back", "1500g", "-", "-", "-", "-", "-", "-", "-"]
    ]

    static func previewTitleColumn() -> DataTableColumn {
        let cells = previewTitles.enumerated().map { (index, title) -> DataTableCell in
            DataTableCell(
                id: "title \(index)",
                value: title,
                type: .boldText
            )
        }
        
        return DataTableColumn(width: 140, cells: cells)
    }

    static func previewColumns() -> [DataTableColumn] {
        let numberOfRows = previewValues.count

        return (1...numberOfRows).map { row in
            let cells = (1...previewTitles.count).map { column in
                let value = previewValues[row - 1][column - 1]
                return DataTableCell(
                    id: "\(column)-\(row)",
                    value: value,
                    type: .text
                )
            }
            return DataTableColumn(width: 100, cells: cells)
        }
    }



}

private struct ScrollViewSizeReaderKey: SizeReaderKey {
    static var defaultValue: CGSize = .zero
}

struct ScrollableDataTable_Previews: PreviewProvider {
    static var previews: some View {
        GeometryReader { geo in
            VStack(spacing: 0) {
                ScrollView {
                    ScrollableDataTable(
                        valueColumns: ScrollableDataTable.previewColumns(),
                        titleColumnRows: ScrollableDataTable.previewTitleColumn(),
                        width: geo.size.width,
                        leadingColumnPadding: 0,
                        trailingColumnPadding: 0,
                        scrollOffset: .constant(.zero),
                        scrollContentSize: .constant(.zero),
                        scrollViewSize: .constant(.zero)
                    )
                }
                Divider()
                // Illustrates the stretch of width columns
                ScrollView {
                    ScrollableDataTable(
                        valueColumns: ScrollableDataTable.previewColumns().suffix(1),
                        titleColumnRows: ScrollableDataTable.previewTitleColumn(),
                        width: geo.size.width,
                        leadingColumnPadding: 0,
                        trailingColumnPadding: 0,
                        scrollOffset: .constant(.zero),
                        scrollContentSize: .constant(.zero),
                        scrollViewSize: .constant(.zero)
                    )
                }

            }
        }
    }
}
