// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import Rswift

enum SmileyRating: Int, CaseIterable, Identifiable {
    case verySad = 1
    case sad = 2
    case medium = 3
    case happy = 4
    case veryHappy = 5

    var id: Int {
        rawValue
    }

    var image: Rswift.ImageResource {
        switch self {
        case .verySad: return R.image.icons.ratings.emoji1
        case .sad: return R.image.icons.ratings.emoji2
        case .medium: return R.image.icons.ratings.emoji3
        case .happy: return R.image.icons.ratings.emoji4
        case .veryHappy: return R.image.icons.ratings.emoji5
        }
    }
}

struct SmileyRateView: View {

    @Binding var rating: SmileyRating?
    @State private var angleForSmiley: [SmileyRating: Angle] = [:]

    var body: some View {
        HStack(spacing: 18) {
            ForEach(SmileyRating.allCases) { smiley in
                Button(
                    action: {
                        self.rating = smiley
                        self.updateAllSmileyAngles()
                    },
                    label: {
                        Image(smiley.image.name)
                            .resizable()
                            .renderingMode(.template)
                            .foregroundColor(rating == smiley ? .red : .blue)
                            .animation(.linear(duration: 0.1))
                            .frame(size: 38)
                            .rotationEffect(angleForSmiley[smiley] ?? .degrees(0))
                   }
                )
                .buttonStyle(PlainButtonStyle()) // Remove highlight
                .tag(smiley.id)
            }
        }
    }

    /// This might look a little weird, but it is needed to make sure that
    /// only the selected one is animated. Otherwise the deselected would
    /// also animate its state change.
    private func updateAllSmileyAngles() {
        for smiley in SmileyRating.allCases {
            if smiley == rating {
                withAnimation(.easeOut(duration: 0.4)) {
                    angleForSmiley[smiley] = .degrees(360)
                }
            } else {
                angleForSmiley[smiley] = .degrees(0)
            }
        }
    }
}

struct SmileyRateView_Previews: PreviewProvider {
    static var previews: some View {
        SmileyRateView(rating: .constant(nil))
    }
}
