// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import LocalAuthentication

enum NumberPadKeyboardKeyType: Hashable {
    case number(UInt)
    case delete
    case bio
    case empty

    var value: String? {
        switch self {
        case .number(let number):
            return "\(number)"
        default:
            return nil
        }
    }

    @ViewBuilder
    var view: some View {
        switch self {
        case .bio:
            // Direct use of `LAContext`, since DI isn't available in an enum
            let image = LAContext().biometryType == .faceID ? R.image.icons.keyboard.faceid : R.image.icons.keyboard.fingerprint
            Image(image.name)
                .renderingMode(.template)
                .accessibility(identifier: "bio")
                .accessibility(label: Text(LAContext().authentiocationName))
        case .delete:
            Image(R.image.icons.keyboard.delete.name).renderingMode(.template)
                .accessibility(identifier: "delete")
                .accessibility(label: Text(Localizable.common_keyboard_accessbility_delete()))
        case .number(let n):
            Text("\(n)")
                .accessibility(identifier: "\(n)")
        case .empty:
            Text("")
                .accessibility(identifier: "empty")
        }
    }

    var color: Color {
        switch self {
        case .bio:
            return R.color.keyboardBioAction.color()
        default:
            return ColorManager.textPrimary.currentColor()
        }
    }

    static func isValidInput(_ input: String) -> Bool {
        UInt(input) != nil
    }
}

struct NumberPadKeyboardKey: View {
    let key: NumberPadKeyboardKeyType
    var activated: (NumberPadKeyboardKeyType) -> Void

    var body: some View {
        GeometryReader { geo in
            Button(
                action: {
                    UIDevice.current.playInputClick()
                    activated(key)
                },
                label: {
                    key.view
                        .frame(width: geo.size.width, height: geo.size.height)
                }
            )
            .buttonStyle(NumberPadKeyboardKeyStyle(color: key.color))
        }
    }
}

struct NumberPadKeyboardKeyStyle: ButtonStyle {
    let color: Color

    func makeBody(configuration: Self.Configuration) -> some View {
        configuration.label
            .background(ColorManager.bgSurfaceNeutralSecondary.currentColor())
            .opacity(configuration.isPressed ? 0.8 : 1.0)
            .font(FontStyle.keyboard.scaledFont())
            .foregroundColor(color)
      }
}

struct NumberPadKeyboardKey_Previews: PreviewProvider {
    static var previews: some View {
        NumberPadKeyboardKey(key: .number(1), activated: { _ in })
    }
}
