// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import ErrorData

enum DataLoadingState {
    case initial
    /// Title, Description
    case empty(String, String)
    case data
    case loading
    case refreshing
    case error(Error)

    var allowsDataFetch: Bool {
        let allow: Bool

        switch self {
        case .initial,
             .error,
             .empty,
             .refreshing:
            allow = true
        case .loading, .data:
            allow = false
        }

        return allow
    }

    var scrollAxis: Axis.Set {
        self == .data ? .vertical : []
    }

    var isEmpty: Bool {
        if case .empty = self {
            return true
        } else {
            return false
        }
    }

    var allowRefresh: Bool {
        if case .data = self {
            return true
        } else if case .empty = self {
            return true
        } else {
            return false
        }
    }
}

extension DataLoadingState: Equatable {
    static func == (lhs: DataLoadingState, rhs: DataLoadingState) -> Bool {
        switch (lhs, rhs) {
        case (.initial, .initial),
            (.data, .data),
            (.loading, .loading),
            (.refreshing, .refreshing):
            return true

        case (.empty(let lhsTitle, let lhsDescription), .empty(let rhsTitle, let rhsDescription)):
            return lhsTitle == rhsTitle && lhsDescription == rhsDescription

        case (.error(let lhsError), .error(let rhsError)):
            return lhsError.localizedDescription == rhsError.localizedDescription
        default:
            return false
        }
    }
}

extension DataLoadingState {
    static func noSelectedMaternityEmpty() -> DataLoadingState {
        .empty(
            Localizable.common_no_episode_of_care_title(),
            Localizable.common_no_episode_of_care_text()
        )
    }
}

struct DataLoadingView<Content: View>: View {

    let state: DataLoadingState
    let retryAction: () -> Void
    let content: Content
    let animateStateChanges: Bool
    let contentHeightForNoDataState: CGFloat?

    init(state: DataLoadingState, contentHeightForNoDataState: CGFloat? = nil, animateStateChanges: Bool = true, retryAction: @escaping () -> Void, @ViewBuilder content: @escaping () -> Content) {
        self.state = state
        self.animateStateChanges = animateStateChanges
        self.retryAction = retryAction
        self.contentHeightForNoDataState = contentHeightForNoDataState
        self.content = content()
    }

    var body: some View {
        VStack {
            switch state {
            case .empty(let title, let description):
                StretchingVStack(height: contentHeightForNoDataState) {
                    EmptyStateView(title: title, text: description)
                }
            case .data, .refreshing:
                self.content
            case .initial,
                 .loading:
                StretchingVStack(height: contentHeightForNoDataState) {
                    ActivityIndicatorView(
                        isAnimating: true,
                        style: .medium,
                        hidesWhenStopped: true
                    )
                }
            case .error(let error):
                StretchingVStack(height: contentHeightForNoDataState) {
                    ErrorStateView(
                        text: error.localizedDescription,
                        retryAction: retryAction
                    )
                }
            }
        }
        .frame(maxWidth: .infinity, maxHeight: .infinity)
        .transition(
            animateStateChanges ? AnyTransition.opacity.animation(.easeInOut(duration: 0.2)) : .identity
        )
    }
}

private struct StretchingVStack<Content: View>: View {
    let height: CGFloat?
    let content: () -> Content

    var body: some View {
        VStack(spacing: 0) {
            Spacer(minLength: 0)
            content()
            Spacer(minLength: 0)
        }
        .modifier(StretchingViewModifier(height: height))
    }
}

private struct StretchingViewModifier: ViewModifier {
    let height: CGFloat?

    @ViewBuilder
    func body(content: Content) -> some View {
        if let height = height, height > 0 {
            content.frame(height: height)
        } else {
            content
        }
    }
}

struct DataLoadingView_Previews: PreviewProvider {
    static var previews: some View {
        VStack {
            DataLoadingView(state: .empty("Empty!", "Øv, der kom ingen data!"), retryAction: {}, content: {
                Text("Should not be shown!")
            })
            DataLoadingView(state: .data, retryAction: {}, content: {
                Text("Content! 🥳")
            })
            DataLoadingView(state: .loading, retryAction: {}, content: {
                Text("Should not be shown!")
            })
            DataLoadingView(state: .error(ServiceError.failedToParseJson), retryAction: {}, content: {
                Text("Should not be shown!")
            })
        }
    }
}
