// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct PartialSheetButton {
    let title: String
    let action: (() -> Void)?

    static func `default`() -> PartialSheetButton {
        PartialSheetButton(
            title: Localizable.common_cta_close(),
            action: nil
        )
    }
}

struct PartialSheetView<Content: View>: View {
    @Environment(\.viewController) private var viewControllerHolder: UIViewController?

    let showCloseButton: Bool
    let primaryButton: PartialSheetButton?
    let secondaryButton: PartialSheetButton?
    let content: Content

    // We need to use this because a bool is just flipped and has no progress.
    // Thus the view controller dismiss animation will be called right away.
    @State private var presentingAnimationProgress: CGFloat = 0.0
    @State private var presentedViewSize: CGSize = .zero
    private var isPresented: Bool { presentingAnimationProgress >= 1.0 }

    var body: some View {
        GeometryReader { reader in
            ZStack {
                VisualBlurEffectView(style: .systemChromeMaterialDark)
                    .edgesIgnoringSafeArea(.all)
                    .opacity(self.isPresented ? 0.9 : 0)
                    .onTapGesture(perform: closeSheet)

                VStack(spacing: 0) {
                    Spacer()

                    VStack(spacing: 0) {
                        if showCloseButton {
                            HStack {
                                Spacer()
                                Button(
                                    action: { closeSheet() },
                                    label: { Image(R.image.icons.basic.close.name)
                                            .resizable()
                                            .renderingMode(.template)
                                            .smallIconStyle()
                                            .foregroundStyle(ColorManager.iconPrimary.currentColor())
                                    
                                }
                                )
                                .padding(Metrics.Spacing.space16)
                            }
                        }
                            ScrollView {
                                self.content
                                    .padding(.horizontal)
                                    .padding(.bottom, Metrics.Spacing.space12)
                                    .onSizeChanged(ContentSizeReaderKey.self, perform: { (contentSize: CGSize) in
                                        presentedViewSize = contentSize
                                    })
                            }
                            .frame(maxHeight: presentedViewSize.height > 0 ? presentedViewSize.height : .infinity)
                            .onAnimationCompleted(for: presentingAnimationProgress) {
                                guard !self.isPresented else { return }
                                viewControllerHolder?.dismiss(animated: false, completion: nil)
                        }
                        VStack {
                            if let primaryButton = primaryButton {
                                ButtonPrimary(
                                    title: primaryButton.title,
                                    action: {
                                        primaryButton.action?()
                                        closeSheet()
                                    }
                                )
                            }
                            
                            if let secondaryButton = secondaryButton {
                                ButtonTextPrimary(
                                    title: secondaryButton.title,
                                    action: {
                                        secondaryButton.action?()
                                        closeSheet()
                                    }
                                )
                            }
                        }
                        .padding(.horizontal, Metrics.Spacing.space16)
                        .padding(.top, Metrics.Spacing.space12)
                        .padding(.bottom, Metrics.Spacing.space24)
                    }
                    .background(ColorManager.bgSurfaceNeutralSecondary.currentColor())
                    .clipShape(RoundedTopCornersShape(radius: Metrics.Sizes.cardCornerRadius))
                    .offset(x: 0, y: self.isPresented ? 0 : reader.frame(in: .global).height)
                }
                .background(Color.clear)
                .edgesIgnoringSafeArea(.bottom)
            }
        }
        .onAppear {
            withAnimation(.spring()) {
                self.presentingAnimationProgress = 1
            }
        }
    }

    private func closeSheet() {
        withAnimation {
            self.presentingAnimationProgress = 0
        }
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.3) {
            viewControllerHolder?.dismiss(animated: false, completion: nil)
        }
    }
}

private struct ContentSizeReaderKey: SizeReaderKey {
    static var defaultValue: CGSize = .zero
}

struct PartialSheetView_Previews: PreviewProvider {
    static var previews: some View {
        PartialSheetView(
            showCloseButton: true,
            primaryButton:
                PartialSheetButton(
                    title: "Slet",
                    action: nil
                ),
            secondaryButton:
                PartialSheetButton(
                    title: "Luk",
                    action: nil
                ),
            content:
                VStack(alignment: .leading) {
                    Text("Title")
                        .applyTextStyle(fontStyle: .headlineL)
                    Text(String.loremIpsum(.medium))
                }
        )
    }
}
