// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

/// A view that displays an image that is fetched.
public struct URLImage<PlaceholderView: View, ErrorView: View, ImageView: View>: View {
    @ObservedObject private var imageLoader: ImageLoader

    /// Image view
    private let imageView: (UIImage) -> ImageView

    /// The placeholder while the image is fetching.
    private let placeholderView: () -> PlaceholderView

    /// The view to show when an error is occoured.
    private let errorView: (Error) -> ErrorView

    /// Creates an image with the placeholder
    /// - Parameters:
    ///   - url: The image to fetch
    ///   - placeholderView: A view that should be showed while fetching.
    init(url: URL,
         @ViewBuilder imageView: @escaping (UIImage) -> ImageView,
         @ViewBuilder placeholderView: @escaping () -> PlaceholderView,
         @ViewBuilder errorView: @escaping (Error) -> ErrorView) {
        imageLoader = ImageLoader(url: url)
        self.imageView = imageView
        self.placeholderView = placeholderView
        self.errorView = errorView
    }

    public var body: some View {
        if let imageData = imageLoader.imageData,
           let uiImage = UIImage(data: imageData) {
            imageView(uiImage)
        } else if let error = imageLoader.error {
            errorView(error)
        } else {
            placeholderView()
                .onAppear(perform: imageLoader.fetchImage)
        }
    }
}

struct URLImage_Previews: PreviewProvider {
    static var previews: some View {
        URLImage(url: URL(string: "https://cataas.com/cat")!,
                 imageView: { uiImage in
                    Image(uiImage: uiImage)
                 },
                 placeholderView: {
                    Text("Placeholder")
                 }, errorView: { error in
                    Text(error.localizedDescription)
                 })
    }
}
