// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import WebKit
import CMSData
import Combine

/// View for showing HTML.
///
struct HTMLViewWithInternalLinking: UIViewRepresentable {
    
    @Binding var idPublisher: PassthroughSubject<Int, Never>
    @Binding var urlPublisher: PassthroughSubject<String, Never>

    /// The height will be published when the HTML is loaded
    @Binding var dynamicHeight: CGFloat

    /// Will return `false` when html is loaded
    @Binding var isLoading: Bool

    /// The file path to the HTML file
    let htmlFilePath: URL

    /// The HTML content that need to be replaced
    let html: String

    /// Determines whether the web view is hidden at creation and is faded in when content is loaded.
    let fadeInOnLoad: Bool

    /// Enables/disables scroll in the web view
    let scrollEnabled: Bool
    
    func makeUIView(context: Context) -> WKWebView {
        let webView = WKWebView()

        webView.scrollView.isScrollEnabled = scrollEnabled
        webView.scrollView.alwaysBounceVertical = scrollEnabled
        webView.navigationDelegate = context.coordinator
        webView.isOpaque = false
        webView.backgroundColor = .clear
        webView.scrollView.backgroundColor = .clear
 //       setWebviewDebugging(webView)
        
        if fadeInOnLoad {
            webView.alpha = 0.0
        }

        return webView
    }
    
//    func setWebviewDebugging(_ webView: WKWebView) {
//    #if DEBUG
//        if #available(iOS 16.4, *) {
//            webView.isInspectable = true
//        }
//    #endif
//    }

    func updateUIView(_ uiView: WKWebView, context: Context) {
        if let fileTemplate = try? String(contentsOf: htmlFilePath, encoding: .utf8) {
            var html = fileTemplate.replacingOccurrences(of: "[CONTENT]", with: self.html)
            var stylesheetCss = (try? String(contentsOfFile: R.file.stylesCss()!.path, encoding: .utf8)) ?? ""
            let fontStyles = [
                ("H1", FontStyle.headlineL),
                ("H2", FontStyle.headlineM),
                ("H3", FontStyle.headlineS),
                ("H4", FontStyle.headlineXS),
                ("H5", FontStyle.headlineXXS),
                ("BODY", FontStyle.textM)
            ]
            for fontStyle: (tag: String, style: FontStyle) in fontStyles {
                stylesheetCss = stylesheetCss.replacingOccurrences(
                    of: "[CSS_FONT_SIZE_\(fontStyle.tag)]",
                    with: Int(fontStyle.style.scaledFontSize).description
                )
            }

            html = html.replacingOccurrences(of: "[STYLE_CSS]", with: stylesheetCss)
            uiView.loadHTMLString(html, baseURL: htmlFilePath.deletingLastPathComponent())
        }
    }

    func makeCoordinator() -> Coordinator {
        Coordinator(
            dynamicHeight: $dynamicHeight,
            isLoading: $isLoading,
            fadeInOnLoad: fadeInOnLoad,
            idPublisher: $idPublisher,
            urlPublisher: $urlPublisher
        )
    }

    final class Coordinator: NSObject, WKNavigationDelegate {
        @DI private var logger: Logger

        @Binding var idPublisher: PassthroughSubject<Int, Never>
        @Binding var urlPublisher: PassthroughSubject<String, Never>
        @Binding var dynamicHeight: CGFloat
        @Binding var isLoading: Bool
        let fadeInOnLoad: Bool

        private static let fileScheme: String = "file"
        private static let httpsScheme: String = "https"

        /// The allowed URL schemes that are allowed to open
        let allowedNavigationSchemes: [String] = [ httpsScheme, "http", "tel", "mailto", fileScheme ]

        init(dynamicHeight: Binding<CGFloat>,
             isLoading: Binding<Bool>,
             fadeInOnLoad: Bool,
             idPublisher: Binding<PassthroughSubject<Int, Never>>,
             urlPublisher: Binding<PassthroughSubject<String, Never>>) {
            self._dynamicHeight = dynamicHeight
            self._isLoading = isLoading
            self.fadeInOnLoad = fadeInOnLoad
            self._idPublisher = idPublisher
            self._urlPublisher = urlPublisher
        }

        func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
            webView.evaluateJavaScript("document.documentElement.scrollHeight", completionHandler: { height, _ in
                DispatchQueue.main.async { [weak self] in
                    if let height = height as? CGFloat, height != self?.dynamicHeight {
                        self?.dynamicHeight = height
                    }

                    if self?.isLoading == true {
                        self?.isLoading = false
                    }

                    if self?.fadeInOnLoad == true && webView.alpha == 0.0 {
                        UIView.animate(withDuration: 0.2) {
                            webView.alpha = 1.0
                        }
                    }
                }
            })
        }


        func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
            // Check if the scheme is allowed, and then open it
            if let url = navigationAction.request.url, let scheme = url.scheme {
                let isAppDefinedSource: Bool = scheme == Self.fileScheme && url.absoluteString.contains(Bundle.main.bundleURL.absoluteString)
                let isAutoLoadedSourceFromHTMLContent: Bool = scheme == Self.httpsScheme && navigationAction.navigationType == .other //e.g. Vimeo

                if isAppDefinedSource || isAutoLoadedSourceFromHTMLContent {
                    decisionHandler(.allow) // Allow to load local content from the app
                } else {
                    decisionHandler(.cancel) // Always cancel to avoid navigation inside the embedded web view

                    if allowedNavigationSchemes.contains(scheme) {
                        handleUrlTap(url: url)
                    } else {
                        // Raising an error at this point, since something might be wrong on the CMS if this occurs...
                        logger.error("Cancelled web view navigation in `HTMLView` for disallowed scheme: \"\(scheme)\"\(scheme == Self.fileScheme ? " (FILE SCHEME NOT IN APP BUNDLE)" : "")")
                    }
                }
            } else {
                decisionHandler(.cancel) // Cancel if there is no URL
            }
        }
        
        private func handleUrlTap(url: URL) {
            if url.absoluteString.starts(with: "https://graviditetscms.sundhedsdata.dk") {
                handleInternalUrl(url)
            } else {
                handleExternalUrl(url)
            }
        }
        
        private func handleInternalUrl(_ url: URL) {
            let config = URLSessionConfiguration.default
            let delegate = DelegateToHandleInternalLinking(idPublisher: $idPublisher)
            let session = URLSession(configuration: config, delegate: delegate, delegateQueue: nil)
            session.dataTask(with: url).resume()
        }
        
        private func handleExternalUrl(_ url: URL) {
            DispatchQueue.main.async {
                self.urlPublisher.send(url.absoluteString)
            }
        }
    }
}

class DelegateToHandleInternalLinking: NSObject, URLSessionTaskDelegate {
    
    @Binding var idPublisher: PassthroughSubject<Int, Never>
    
    init(idPublisher: Binding<PassthroughSubject<Int, Never>>) {
        self._idPublisher = idPublisher
    }
    
    func urlSession(
        _ session: URLSession,
        task: URLSessionTask,
        willPerformHTTPRedirection response: HTTPURLResponse,
        newRequest request: URLRequest,
        completionHandler: @escaping (URLRequest?) -> Void) {
            
            guard let redirectLocation = response.value(forHTTPHeaderField: "location"),
                  let articleId = redirectLocation.split(separator: "/").last,
                  let articleIdInt = Int(articleId) else { return }
            
            DispatchQueue.main.async {
                print("SENDING ID: \(articleIdInt)")
                self.idPublisher.send(articleIdInt)
            }
        }
}

