// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct KeyValueView: View {
    typealias DictionaryAction = () -> Void
    let title: String
    let primaryValue: String?
    let secondaryValue: String?
    let dictionaryAction: DictionaryAction?

    private let cornerRadius: CGFloat = 4
    
    var body: some View {
        VStack(alignment: .leading) {
            if let dictionaryAction = dictionaryAction {
                Button(action: dictionaryAction, label: {
                    Text(title)
                        .applyTextStyle(fontStyle: .headlineXS, textColor: R.color.aqua70.color())
                })
            } else {
                Text(title)
                    .applyTextStyle(fontStyle: .headlineXS)
            }

            Group {
                HStack {
                    Text(primaryValue ?? Localizable.common_missing_data())
                        .applyTextStyle(fontStyle: .textM)
                        .fixedSize(horizontal: false, vertical: true)
                        .accessibility(label: Text(primaryValue ?? Localizable.common_missing_data_accessibility_label()))

                    Spacer()

                    if let secondaryText = secondaryValue {
                        Text(secondaryText)
                            .applyTextStyle(
                                fontStyle: .textM,
                                textColor: ColorManager.borderNeutralPrimary.currentColor()
                            )
                            .fixedSize(horizontal: false, vertical: true)
                    }
                }
                .padding()
            }
            .frame(minHeight: 64)
            .background(ColorManager.bgTableCell.currentColor())
            .overlay(
                RoundedRectangle(
                    cornerRadius: cornerRadius
                )
                .stroke(
                    ColorManager.borderNeutralPrimary.currentColor()
                )
            )
            .cornerRadius(cornerRadius)
            .innerShadow(
                RoundedRectangle(cornerRadius: cornerRadius),
                spread: 2.0,
                radius: 2.0
            )
        }
    }
}

struct KeyValueView_Previews: PreviewProvider {
    static var previews: some View {
        VStack {
            KeyValueView(title: "Vurdering på baggrund ",
                         primaryValue: "I sidste ",
                         secondaryValue: nil, dictionaryAction: { })
            
            KeyValueView(title: "Vurdering på baggrund af sygehistorie, undersøgelser og psykosociale forhold",
                         // swiftlint:disable:next line_length
                         primaryValue: "I sidste graviditet præeklampsi, men kontrolleret bagefter og har normal BT. Dårlig fødselsoplevelse efter akut sectio sidst. Overvejer sectio denne fødsel. Faderen arbejder delvist i udlandet så er meget alene, men har godt netværk.",
                         secondaryValue: nil, dictionaryAction: nil)
            
            KeyValueView(title: "Vurdering på baggrund af sygehistorie, undersøgelser og psykosociale forhold",
                         // swiftlint:disable:next line_length
                         primaryValue: "I sidste graviditet præeklampsi, men kontrolleret bagefter og har normal BT. Dårlig fødselsoplevelse efter akut sectio sidst. Overvejer sectio denne fødsel. Faderen arbejder delvist i udlandet så er meget alene, men har godt netværk.",
                         secondaryValue: "11.11.1111", dictionaryAction: { })
        }
        .padding()
        
    }
}
