// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
import DependencyInjection
import Combine
import MaternityServices
import UserSettingsServices
import Logger
import SharedServices
import GMAppData
import CMSData
import ConfigurationServices
@testable import MinGraviditet

final class OverviewViewModelTests: XCTestCase {
    private let envStore = MaternityRepositoryImpl.shared

    private let userSettingsService = UserSettingsServiceMock()
    private let configurationService = ConfigurationServiceMock()

    override func setUp() {
        super.setUp()

        Resolver.shared.allowOverride = true
        Resolver.shared.registerAsSingleton(UserSettingsService.self, userSettingsService)
        Resolver.shared.registerAsSingleton(ConfigurationService.self, configurationService)
        Resolver.shared.registerAsSingleton(Logger.self, OSLogger())
        Resolver.shared.registerAsSingleton(MaternityRepository.self, envStore)
        Resolver.shared.registerAsSingleton(MaternityStorageService.self, MaternityStorageServiceImpl())
        
        envStore.clear()
        MockFlags.failRisk = .disabled

        // Select maternity please!
        var cancelBag = CancelBag()
        let expect = XCTestExpectation()
        envStore.requestUpdate()
            .sink(receiveCompletion: { _ in expect.fulfill() }, receiveValue: { _ in })
            .store(in: &cancelBag)
        wait(for: [expect], timeout: 2.0)
    }

    func testNoArticles() {
        let viewModel = OverviewView.ViewModel()
        viewModel.currentRelevantArticleResults = []
        viewModel.nextRelevantArticleResults = []

        // Empty state
        XCTAssertFalse(viewModel.showArticlesSection)
        XCTAssertFalse(viewModel.showCurrentWeekArticles)
        XCTAssertFalse(viewModel.showNextWeekArticles)
    }

    func testCurrentArticles() {
        let viewModel = OverviewView.ViewModel()
        viewModel.currentRelevantArticleResults = CMSData.Category.mockList()
        viewModel.nextRelevantArticleResults = []

        // Still the same
        XCTAssertTrue(viewModel.showArticlesSection)
        XCTAssertTrue(viewModel.showCurrentWeekArticles)
        XCTAssertFalse(viewModel.showNextWeekArticles)
    }

    // MARK: -

    private func setDateOfBirth(_ date: Date) {
        var cancelBag = CancelBag()
        // Now mark as born!
        let expect = XCTestExpectation(description: "Should have returned from dateOfBirth.")
        envStore.changeDateOfBirthForSelectedMaternity(date)
            .sink(
                receiveCompletion: { _ in
                    expect.fulfill()
                },
                receiveValue: { _ in  }
            ).store(in: &cancelBag)
        wait(for: [expect], timeout: 1.0)
    }
}
