// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Combine
import CMSData
import ErrorData
import HTTPNetworking
import Logger
import DependencyInjection

public final class ArticlesServiceImpl : ArticlesService {
    @Inject private var networkSession: NetworkSession
    @Inject private var apiConfiguration: CMSApiConfiguration
    @Inject private var logger: LoggerProtocol

    public init() { }

    public func fetchArticleCategories() -> AnyPublisher<[CMSData.Category], ServiceError> {
        let request = ArticlesRequest(apiConfiguration: self.apiConfiguration)
        return self.networkSession.dataTaskPublisher(for: request)
            .map(\.data)
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    public func search(query: String) -> AnyPublisher<[SearchResultDomain], ServiceError> {
        let request = ArticlesSearchRequest(apiConfiguration: self.apiConfiguration, query: query)
        return self.networkSession.dataTaskPublisher(for: request)
            .map(\.data)
            .map{ (dtoList: [SearchResultDTO] ) in
                dtoList.toDomain()
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    public func fetchArticleCategories(for week: Int) -> AnyPublisher<CMSData.RecommendedCategories, ServiceError> {
        let request = WeekArticlesRequest(apiConfiguration: self.apiConfiguration, week: week)
        return self.networkSession.dataTaskPublisher(for: request)
            .map(\.data)
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    public func fetchAfterBirthArticleCategories(for weeksAfterBirth: Int) -> AnyPublisher<CMSData.RecommendedCategories, ServiceError> {
        let request = AfterBirthArticlesRequest(apiConfiguration: self.apiConfiguration, weeksAfterBirth: weeksAfterBirth)
        return self.networkSession.dataTaskPublisher(for: request)
            .map(\.data)
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    public func fetchLinkedArticle(for id: Int) -> AnyPublisher<CMSData.LinkedArticle, ServiceError> {
        let request = LinkedArticleRequest(apiConfiguration: self.apiConfiguration, articleId: id)
        return self.networkSession.dataTaskPublisher(for: request)
            .map(\.data)
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
}
