// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import CMSData
import ErrorData
import SharedServices

public final class ArticlesServiceMock : ArticlesService {

    let fiveDaysInSeconds: TimeInterval = 3600 * 24 * 5

    private func randomDescription() -> String {
        (0...Int.random(in: 5...50))
            .map { _ in "text"}
            .joined(separator: " ")
            .capitalized
    }

    private func categories(query: String? = nil) -> [CMSData.Category] {
        (1...3)
        .map({ i in
            CMSData.Category(
                id: i,
                name: "Kategori \(i)",
                slug: "mock",
                description: randomDescription(),
                iconUrl: URL(string: "https://robohash.org/\(i)")!,
                articles: (1...Int.random(in: 2...5)).map({ a in
                    Article(
                        id: a,
                        title: "Artikel \(a)",
                        lastUpdated: Date().addingTimeInterval(TimeInterval.random(in: -fiveDaysInSeconds...0)),
                        description: randomDescription(),
                        content: a % 2 == 0 ? randomDescription() : nil,
                        searchOrder: query?.contains("\(a)") == true || "artikel".contains((query ?? "-").lowercased()) ? Int.random(in: 1...20) : nil,
                        sections: (2...Int.random(in: 2...10))
                            .map({ s in
                                ArticleSection(
                                    id: s,
                                    title: "Sektion \(s)",
                                    content: randomDescription(),
                                    searchOrder: query?.contains("\(s)") == true || "sektion".contains((query ?? "-").lowercased()) ? Int.random(in: 1...20) : nil
                                )
                            })
                    )
                })
            )
        })
    }


    public init() { }

    public func fetchArticleCategories() -> AnyPublisher<[CMSData.Category], ServiceError> {
        MockPublisher.mockFailable(
            value: categories(),
            error: ServiceError.networkError(.missingInternetConnection)
        )
    }
    
    public func search(query: String) -> AnyPublisher<[SearchResultDomain], ServiceError> {
        MockPublisher.mockFailable(
            value: [],
            error: ServiceError.networkError(.missingInternetConnection)
        )
    }
    
    public func fetchArticleCategories(for week: Int) -> AnyPublisher<CMSData.RecommendedCategories, ServiceError> {
        let category = categories(query: "sektion").first!
        let article = category.articles.first!
        let sections = Array(article.sections.filter({ $0.searchOrder != nil }).prefix(2))
        let recommendedCategories = CMSData.RecommendedCategories(
            current: [CMSData.Category(
                id: category.id,
                name: category.name,
                slug: "mock",
                description: category.description,
                iconUrl: category.iconUrl,
                articles: [
                    Article(
                        id: article.id,
                        title: article.title,
                        lastUpdated: article.lastUpdated,
                        description: article.description,
                        content: article.content,
                        searchOrder: article.searchOrder,
                        sections: sections
                    )
                ])],
                next: [CMSData.Category(
                    id: category.id,
                    name: category.name,
                    slug: "mock",
                    description: category.description,
                    iconUrl: category.iconUrl,
                    articles: [
                        Article(
                            id: article.id,
                            title: article.title,
                            lastUpdated: article.lastUpdated,
                            description: article.description,
                            content: article.content,
                            searchOrder: article.searchOrder,
                            sections: sections
                        )
                    ])])
        return MockPublisher.mockFailable(
            value: recommendedCategories,
            error: ServiceError.networkError(.missingInternetConnection)
        )
    }

    public func fetchAfterBirthArticleCategories(for weeksAfterBirth: Int) -> AnyPublisher<CMSData.RecommendedCategories, ServiceError> {
        fetchArticleCategories(for: 1)
    }
    
    public func fetchLinkedArticle(for id: Int) -> AnyPublisher<CMSData.LinkedArticle, ErrorData.ServiceError> {
        let category = categories(query: "sektion").first!
        let article = category.articles.first!
        let sections = Array(article.sections.filter({ $0.searchOrder != nil }).prefix(2))
        
        let linkedArticle = LinkedArticle(
            sectionId: 850,
            category: "uge for uge",
            article: Article(
                id: article.id,
                title: article.title,
                lastUpdated: article.lastUpdated,
                description: article.description,
                content: article.content,
                searchOrder: article.searchOrder,
                sections: sections)
        )
        
        return MockPublisher.mockFailable(
            value: linkedArticle,
            error: ServiceError.networkError(.missingInternetConnection)
        )    }
}
