// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import CMSData
import ErrorData
import DependencyInjection
import HTTPNetworking
import Logger

public final class DictionaryServiceImpl : DictionaryService {

    @Inject private var networkSession: NetworkSession
    @Inject private var apiConfiguration: CMSApiConfiguration
    @Inject private var logger: LoggerProtocol

    private let cacheTimeout: TimeInterval
    private var cachedDictionary: CMSDictionary = .empty()
    private var previousFetchDate: Date? = nil

    /// Used if fetch is called multiple times before the first one has returned.
    /// This will make sure that the call is only performed once.
    private var loadingPublisher: AnyPublisher<CMSDictionary, ServiceError>?

    /// Defaults to 6 hours as cacheTimeout
    /// - Parameter cacheTimeout: Duration in seconds before the cache times out and requires new fetch of dictionary
    public init(cacheTimeout: TimeInterval = 3600 * 6) {
        self.cacheTimeout = cacheTimeout
    }
    
    public func fetchDictionary() -> AnyPublisher<CMSDictionary, ServiceError> {
        fetchOrUseCacheIfAvailable()
    }

    /// Lets dictionary become 6 hours old before refetching (only in memory - a fresh app launch will always fetch data)
    private var shouldUseCache: Bool {
        let useCache: Bool
        if let previousFetchDate = previousFetchDate {
            useCache = Date().timeIntervalSince(previousFetchDate) < cacheTimeout
        } else {
            // No cache available, we have never fetched before
            useCache = false
        }
        return useCache
    }

    /// Uses cache if available and not outdated
    private func fetchOrUseCacheIfAvailable() -> AnyPublisher<CMSDictionary, ServiceError> {
        if let loadingPublisher = loadingPublisher {
            logger.debug("DictionaryService is already loading - returning working publisher...")
            return loadingPublisher
        }
        else if shouldUseCache {
            logger.debug("Returning cached version of dictionary.")
            return Just(self.cachedDictionary)
                .setFailureType(to: ServiceError.self)
                .eraseToAnyPublisher()
        } else {
            logger.debug("Fetching new version of dictionary...")
            let request = DictionaryRequest(apiConfiguration: self.apiConfiguration)
            let publisher = self.networkSession.dataTaskPublisher(for: request)
                .map { CMSDictionary(items: $0.data) }
                .handleEvents(receiveOutput: { (dic: CMSDictionary) in
                    self.logger.debug("Fetched new dictionary.")
                    self.cachedDictionary = dic
                    self.previousFetchDate = Date()
                },
                receiveCompletion: { _  in
                    self.loadingPublisher = nil
                })
                .mapToServiceError()
                .eraseToAnyPublisher()
            self.loadingPublisher = publisher
            return publisher
        }
    }
}
