// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import ErrorData
import HTTPNetworking
import DependencyInjection
import SharedServices
import GMAppData

final public class ConfigurationServiceImpl: ConfigurationService {
    @Inject private var tokenService: TokenService
    @Inject private var networkSession: NetworkSession
    @Inject private var apiConfiguration: ConfigurationApiConfiguration

    public init() { }

    public func fetchChecklists(maternityId: String) -> AnyPublisher<[String], ServiceError> {
        ConfigurationServiceMock().fetchChecklists(maternityId: "")
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    public func updateChecklistItem(maternityId: String, itemId: String, isChecked: Bool) -> AnyPublisher<Void, ServiceError> {
        tokenService.accessToken()
            .flatMap { (accessToken) -> AnyPublisher<Void, Error> in
                do {
                    let request = try ToggleChecklistsItemRequest(
                        apiConfiguration: self.apiConfiguration,
                        itemId: itemId,
                        value: isChecked,
                        accessToken: accessToken,
                        maternityId: maternityId
                    )
                    return self.networkSession.dataTaskPublisher(for: request)
                        .mapToVoid()
                        .eraseToAnyPublisher()
                }
                catch {
                    return Fail(error: ServiceError.failedToParseJson)
                        .eraseToAnyPublisher()
                }
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    public func dateOfBirth(maternityId: String) -> AnyPublisher<DateOfBirth, ServiceError> {
        tokenService.accessToken()
            .flatMap { (accessToken) -> AnyPublisher<DateOfBirth, Error> in
                let request = DateOfBirthRequest(
                    apiConfiguration: self.apiConfiguration,
                    accessToken: accessToken,
                    maternityId: maternityId
                )
                return self.networkSession.dataTaskPublisher(for: request)
                    .map(\.data)
                    .eraseToAnyPublisher()
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    public func updateDateOfBirth(maternityId: String, date: Date?) -> AnyPublisher<DateOfBirth, ServiceError> {
        tokenService.accessToken()
            .flatMap { (accessToken) -> AnyPublisher<DateOfBirth, Error> in
                do {
                    let request = try UpdateDateOfBirthRequest(
                        apiConfiguration: self.apiConfiguration,
                        accessToken: accessToken,
                        maternityId: maternityId,
                        date: date
                    )
                    return self.networkSession.dataTaskPublisher(for: request)
                        .map(\.data)
                        .eraseToAnyPublisher()
                }
                catch {
                    return Fail(error: ServiceError.failedToParseJson)
                        .eraseToAnyPublisher()
                }
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }

    public func deleteAllData() -> AnyPublisher<Void, ServiceError> {
        tokenService.accessToken()
            .flatMap { (accessToken) -> AnyPublisher<Void, Error> in
                let request = DeleteAllRequest(apiConfiguration: self.apiConfiguration, accessToken: accessToken)
                return self.networkSession.dataTaskPublisher(for: request)
                    .mapToVoid()
                    .eraseToAnyPublisher()
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
}
