// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import HTTPNetworking
import DependencyInjection

private struct HTTPRequestPlatformHelper: CustomStringConvertible {
    @Inject private var systemInformationService: SystemInformationService

    var description: String {
        "\(systemInformationService.osName ?? "iOS")_\(systemInformationService.appVersion ?? "0.0.0")(\(systemInformationService.appBuildVersion ?? "0000"))"
    }
}

public extension HTTPRequest {
    /// Returns a dictionary with an `Authorization` as key and Bearer token.
    /// - Parameter accessToken: The access token. *OBS* Don't add `Bearer`
    /// - Returns: Returns a dictionary with a `Authorization` as key and `Bearer <access token>` as value
    static func bearerToken(_ accessToken: String) -> [String: String] { ["Authorization": "Bearer \(accessToken)"] }

    /// Returns a dictionary with a `Content-Type` as key and `application/json` as value
    static func jsonContentType() -> [String: String] { ["Content-Type": "application/json; charset=UTF-8"] }

    /// Returns a dictionary with a `Content-Type` as key and `application/json` as value
    static func platform() -> [String: String] { [Constants.RequestHeaderKeys.platform: HTTPRequestPlatformHelper().description ] }

    /// Combines default headers to one header-dictionary.
    /// Bearer token is only set if not nil
    static func defaultHeaders(accessToken: String? = nil) -> [String: String] {
        var headers: [String: String] = [:]
        if let at = accessToken {
            headers.merge(bearerToken(at), uniquingKeysWith: { current, _ in current })
        }
        headers.merge(jsonContentType(), uniquingKeysWith: { current, _ in current })
        headers.merge(platform(), uniquingKeysWith: { current, _ in current })
        return headers
    }

    /// Encodes object to JSON with ISO8601 date format
    static func encode<T: Encodable>(_ encodable: T) throws -> Data? {
        let jsonEncoder: JSONEncoder = .iso8601()
        return try jsonEncoder.encode(encodable)
    }
}
