// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import HTTPNetworking
import ErrorData
import GMAppData
import DependencyInjection
import Logger

public final class ResponseParser {
    @Inject private static var logger: LoggerProtocol

    static private let jsonDecoder: JSONDecoder = .iso8601()

    public static func parseAndHandle<OutputType: Decodable>(input: Data, urlResponse: HTTPURLResponse?) throws -> OutputType {
        let result: OutputType

        do {
            try validate(input, urlResponse: urlResponse)
            result = try jsonDecoder.decode(OutputType.self, from: input)
        } catch let decodingError as DecodingError {
            logger.debug("Failed to parse result: \(decodingError)")
            throw ServiceError.failedToParseJson
        } catch let validationError {
            throw validationError
        }

        return result
    }

    public static func validate(_ input: Data, urlResponse: HTTPURLResponse?) throws {
        switch urlResponse?.statusCode {
        case .some(200..<400): break // All good 👍🏻
        case .some(400..<600):
            do {
                let error = try jsonDecoder.decode(ProblemDetails.self, from: input)
                let bodyString = String(data: input, encoding: .utf8) ?? "-"
                logger.warning("\(urlResponse?.url?.path ?? "Request") failed:\n\(bodyString)")
                throw ServiceError.backendError(error)
            } catch let decodingError as DecodingError {
                logger.warning("Failed to parse result: \(decodingError)")
                throw ServiceError.failedToParseJson
            }
        default: // This is not good! 🤯
            throw ServiceError.unexpectedError(nil)
        }
    }
}
