// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import XCTest
import DependencyInjection
import Logger
import CMSData
import HTTPMockHelpers
@testable import HTTPNetworking
@testable import CMSServices

final class DictionaryServiceImplTests: CMSURLSessionMockTestCase {
    private let dictionaryService = DictionaryServiceImpl(cacheTimeout: 1.0)
    private var dictionaryUrl: URL {
        URL(string: "\(Self.config.cmsBaseUrl)/dictionary")!
    }

    func testDictionaryCaching() {
        let responseData = [ DictionaryItem(title: "Test", text: "Test test") ]
        URLSessionStubResults.resultsForUrls[dictionaryUrl] = .dataResponse(
            data: try! JSONEncoder.iso8601().encode(responseData),
            response: URLSessionStubResponses.response(url: dictionaryUrl)!
        )

        fetchDictionaryAndAssert { (dictionary) in
            XCTAssertTrue(dictionary.hasText(for: "Test"))
        }

        // Test that we get a cached dictionary
        let responseData2 = [ DictionaryItem(title: "Test2", text: "Test test") ]
        URLSessionStubResults.resultsForUrls[dictionaryUrl] = .dataResponse(
            data: try! JSONEncoder.iso8601().encode(responseData2),
            response: URLSessionStubResponses.response(url: dictionaryUrl)!
        )

        fetchDictionaryAndAssert { (dictionary) in
            XCTAssertTrue(dictionary.hasText(for: "Test"))
            XCTAssertFalse(dictionary.hasText(for: "Test2"))
        }

        // Wait for 1 second and request again - the cached should have been invalidated now
        _ = XCTWaiter.wait(for: [XCTestExpectation(description: "Never returns 😉")], timeout: 1.0) // wait for one second!
        fetchDictionaryAndAssert { (dictionary) in
            XCTAssertTrue(dictionary.hasText(for: "Test2"))
            XCTAssertFalse(dictionary.hasText(for: "Test"))
        }
    }

    // MARK: -
    private func fetchDictionaryAndAssert(_ performAssert: @escaping (CMSDictionary) -> Void) {
        let expect = XCTestExpectation(description: "Should ahve returned a dictionary!")
        dictionaryService.fetchDictionary()
            .sink(
                receiveCompletion: { self.handleSubscriberCompletion($0, expectation: expect) },
                receiveValue: performAssert
            )
            .store(in: &cancelBag)
        wait(for: [expect], timeout: 1.0)
    }
}

