// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import XCTest
import DependencyInjection
import Logger
import CMSData
import HTTPMockHelpers
import ErrorData
import SharedServices
@testable import HTTPNetworking
@testable import CMSServices

final class PageServiceImplTests: CMSURLSessionMockTestCase {
    private let pageService = PagesServiceImpl()
    private static var pageUrl: URL {
        URL(string: "\(Self.config.cmsBaseUrl)/page")!
    }

    override class func setUp() {
        super.setUp()

        for page in PageType.allCases {
            let responseData = Page(title: page.path.capitalized, content: "Content", excerpt: nil, slug: nil)
            let pageSubUrl = pageUrl.appendingPathComponent(page.path)
            URLSessionStubResults.resultsForUrls[pageSubUrl] = .dataResponse(
                data: try! JSONEncoder.iso8601().encode(responseData),
                response: URLSessionStubResponses.response(url: pageSubUrl)!
            )
        }
    }

    func testAboutTheApp() {
        fetchPage(publisher: pageService.fetchAboutTheApp(), assertType: .aboutTheApp)
    }

    func testBehindTheApp() {
        fetchPage(publisher: pageService.fetchBehindTheApp(), assertType: .behindTheApp)
    }

    func testConsent() {
        fetchPage(publisher: pageService.fetchConsent(), assertType: .consent)
    }

    func testMasterData() {
        fetchPage(publisher: pageService.fetchMasterData(), assertType: .masterData)
    }

    func testMyLocking() {
        fetchPage(publisher: pageService.fetchMyLocking(), assertType: .myLocking)
    }

    func testMyLog() {
        fetchPage(publisher: pageService.fetchMyLog(), assertType: .myLog)
    }

    func testPrivacyPolicy() {
        fetchPage(publisher: pageService.fetchPrivacyPolicy(), assertType: .privacyPolicy)
    }

    func testSupport() {
        fetchPage(publisher: pageService.fetchSupport(), assertType: .support)
    }

    // MARK: -
    private func fetchPage(publisher: AnyPublisher<Page, ServiceError>, assertType pageType: PageType) {
        let expect = XCTestExpectation(description: "Should have returned \(pageType.path) page!")
        publisher
            .sink(
                receiveCompletion: { self.handleSubscriberCompletion($0, expectation: expect) },
                receiveValue: { page in
                    XCTAssertEqual(page.title, pageType.path.capitalized)
                }
            )
            .store(in: &cancelBag)
        wait(for: [expect], timeout: 1.0)
    }
}

