#!/bin/bash

generation_folder="./OpenAPIGeneration"
models_base_folder="./MinGraviditet/MinGraviditet/Data/Models/Dto"

function generate {
    set -e
    openapi-generator generate -i "$1" -g swift6 -o "$generation_folder/$2" --skip-validate-spec --global-property models --skip-overwrite
}

function copyToSourceLocation {
    mkdir -p "$models_base_folder"
    cp -a "$generation_folder/generated_models/Sources/OpenAPIClient/Models/." "$models_base_folder"
    enforceOnlyCodableAndHashable # Ensure only Codable, Hashable
}

function enforceOnlyCodableAndHashable {
    echo "🔧 Ensuring only Codable & Hashable in generated models..."
    find "$models_base_folder" -type f -name "*.swift" -exec sed -i '' -E 's/(struct|class) ([a-zA-Z0-9_]+): .*/\1 \2: Codable, Hashable {/' {} +
}

function preCleanup {
    rm -rf "$models_base_folder/" # remove old source files
    postCleanup # clean up generation folder after process
}

function postCleanup {
    rm -rf "$generation_folder" # clean up generation folder
}

function checkIfToolsAreInstalled {
    local failed=false
    if ! [ -x "$(command -v openapi-generator)" ]; then
        echo '🛑  openapi-generator is not installed! ➡️ Run `brew install openapi-generator`' >&2
        failed=true
    fi

    if [ "$failed" = true ]; then
        exit 1
    fi
}

echo "🔧 Checking if required tools are installed on this machine ..."
checkIfToolsAreInstalled

read -r server_version<"./Tools/models-version.config"
echo "🚀 Starting generation of OpenAPI models (version $server_version)..."

declare -a Urls=(
    "./tools/gravid.yml"
)

preCleanup # Remove any old models
for i in "${Urls[@]}"; do
    set -- $i
    path="$1"

    echo "🔨 Generating files for path: $path"
    generate "$path" "generated_models"
    copyToSourceLocation
    echo "🏁 Finished generation for path: $path"
done
postCleanup

echo -n $server_version > "./Tools/models-version.config.resolved"
echo "✅ All done, all generated files were moved to: $models_base_folder"
